// DGuidedMeasurePipeline.cpp : implementation file
//

#include "stdafx.h"
#include "AICONMoveInspectPipelineApp.h"
#include "DMoveInspectPipelineDlg.h"
#include "DGuidedMeasurePipeline.h"


// DGuidedMeasurePipeline dialog

IMPLEMENT_DYNAMIC(DGuidedMeasurePipeline, CDialog)

DGuidedMeasurePipeline::DGuidedMeasurePipeline(CWnd* pParent /*=NULL*/)
	: CDialog(DGuidedMeasurePipeline::IDD, pParent)
	, m_groupTargetInfo(_T(""))
	, m_deltaInfo(_T(""))
	, m_deltaMagInfo(_T(""))
{
	DMoveInspectPipelineDlg* pDlg = dynamic_cast<DMoveInspectPipelineDlg*>(theApp.m_pMainWnd);
	if (pDlg != nullptr)
	{
		pDlg->SetGuidePointIndex(0);
	}
}

DGuidedMeasurePipeline::~DGuidedMeasurePipeline()
{
}

void DGuidedMeasurePipeline::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_GROUP_TARGET_STATIC, m_groupTargetInfo);
	DDX_Text(pDX, IDC_DELTA_STATIC, m_deltaInfo);
	DDX_Text(pDX, IDC_DELTA_MAG_STATIC, m_deltaMagInfo);
	DDX_Control(pDX, IDC_GROUP_TARGET_STATIC, m_grpTargCtrl);
}


BEGIN_MESSAGE_MAP(DGuidedMeasurePipeline, CDialog)
	ON_WM_DESTROY()
	ON_WM_SHOWWINDOW()
	ON_BN_CLICKED(IDCANCEL, &DGuidedMeasurePipeline::OnBnClickedCancel)
	ON_BN_CLICKED(IDC_DELETE_LAST_POINT_BUTTON, &DGuidedMeasurePipeline::OnBnClickedDeleteLastPoint)
END_MESSAGE_MAP()


// DGuidedMeasurePipeline message handlers

BOOL DGuidedMeasurePipeline::OnInitDialog()
{
	CDialog::OnInitDialog();

	UpdateFields();	

	// Need to update the main interface with the initial target info..
	CString collectionName, groupName, targetName;
	if (GetCurrentTarget(collectionName, groupName, targetName))
	{
		DMoveInspectPipelineDlg* pDlg = dynamic_cast<DMoveInspectPipelineDlg*>(theApp.m_pMainWnd);
		if (pDlg)
		{
			pDlg->SetObservationNames(collectionName, groupName, targetName);
		}
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void DGuidedMeasurePipeline::OnDestroy()
{
	//PersistentWnd::SaveWndRect(_T("windows"),_T("SA_MOVEINSPECT_PIPELINE_DGuideMeas"),this);

	CDialog::OnDestroy();
}

void DGuidedMeasurePipeline::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow, nStatus);

	if (bShow)
	{
		//PersistentWnd::RestoreWndRect(_T("windows"),_T("SA_MOVEINSPECT_PIPELINE_DGuideMeas"),this);	
	}
}

void DGuidedMeasurePipeline::OnBnClickedCancel()
{
	OnCancel();
}

void DGuidedMeasurePipeline::SetGTNames()
{
	DMoveInspectPipelineDlg* pDlg = dynamic_cast<DMoveInspectPipelineDlg*>(theApp.m_pMainWnd);
	if (pDlg != nullptr)
	{
		int idx = pDlg->GetGuidePointIndex();

		CString collection, group, target, grpSuffix, tgtSuffix;
		Vector pt;
		pDlg->GetGuideInfo(idx, pt, collection, group, target, grpSuffix, tgtSuffix);

		m_groupTargetInfo.Format(_T("%s::%s"),
			group + grpSuffix,
			target + tgtSuffix);
		m_grpTargCtrl.SetWindowText(m_groupTargetInfo);
	}
}


void DGuidedMeasurePipeline::UpdateFields()
{
	DMoveInspectPipelineDlg* pDlg = dynamic_cast<DMoveInspectPipelineDlg*>(theApp.m_pMainWnd);
	if (pDlg != nullptr)
	{
		int idx = pDlg->GetGuidePointIndex();
		pDlg->ServoView(idx);

		SetGTNames();

		Vector DeltaInWork;
		double DeltaMag;
		pDlg->GetGuideDeltaInWorking(idx, m_curXYZ, DeltaInWork, DeltaMag);

		////////////////////////////////////////////////////////////////
        // Convert deltas to display units
		auto displayUnits = pDlg->GetDisplayUnits();
		// Values stored in inches...
		double tmp = pDlg->ConvertLengthToMeters(DMoveInspectPipelineDlg::LU_INCHES, DeltaInWork.x);
		DeltaInWork.x = pDlg->ConvertLengthFromMeters(displayUnits, tmp);
		tmp = pDlg->ConvertLengthToMeters(DMoveInspectPipelineDlg::LU_INCHES, DeltaInWork.y);
		DeltaInWork.y = pDlg->ConvertLengthFromMeters(displayUnits, tmp);
		tmp = pDlg->ConvertLengthToMeters(DMoveInspectPipelineDlg::LU_INCHES, DeltaInWork.z);
		DeltaInWork.z = pDlg->ConvertLengthFromMeters(displayUnits, tmp);
		////////////////////////////////////////////////////////////////

		m_deltaInfo.Format(_T("%8.*f %8.*f %8.*f"),
		    4, DeltaInWork.x,
			4, DeltaInWork.y,
			4, DeltaInWork.z);
		m_deltaMagInfo.Format(_T("Mag.= %8.*f"), 4, DeltaInWork.Length());

		UpdateData(FALSE);
	}
}

bool DGuidedMeasurePipeline::GetCurrentTarget(CString& col, CString& grp, CString& targ)
{
	bool bStatus = false;

	DMoveInspectPipelineDlg* pDlg = dynamic_cast<DMoveInspectPipelineDlg*>(theApp.m_pMainWnd);
	if (pDlg != nullptr)
	{
		int idx = pDlg->GetGuidePointIndex();

		CString collection, group, target, grpSuffix, tgtSuffix;
		Vector nominal;
		pDlg->GetGuideInfo(idx, nominal, collection, group, target, grpSuffix, tgtSuffix);
		col = collection;
		grp = group;
		if (!grpSuffix.IsEmpty())
		{
			grp += grpSuffix;
		}
		targ = target;
		if (!tgtSuffix.IsEmpty())
		{
			targ += tgtSuffix;
		}
		bStatus = true;
	}

	return bStatus;
}

BOOL DGuidedMeasurePipeline::ProcessNewData(const Vector& newpt, const int& inc)
{
	m_curXYZ = newpt;

	DMoveInspectPipelineDlg* pDlg = dynamic_cast<DMoveInspectPipelineDlg*>(theApp.m_pMainWnd);
	if (pDlg != nullptr)
	{
		if (pDlg->AdvanceGuide(inc))
		{
			CDialog::OnOK();
			return TRUE;
		}
	}

	UpdateFields();
	return TRUE;
}

void DGuidedMeasurePipeline::OnBnClickedDeleteLastPoint()
{
	ProcessNewData(m_curXYZ, -1);
}
