// DMoveInspectPipelineDlg.cpp : implementation file
//

#include "stdafx.h"
#include "AICONMoveInspectApp.h"
#include "DMoveInspectPipelineDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#include "DGuidedMeasurePipeline.h"

static double DTR = acos(-1.0) / 180.0;

const unsigned short VALID_UNCERT_PARAM = 0x01;
const unsigned short VALID_PROBEDIR_PARAM = 0x02;

void AiconReceiverEventCallback(int eventCode, void* pUserData);

IMPLEMENT_DYNCREATE(AiconCamera, CObject);

// DMoveInspectPipelineDlg dialog

DMoveInspectPipelineDlg::DMoveInspectPipelineDlg(CWnd* pParent /*=NULL*/)
	: CDialog(DMoveInspectPipelineDlg::IDD, pParent)
	, m_nIDConnection(0)
	, m_nIDData(0)
	, m_bConnection(false)
	, m_version(_T(""))
	, m_collectionName(_T(""))
	, m_groupName(_T("Main"))
	, m_targetName(_T("P0"))
	, m_actionTypeMode(0)
	, m_lastX(_T("0.00"))
	, m_lastY(_T("0.00"))
	, m_lastZ(_T("0.00"))
	, m_lastRadius(_T("0.00"))
	, m_bStorePointsAlways(FALSE)
    , m_bStorePointsTriggered(TRUE)
	, m_lastProbeDiameterInMM(0.0)
	, m_bAutoGroupIncrement(FALSE)
    , m_sReferenceName(_T(""))
    , m_bCoded(TRUE)
    , m_bNonCoded(TRUE)
    , m_bAdapterFrames(TRUE)
    , m_bAdapterPoints(TRUE)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_pGuidedMeasDlg = NULL;
	m_bIgnoreAllPoints = false;
	m_pvCameras = NULL;
	m_numCameras = 0;
	m_bSnapShot = FALSE;
    m_bSingleMeasureButtonPressed = false;

	m_bAutoProxActive = false;
	m_bGuidedMeasActive = false;
	m_bCaptureNextGuidedMeasure = false;

	m_displayLengthUnits = LU_INCHES;
}

DMoveInspectPipelineDlg::~DMoveInspectPipelineDlg()
{
	ClearMeasurementModes();

	if (m_pvCameras)
	{
		delete[] m_pvCameras;
	}
	m_lastCameras.DeleteAll();
}

void DMoveInspectPipelineDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_MSGEDIT, m_MessageCtrl);
    //DDX_Control(pDX, IDC_DOCKING_BUTTON, m_dockingBtnCtrl);
    DDX_Text(pDX, IDC_VERSION_STATIC, m_version);
	DDX_Text(pDX, IDC_COLLECTION_COMBO, m_collectionName);
	DDX_Text(pDX, IDC_GROUP_COMBO, m_groupName);
    DDX_Text(pDX, IDC_TARGET_EDIT, m_targetName);
    DDX_Control(pDX, IDC_COLLECTION_COMBO, m_collectionComboCtrl);
    DDX_Control(pDX, IDC_GROUP_COMBO, m_groupComboCtrl);
    DDX_Control(pDX, IDC_TARGET_EDIT, m_targetEditCtrl);
    DDX_Radio(pDX, IDC_PROBING_RADIO, m_actionTypeMode);
    DDX_Text(pDX, IDC_LAST_X_STATIC, m_lastX);
    DDX_Text(pDX, IDC_LAST_Y_STATIC, m_lastY);
    DDX_Text(pDX, IDC_LAST_Z_STATIC, m_lastZ);
    DDX_Text(pDX, IDC_LAST_RADIUS_STATIC, m_lastRadius);
    DDX_Control(pDX, IDC_REF_COMBO, m_referenceComboBox);
    DDX_Text(pDX, IDC_REFNAME_EDIT, m_sReferenceName);
    DDX_Text(pDX, IDC_REF_TARGETS_STATIC, m_sRefTargets);
    DDX_Text(pDX, IDC_REF_SIGMA_STATIC, m_sRefSigma0);
    DDX_Text(pDX, IDC_REF_VOLUME_STATIC, m_sRefVolume);

	DDX_Control(pDX, IDC_SAINSTPIPELINECTRL1, m_pipelineSDK);
}

BEGIN_MESSAGE_MAP(DMoveInspectPipelineDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
	ON_EN_SETFOCUS(IDC_MSGEDIT, OnSetfocusErroredit)
	ON_BN_CLICKED(IDC_STOP_SERVER_BUTTON, &DMoveInspectPipelineDlg::OnBnClickedStopServer)
	ON_MESSAGE(AFTER_INIT, OnAfterInit)	
	ON_WM_DESTROY()
	ON_WM_SHOWWINDOW()
	ON_MESSAGE(WM_SETOBSERVATIONNAMES,		OnSetObservationNamesMessage)
	ON_MESSAGE(WM_GUIDEDMODESTATECHANGE,	OnSetGuidedModeStateChangeMessage)
    ON_MESSAGE(WM_CREATEDYNAMICREFERENCE,		OnCreateDynamicReferenceMessage)
    ON_MESSAGE(WM_DYNAMICREFLISTREBUILD,		OnRebuildDynamicRefListMessage)
    ON_MESSAGE(WM_DYNAMICREFCHANGED,		OnDynamicRefChangedMessage)
	ON_BN_CLICKED(IDC_SINGLE_BUTTON, &DMoveInspectPipelineDlg::OnBnClickedSingleMeasure)
	ON_BN_CLICKED(IDC_CONTINUOUS_ON_BUTTON, &DMoveInspectPipelineDlg::OnBnClickedContinuousMeasureON)
    ON_BN_CLICKED(IDC_CONTINUOUS_OFF_BUTTON, &DMoveInspectPipelineDlg::OnBnClickedContinuousMeasureOFF)
	ON_CBN_SELCHANGE(IDC_REF_COMBO,		&DMoveInspectPipelineDlg::OnCbnSelchangeReferenceCombo)
	ON_BN_CLICKED(IDC_OPTIONS_BUTTON, &DMoveInspectPipelineDlg::OnBnClickedOptions)
    ON_BN_CLICKED(IDC_PROBING_RADIO, &DMoveInspectPipelineDlg::OnBnClickedProbingRadio)
    ON_BN_CLICKED(IDC_TRACKING_RADIO, &DMoveInspectPipelineDlg::OnBnClickedTrackingRadio)
	ON_BN_CLICKED(IDC_CONNECT_BUTTON, &DMoveInspectPipelineDlg::OnBnClickedConnect)
    ON_BN_CLICKED(IDC_CREATE_REF_BUTTON, &DMoveInspectPipelineDlg::OnBnClickedCreateRef)
END_MESSAGE_MAP()

BEGIN_EVENTSINK_MAP(DMoveInspectPipelineDlg, CDialog)
	// Event handler invoked once connected to Spatial Analyzer
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x64, DMoveInspectPipelineDlg::Pipeline_OnConnectToHardware, VTS_BSTR)
	// Event handler invoked to notify instrument to begin live updates
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x6c, DMoveInspectPipelineDlg::Pipeline_OnActivateWatchUpdating, VTS_NONE)
	// Event handler invoked when the length/temp units within Spatial Analyzer have changed
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x6b, DMoveInspectPipelineDlg::Pipeline_OnUnitsChange, VTS_I4 VTS_I4)
	// Event handler invoked via MP Instrument OpCheck.
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x66, DMoveInspectPipelineDlg::Pipeline_OnOpCheck, VTS_BSTR)
	// Event handler invoked via MP when the collection / group / target names have changed.
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x65, DMoveInspectPipelineDlg::Pipeline_OnSetColGroupTarget, VTS_BSTR VTS_BSTR VTS_BSTR)
	// Dynamic Reference Event Handlers
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x67, DMoveInspectPipelineDlg::Pipeline_OnSAIPRequestPoints, VTS_BSTR)
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x68, DMoveInspectPipelineDlg::Pipeline_OnSAIPCreateDynamicReference, VTS_BSTR VTS_BSTR)
	// Auto Proximity Event Handler
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x6a, DMoveInspectPipelineDlg::Pipeline_OnSAIPAutoMeasProximity, VTS_BOOL)
	// Guided Measure Event Handler
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x69, DMoveInspectPipelineDlg::Pipeline_OnSAIPMeasGroup, VTS_BSTR)
	// Transform Updates
	ON_EVENT(DMoveInspectPipelineDlg, IDC_SAINSTPIPELINECTRL1, 0x6d, DMoveInspectPipelineDlg::Pipeline_OnInstToWorkingTransformChanged, VTS_BSTR VTS_BSTR VTS_R8)
END_EVENTSINK_MAP()

// DMoveInspectPipelineDlg message handlers

BOOL DMoveInspectPipelineDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	ConnectionSetupCompleteEvent = CreateEvent(NULL,TRUE,FALSE,NULL);

	// TODO: Not presently supported by Pipeline
	//DockWrapper& dockWrapper = theApp.GetSAIB()->GetDockWrapper();
	//dockWrapper.SetDockingBtnCtrl(m_dockingBtnCtrl);
	//dockWrapper.CheckDockability(*theApp.GetSAIB());

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	m_collectionComboCtrl.EnableWindow(FALSE);
	m_groupComboCtrl.EnableWindow(FALSE);
	GetDlgItem(IDC_TARGET_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_CONNECT_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_STOP_SERVER_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_SINGLE_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_CONTINUOUS_ON_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_CONTINUOUS_OFF_BUTTON)->EnableWindow(FALSE);
    GetDlgItem(IDC_PROBING_RADIO)->EnableWindow(FALSE);
    GetDlgItem(IDC_TRACKING_RADIO)->EnableWindow(FALSE);
    GetDlgItem(IDC_REFNAME_EDIT)->EnableWindow(FALSE);
    GetDlgItem(IDC_CREATE_REF_BUTTON)->EnableWindow(FALSE);
    m_referenceComboBox.EnableWindow(FALSE);

	//GetDlgItem(IDC_LAST_X_STATIC)->EnableWindow(FALSE);
	//GetDlgItem(IDC_LAST_Y_STATIC)->EnableWindow(FALSE);
	//GetDlgItem(IDC_LAST_Z_STATIC)->EnableWindow(FALSE);
	//GetDlgItem(IDC_LAST_RADIUS_STATIC)->EnableWindow(FALSE);
	//GetDlgItem(IDC_XUNITS_STATIC)->EnableWindow(FALSE);
	//GetDlgItem(IDC_YUNITS_STATIC)->EnableWindow(FALSE);
	//GetDlgItem(IDC_ZUNITS_STATIC)->EnableWindow(FALSE);
	//GetDlgItem(IDC_RADIUS_UNITS_STATIC)->EnableWindow(FALSE);

    GetDlgItem(IDC_XUNITS_STATIC)->SetWindowText(_T(""));
    GetDlgItem(IDC_YUNITS_STATIC)->SetWindowText(_T(""));
    GetDlgItem(IDC_ZUNITS_STATIC)->SetWindowText(_T(""));
    GetDlgItem(IDC_RADIUS_UNITS_STATIC)->SetWindowText(_T(""));
    GetDlgItem(IDC_REF_SIGMA_UNITS_STATIC)->SetWindowText(_T(""));

	PostMessage(AFTER_INIT,0,0);

	m_version = theApp.GetVersion();

	// TODO: Not available in Pipeline
	// Retrieve any previously stored IDs
	//theApp.m_SAConnection.GetTargetIDs(m_codedTargetIDs);

	UpdateData(FALSE);

    //DialogHelper::MakeAllGroupBoxesBold(this);

	if (m_pipelineSDK.PopLogonDialog() == FALSE)
	{
		PostQuitMessage(-999);
	}
	else
	{
		m_pipelineSDK.AboutBox();
	}

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void DMoveInspectPipelineDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR DMoveInspectPipelineDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void DMoveInspectPipelineDlg::OnSetfocusErroredit() 
{
	m_MessageCtrl.ShowWindow(SW_HIDE);
}

LRESULT DMoveInspectPipelineDlg::OnAfterInit(WPARAM a, LPARAM b)
{   
	UINT uRet = gReceiver_Init(_T("SA MoveInspect Interface"), NULL);
	//----------------------- not successful?
	if(uRet != rcSuccess)
	{
		//------------------- show the message
		CString msg;
		msg.Format(_T("%s: %d"), _T("MoveInspect Init Failed"), uRet);
		PostError(msg);
		//------------------- quit the dialog
		//EndDialog(IDCANCEL);
	}
	else
	{
		// Register callback so we will be notified when data arrives...
		// NOTE: Toooo slow... reverted back to polling!  Ugh!
		// Art 12/12/2016 - Added establishing this callback again so that events for
		// current measurement parameters (eventMeasureParamChanged) can be monitored
		// for setting references.
		uRet = gReceiver_SetEventCallback(AiconReceiverEventCallback, (void*)this);

        GetDlgItem(IDC_CONNECT_BUTTON)->EnableWindow(TRUE);
		PostAlert(_T("No device connected"), RGB(0,0,0), RGB(255,255,0));
	}

	return 0;
}

void DMoveInspectPipelineDlg::PostError(const CString& msg)
{
	PostAlert(msg, RGB(0,0,0), RGB(255,0,0));
}

void DMoveInspectPipelineDlg::PostAlert(const CString& msg, const COLORREF& txtColor, const COLORREF& bkgndColor)
{
	//m_MessageCtrl.SetBkColor(bkgndColor);
	//m_MessageCtrl.SetTextColor(txtColor);
	m_MessageCtrl.SetWindowText(msg);
	m_MessageCtrl.ShowWindow(SW_SHOW);
}

void DMoveInspectPipelineDlg::ClearAlert(void)
{
	if (m_MessageCtrl.IsWindowVisible())
	{
		m_MessageCtrl.SetWindowText(_T(""));
		m_MessageCtrl.ShowWindow(SW_HIDE);
	}
}

BOOL DMoveInspectPipelineDlg::StartServer()
{
	BOOL bSuccess = FALSE;

	// init the receiver
	UINT uRet = gReceiver_StartServer(0x100);
	// not successful?
	if(uRet != rcSuccess && uRet != rcServerAlreadyRunning)
	{
		CString msg;
		msg.Format(_T("%s: %d"), _T("Failed to Start Server"), uRet);
		PostError(msg);
	}
	else
	{
		if (m_pvCameras)
		{
			delete[] m_pvCameras;
			m_pvCameras = NULL;
			m_numCameras = 0;
		}
		m_lastCameras.DeleteAll();

        // set buttons state
		GetDlgItem(IDC_STOP_SERVER_BUTTON)->EnableWindow(TRUE);

		ClearAlert();

        // Added this as when debugging from desktop and immediately doing a connect
        //  and looking for AICON data source would always return zero
        //Delay(0.25);

        if (m_nIDConnection)
        {
            KillTimer(m_nIDConnection);
        }
        // start "connection checker"-timer
        m_nIDConnection = SetTimer(12345, 1000, NULL);

		bSuccess = TRUE;
	}

	return bSuccess;
}

void DMoveInspectPipelineDlg::OnBnClickedStopServer()
{
	StopServer();
}

BOOL DMoveInspectPipelineDlg::StopServer()
{
	BOOL bSuccess = FALSE;

	// stop the receiver
	UINT uRet = gReceiver_StopServer();
	// not successful?
	if(uRet != rcSuccess)
	{
		CString msg;
		msg.Format(_T("%s: %d"), _T("Failed to Stop Server"), uRet);
		PostError(msg);
	}
	else
	{
		// no more timer
		KillTimer(m_nIDConnection);
		m_nIDConnection = 0;

        m_bConnection = false;

		// reset buttons
        GetDlgItem(IDC_CONNECT_BUTTON)->EnableWindow(TRUE);
		GetDlgItem(IDC_STOP_SERVER_BUTTON)->EnableWindow( FALSE);

		m_collectionComboCtrl.EnableWindow(FALSE);
		m_groupComboCtrl.EnableWindow(FALSE);
		GetDlgItem(IDC_TARGET_EDIT)->EnableWindow(FALSE);
		GetDlgItem(IDC_SINGLE_BUTTON)->EnableWindow(FALSE);
        GetDlgItem(IDC_CONTINUOUS_ON_BUTTON)->EnableWindow(FALSE);
        GetDlgItem(IDC_CONTINUOUS_OFF_BUTTON)->EnableWindow(FALSE);
        GetDlgItem(IDC_PROBING_RADIO)->EnableWindow(FALSE);
        GetDlgItem(IDC_TRACKING_RADIO)->EnableWindow(FALSE);
        GetDlgItem(IDC_REFNAME_EDIT)->EnableWindow(FALSE);
        GetDlgItem(IDC_CREATE_REF_BUTTON)->EnableWindow(FALSE);

        m_referenceComboBox.ResetContent();
        m_referenceComboBox.EnableWindow(FALSE);

		PostAlert(_T("No device connected"), RGB(0,0,0), RGB(255,255,0));

		bSuccess = TRUE;
	}

	return bSuccess;
}

bool DMoveInspectPipelineDlg::AwaitConnectionSetupEvent(DWORD dwMilliseconds)
{
    bool bStatus = false;
	//bool bSignaled = WaitForSingleObject(ConnectionSetupCompleteEvent,dwMilliseconds) == WAIT_OBJECT_0 ? true : false;
	//return bSignaled;

    HANDLE hTimer = CreateWaitableTimer(NULL, TRUE, NULL);

    if (hTimer != NULL)
    {
        LARGE_INTEGER liDueTime;

        LONGLONG delay = (LONGLONG) (dwMilliseconds * 10000000LL);
        // the time is in 100 ns (10**-7 sec) units 
        liDueTime.QuadPart = -delay;

        DWORD dwWakeMask = QS_ALLINPUT;

        SetWaitableTimer(hTimer, &liDueTime, 0, NULL, NULL, 0);

        // Setup the handles to be monitored.
        HANDLE	handles[2];
        handles[0] = hTimer;
        handles[1] = ConnectionSetupCompleteEvent;

        bool bDone = false;
        while (!bDone)
        {
            // One danger of the MsgWaitForMultipleObjects function is calling it when there are 
            //  already messages waiting to be processed, because MsgWaitForMultipleObjects returns 
            //  only when there is a new event in the queue.
            // NOTE: I switched to MsgWaitForMultipleObjectsEx to accommodate this situation.
            DWORD dwRet = MsgWaitForMultipleObjectsEx(2, handles, INFINITE, dwWakeMask, MWMO_INPUTAVAILABLE);

            if ((dwRet == WAIT_OBJECT_0) ||   // Timer event was signaled
                (dwRet == WAIT_OBJECT_0 + 1)) // Event parameter was signaled
            {
                bDone = true;
                bStatus = true;
                continue; // The event or timer was signaled
            }
            else 
            {
                MSG msg;

                // There is one or more window message available. Dispatch them
                while(PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE))
                {
                    TranslateMessage(&msg);
                    DispatchMessage(&msg);
                    if (WaitForSingleObject(hTimer, 0) == WAIT_OBJECT_0)
                    {
                        bDone = true; // Event is now signaled.
                        break;
                    }
                }
            }
        }

        CloseHandle(hTimer);
    }

    return bStatus;
}

bool DMoveInspectPipelineDlg::CheckConnection()
{
	// test connection
	UINT uRet = gReceiver_IsConnected();
	// got new connection
	if( ( uRet == rcSuccess ) && ( m_bConnection == false ) )
	{
		// we are connected
		m_bConnection = true;

		m_collectionComboCtrl.EnableWindow(TRUE);
		m_groupComboCtrl.EnableWindow(TRUE);
		GetDlgItem(IDC_TARGET_EDIT)->EnableWindow(TRUE);
		GetDlgItem(IDC_SINGLE_BUTTON)->EnableWindow(TRUE);
        GetDlgItem(IDC_CONTINUOUS_ON_BUTTON)->EnableWindow(TRUE);
        GetDlgItem(IDC_CONTINUOUS_OFF_BUTTON)->EnableWindow(TRUE);
        GetDlgItem(IDC_PROBING_RADIO)->EnableWindow(TRUE);
        GetDlgItem(IDC_TRACKING_RADIO)->EnableWindow(TRUE);
        GetDlgItem(IDC_REFNAME_EDIT)->EnableWindow(TRUE);
        GetDlgItem(IDC_CREATE_REF_BUTTON)->EnableWindow(TRUE);
        m_referenceComboBox.ResetContent();
        m_referenceComboBox.EnableWindow(TRUE);

		// get references
		GetReferences();

		// get measure modes
		if (GetMeasureModes())
		{
            SetProbingMeasurementMode();
		}

		uRet = gReceiver_RetrieveCameraInformation(true);

		// name buffer
		TCHAR cName[512];
		// get source name
		uRet = gReceiver_GetDataSourceName(cName, 512);
		CString msg;
		if( uRet == rcSuccess )
		{
			// reset status
			msg.Format( _T("Connected to \"%s\""), cName);
		}
		else
		{
			// set status
			msg = _T("Connected!");
		}
		PostAlert(msg, RGB(0,0,0), RGB(0,255,0));

		// Simple event used for MP's to better sync with connections
		SetEvent(ConnectionSetupCompleteEvent);

		// start "data polling/acquisition"-timer
		m_nIDData = SetTimer(54321, 100, NULL);
	}
	// lost old connection
	else if( ( uRet == rcNotConnected ) && ( m_bConnection == true ) )
	{
		if( m_nIDData != 0 )
		{
			KillTimer(m_nIDData);
			m_nIDData = 0;
		}

		// we are not connected
		m_bConnection = false;

		m_collectionComboCtrl.EnableWindow(FALSE);
		m_groupComboCtrl.EnableWindow(FALSE);
		GetDlgItem(IDC_TARGET_EDIT)->EnableWindow(FALSE);
		GetDlgItem(IDC_SINGLE_BUTTON)->EnableWindow(FALSE);
        GetDlgItem(IDC_CONTINUOUS_ON_BUTTON)->EnableWindow(FALSE);
        GetDlgItem(IDC_CONTINUOUS_OFF_BUTTON)->EnableWindow(FALSE);
        GetDlgItem(IDC_PROBING_RADIO)->EnableWindow(FALSE);
        GetDlgItem(IDC_TRACKING_RADIO)->EnableWindow(FALSE);
        GetDlgItem(IDC_REFNAME_EDIT)->EnableWindow(FALSE);
        GetDlgItem(IDC_CREATE_REF_BUTTON)->EnableWindow(FALSE);
        m_referenceComboBox.ResetContent();
        m_referenceComboBox.EnableWindow(FALSE);

		ResetEvent(ConnectionSetupCompleteEvent);

		// reset status
		PostAlert(_T("No device connected"), RGB(0,0,0), RGB(255,255,0));
	}

	return m_bConnection;
}

void AiconReceiverEventCallback(int eventCode, void* pUserData)
{
	DMoveInspectPipelineDlg* pDlg = (DMoveInspectPipelineDlg*) pUserData;
	if (pDlg)
	{
#if 0
		if (pDlg->IsConnected())
		{
			pDlg->ProcessData();
		}
#endif
		pDlg->OnReceiverEvent(eventCode);
	}
}

void DMoveInspectPipelineDlg::OnReceiverEvent(int iEvent)
{
    TRACE1("* * * * * * OnReceiverEvent: %d\n", iEvent);
	switch (iEvent)
	{
	case eventDataReceived:
		// get available data
		//GetData();
		break;
	case eventCalibResultReceived:
		// get available data
		//GetCalibData();
		break;
	case eventConnectionStarted:
		break;
	case eventConnectionEnded:
		break;
	case eventLiveMeasurementStarted:
		break;
	case eventLiveMeasurementStopped:
		break;
	case eventMeasureModeChanged:
		break;
	case eventMeasureParamChanged:
        PostMessage(DMoveInspectPipelineDlg::WM_DYNAMICREFCHANGED, 0, 0);
		break;
	case eventReferenceListChanged:
        PostMessage(DMoveInspectPipelineDlg::WM_DYNAMICREFLISTREBUILD, 0, 0);
		break;
#if 0
	case eventMIProjectStateChanged:
		{
			int iState = gReceiver_MI_GetProjectState();
			switch (iState)
			{
			case mov_Prj_NotInitialized:
				break;
			case mov_Prj_Initialized:
				break;
			case mov_Prj_Started:
				break;
			case mov_Prj_Stopped:
				break;
			case mov_Prj_Cancelled:
				break;
			case mov_Prj_CalculationFinished:
				break;
			}
		}
		break;
#endif
	}
}

void DMoveInspectPipelineDlg::GetReferences()
{
    TCHAR sName[256];

	m_referenceComboBox.ResetContent();
	m_referenceComboBox.SetItemData( m_referenceComboBox.AddString(_T("None")), -1 );

	// how many modes do we have
	size_t uAmount = gReceiver_GetReferenceNameCount();
	if(uAmount > 0 )
	{
		GetDlgItem(IDC_REF_COMBO)->EnableWindow( TRUE );

		// for each ref
		for( size_t iIndex = 0; iIndex < uAmount; iIndex++ )
		{
			// get mode info
			UINT uRet = gReceiver_GetReferenceName((UINT)iIndex, sName, 256);
			if( uRet == rcSuccess )
			{
                CString refName(sName);
				// save ref name
				int iPos = m_referenceComboBox.AddString( refName );
				m_referenceComboBox.SetItemData( iPos, iIndex );
                TRACE1("Added Ref Name: %s\n", refName);
			}
			else
			{
                CString sMessage;
                sMessage.Format(_T("gReceiver_GetReferenceName failed! -> %i"), uRet);
                AfxMessageBox(sMessage);
			}
		}
	}

    SendMessage(DMoveInspectPipelineDlg::WM_DYNAMICREFCHANGED, 0, 0);
}

/// <summary>
/// Classes below MUST be used for communicating trans/track point information
///  to Spatial Analyzer via JSON.
/// </summary>
class SAPoint
{
public:
	SAPoint()
	{

	}

	std::string PointName;

	double x;
	double y;
	double z;
};

static void to_json(nlohmann::json& j, const SAPoint& s)
{
	j["PointName"] = s.PointName;
	j["x"] = s.x;
	j["y"] = s.y;
	j["z"] = s.z;
}

class SATrackingPointSet
{
public:
	SATrackingPointSet()
	{

	}
	std::vector<SAPoint> Pts;
};

static void to_json(nlohmann::json& j, const SATrackingPointSet& s)
{
	j["Pts"] = s.Pts;
}

struct SACheckPointForInstrument
{
	bool DidFindPoint;
	std::string TargetNameToSend;
	double DistanceToClosest;
	double ToleranceForClosest;
};

static void from_json(const nlohmann::json& j, SACheckPointForInstrument& s)
{
	s.DidFindPoint = j.at("DidFindPoint").get<bool>();
	s.TargetNameToSend = j.at("TargetNameToSend").get<std::string>();
	s.DistanceToClosest = j.at("DistanceToClosest").get<double>();
	s.ToleranceForClosest = j.at("ToleranceForClosest").get<double>();
}

struct SAAutoMeasureTolerances
{
	double proximityTolerance;
	double axisProximityTolerance;
	double toneRampStartTolerance;
};

static void from_json(const nlohmann::json& j, SAAutoMeasureTolerances& s)
{
	s.proximityTolerance = j.at("proximityTolerance").get<double>();
	s.axisProximityTolerance = j.at("axisProximityTolerance").get<double>();
	s.toneRampStartTolerance = j.at("toneRampStartTolerance").get<double>();
}

struct SAAutoMeasureDestination
{
	std::string collection;
	std::string group;
	std::string target;
};

static void from_json(const nlohmann::json& j, SAAutoMeasureDestination& s)
{
	s.collection = j.at("Collection").get<std::string>();
	s.group = j.at("Group").get<std::string>();
	s.target = j.at("Target").get<std::string>();
}

// Method invoked to process incoming data from MoveInspect...
//  Done via polling from OnTimer vs OnReceiverEvent.  OnReceiverEvent
//  method used by Aicon SDK too slow to handle Proximity data processing
bool DMoveInspectPipelineDlg::GetData()
{
	bool bGotData = false;

	// do we have any data?
	UINT uRet = gReceiver_HasDataReceived();

	// got new data
	if( uRet == rcSuccess )
	{
		// data buffer
		tData cData;

		ZeroMemory(&cData, sizeof(tData));

		UINT uNumCameras = gReceiver_GetDataCameraCnt();
		if( uNumCameras > 0 )
		{
			// Number of cameras has changed...
			if ((m_numCameras != 0) && (uNumCameras != m_numCameras))
			{
				delete[] m_pvCameras;
				m_pvCameras = NULL;
				m_lastCameras.DeleteAll();
			}
			// Allocate Camera data storage when needed..
			if (m_pvCameras == NULL)
			{
				m_pvCameras = new tCamera[uNumCameras];
				m_numCameras = uNumCameras;
			}
			else
			{
				ZeroMemory(m_pvCameras, sizeof(tCamera)*m_numCameras);
			}
			cData.pvCameras = m_pvCameras;
			cData.uCameraCapacity = uNumCameras;
		}

		// get amount of 6DOF data
		UINT uNumObjs = gReceiver_GetData6DoFCnt();
		if( uNumObjs > 0)
		{
			cData.pv6DoF = new t6DoF[uNumObjs];
			cData.u6DoFCapacity = uNumObjs;
			SecureZeroMemory( cData.pv6DoF, uNumObjs * sizeof(t6DoF) );
		}

		// get amount of points
		UINT uPointAmount = gReceiver_GetDataPointCnt();

		// if any point necessary
		if( uPointAmount > 0 )
		{
			cData.pvPoints = new tPoint[uPointAmount];
			cData.uPointsCapacity = uPointAmount;
		}

		// test connection
		UINT uRet = gReceiver_GetData( cData );

        if (uRet != rcSuccess)
        {
			// TODO: Not available in Pipeline
            //theApp.m_SAConnection.ProcessCompleted(MPResult::DoneFatalError);
            return false;
        }

		// NOTE: cmdDeleteLastPoint is sent whenever the user presses right thumb button on probe.
		if( cData.iCommand == cmdButton1 )
		{
            OnBnClickedDeleteLastPoint();
		}
        // NOTE: cmdEndSession is sent whenever the user presses the left thumb button on probe.
		else if( cData.iCommand == cmdButton2 )
		{
			m_pipelineSDK.SendInspectionNavigationToSA(TRUE);
		}
		else
		{
            if ( cData.iCommand == errMeasurementFailed )
            {
				// TODO: Not available in Pipeline
                //theApp.m_SAConnection.ProcessCompleted(MPResult::DoneFatalError);
                return false;
            }

            UpdateReferenceDetails(&cData);

			// If a 6DoF object is present, send its info as an update 
			//  unless the active mode is to send frames, then create them in SA
			if (uNumObjs && cData.pv6DoF)
			{
                BOOL bUpdateOnly = TRUE;
                // Whenever adapter requested (snapshotted or not)
                if (m_actionTypeMode == TRACKING && m_bAdapterFrames)
                {
                    bUpdateOnly = FALSE;
                }
				for( UINT uIndex = 0; uIndex < uNumObjs; uIndex++)
				{
					t6DoF& curItem = cData.pv6DoF[uIndex];
					if (curItem.bLocationValid && curItem.bRotationValid)
					{
						CString name(curItem.cName);
						if (m_bSnapShot)
						{
							//CString tag = StringHelper::GetStringFromCTime(CTime::GetCurrentTime());
							CString tag = CTime::GetCurrentTime().Format(_T("%c")); // Local time
							CString newName;
							newName.Format(_T("%s ( %s )"), name, tag);
							name = newName;
						}
						Vector posInMM(curItem.dX, curItem.dY, curItem.dZ);
						Vector rotInRadians(curItem.dO, curItem.dP, curItem.dK);
						ProcessFrame(name, posInMM, rotInRadians, bUpdateOnly);
					}
					
					if( curItem.bQualityValuesValid )
					{
						CString buf;
						buf.Format( _T("... Quality: %11.4lf %11.6lf %11.4lf %6i"), 
							curItem.dVolumePercent,		
							curItem.dSigma0,
							curItem.dRayPercent,
							curItem.iRayCount);	
						//AfxMessageBox(buf);
					}
				}
			}

            if (!m_bIgnoreAllPoints)
            {
                CString sBuffer;

                // prepare output
                //sBuffer.Format(_T("DiameterTip: %lf"), cData.dDiameterTip);

                // prepare output
                //sBuffer.Format(_T("DiameterShank: %lf"), cData.dDiameterShank);

                bool bProbingModeActive = IsProbingModeActive();

                double	currentProbeDiameterInMM = cData.dDiameterTip;
                if (cData.dDiameterTip > 0.0)
                {
                    m_lastProbeDiameterInMM = cData.dDiameterTip;
                }
                else
                {
                    if (m_lastProbeDiameterInMM > 0.0)
                    {
                        // Use the last cached tip diameter value
                        currentProbeDiameterInMM = m_lastProbeDiameterInMM;
                    }
                }
                double currentProbeDiameterInches = currentProbeDiameterInMM;
                currentProbeDiameterInches = ConvertLengthToMeters(LU_MILLIMETERS, currentProbeDiameterInches);
                currentProbeDiameterInches = ConvertLengthFromMeters(LU_INCHES, currentProbeDiameterInches);

                // Switching from probing to targeting, make sure to not send over 
                //  target data with prior probe diameter information
                if (!bProbingModeActive && (currentProbeDiameterInches != 0.0))
                {
                    currentProbeDiameterInches = 0.0;
                }

                // Need to scan all points for probe to see if triggered prior to processing...
                // Needed for tracking since probe could be anywhere within received batch of points...
                bool bProbeTriggered = m_bSingleMeasureButtonPressed;
                for( UINT uIndex = 0; !bProbeTriggered && uIndex < uPointAmount; uIndex++)
                {
                    CString name(cData.pvPoints[uIndex].cName);
                    if (name.Trim().Find(_T("(triggered)")) != -1)
                    {
                        bProbeTriggered = true;
                    }
                }
                m_bSingleMeasureButtonPressed = false;

				SATrackingPointSet pointSet;
                // for each point
                for( UINT uIndex = 0; uIndex < uPointAmount; uIndex++)
                {
                    Vector posInMM(cData.pvPoints[uIndex].dX, cData.pvPoints[uIndex].dY, cData.pvPoints[uIndex].dZ);

                    Vector deviationInMM(cData.pvPoints[uIndex].dSx, cData.pvPoints[uIndex].dSy, cData.pvPoints[uIndex].dSz);
                    Vector* pUncertInMM = cData.pvPoints[uIndex].bDeviationValid ? &deviationInMM : NULL;
                    Vector deviationInches = deviationInMM;
                    ConvertVectorToInches(LU_MILLIMETERS, deviationInches);
                    Vector* pUncertInches = cData.pvPoints[uIndex].bDeviationValid ? &deviationInches : NULL;

                    Vector normal(cData.pvPoints[uIndex].dNx, cData.pvPoints[uIndex].dNy, cData.pvPoints[uIndex].dNz);
                    // Scott request... to reverse the normal... getting tip direction and really want probing direction
                    normal.Reverse();
                    Vector* pNormal = cData.pvPoints[uIndex].bNormalValid ? &normal : NULL;
                    int pointID = cData.pvPoints[uIndex].iNum;
#ifdef _DEBUG
                    // prepare output
                    sBuffer.Format(_T("%i \"%s\" %lf %lf %lf %lf %lf %lf %i"), 
                        cData.pvPoints[uIndex].iNum,
                        cData.pvPoints[uIndex].cName,
                        cData.pvPoints[uIndex].dX,
                        cData.pvPoints[uIndex].dY,
                        cData.pvPoints[uIndex].dZ,
                        cData.pvPoints[uIndex].dNx,
                        cData.pvPoints[uIndex].dNy,
                        cData.pvPoints[uIndex].dNz,
                        cData.pvPoints[uIndex].bTriggered ? 1 : 0);

                    TRACE(sBuffer);
                    TRACE("\n");
#endif

                    CString name(cData.pvPoints[uIndex].cName);

                    if (m_actionTypeMode == TRACKING &&
                        !IsAutoProxActive() && !IsGuidedMeasureActive())
                    {
                        // Is this a point that we're interested in?  
                        bool bIgnore = true;
                        bool bCodedFiltering = m_codedTargetIDs.GetSize() ? true : false;
                        if (m_bCoded)
                        {
                            for (INT_PTR i = 0; i < m_codedTargetIDs.GetSize(); i++)
                            {
                                if (m_codedTargetIDs[i] == pointID)
                                {
                                    bIgnore = false;
                                    break;
                                }
                            }   
                        }

                        bool bCanBuffer = false;
                        CString tID;
                        // If target has name, use that...
                        if (StringHelper::IsBlank(name))
                        {
                            tID.Format(_T("%d"), pointID);

                            // XIT-1119 - request to filter by point type (code / non-coded / adapter)
							// XIT-2108 - Uncoded point "1000" not sent to SA from Aicon Move Inspect
                            if ((m_bCoded && pointID < 1000 && !bCodedFiltering) ||
                                (m_bNonCoded && pointID >= 1000))
                            {
                                bIgnore = false;
                                // Buffering support for coded / noncoded...
                                bCanBuffer = true;
                            }
                        }
                        else
                        {
                            tID = name;
                            // XIT-1119 - request to filter by point type (code / non-coded / adapter)
                            // NOTE: Adapters have a valid name... coded/non-coded do not.
                            if (m_bAdapterPoints)
                            {
                                bIgnore = false;
                            }
                        }

                        if (name.TrimLeft().Left(5).CompareNoCase(_T("Probe")) == 0) 
                        {
                            tID = _T("Probe");
                            // Always update dialog with last probe point received information...
                            UpdateDialogInfo(posInMM, currentProbeDiameterInMM);
                            bIgnore = false; // So as to make sure it is sent and not ignored.
                        }

                        // If coded target list specified, use that to filter what gets sent to SA.
                        // If nothing to be filtered, send everything
                        if (!bIgnore)
                        {
                            Vector posInches(posInMM);
                            ConvertVectorToInches(LU_MILLIMETERS, posInches);

							unsigned short flags = 0;
							Vector uncert;
							if (pUncertInches != nullptr)
							{
								uncert = *pUncertInches;
								flags |= VALID_UNCERT_PARAM;
							}
							Vector normal;
							if (pNormal != nullptr)
							{
								normal = *pNormal;
								flags |= VALID_PROBEDIR_PARAM;
							}

                            if (m_bStorePointsAlways || (m_bStorePointsTriggered && bProbeTriggered))
                            {
                                // Only coded / uncoded will be buffered...
                                if (bCanBuffer)
                                {
									m_pipelineSDK.AddNamedMeasurement(tID, posInches.x, posInches.y, posInches.z, uncert.x, uncert.y, uncert.z, flags);
                                }
                                else
                                {
                                    // Make sure to flush anything prior...
                                    FlushBufferedMeasurements();
                                    //////////////////////////////////////////////////////////////////////
                                    // Send over points to SA within the current group.
                                    //////////////////////////////////////////////////////////////////////
									m_pipelineSDK.SendSinglePoint(m_collectionName, m_groupName, tID, posInches.x, posInches.y, posInches.z,
										uncert.x, uncert.y, uncert.z, normal.x, normal.y, normal.z, 0.0, 0.0, flags);
                                }
                            }
                            else
                            {
                                //////////////////////////////////////////////////////////////////////
                                // Scott request to send points over as an update when not storing...
                                //////////////////////////////////////////////////////////////////////
								m_pipelineSDK.UpdateSinglePoint(posInches.x, posInches.y, posInches.z, uncert.x, uncert.y, uncert.z, normal.x, normal.y, normal.z,
									currentProbeDiameterInches / 2.0, currentProbeDiameterInches / 2.0, flags);
                            }

							auto pt = *new SAPoint();
							pt.PointName = CW2A(tID.GetString());
							pt.x = posInches.x;
							pt.y = posInches.y;
							pt.z = posInches.z;
							pointSet.Pts.push_back(pt);
                        }
                    }
                    else
                    {
#ifdef false
                        TextFileContainer textFile;
                        CString line;
                        line.Format(_T("PROBING %s: Tip(%f)\tShank(%f)\n"), name, cData.dDiameterTip, cData.dDiameterShank);
                        textFile.AddLine(line);
                        textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
#endif
                        // If we're in guided measurement mode, this signals the
                        //  user request to capture a measurement...
                        if (IsGuidedMeasureActive())
                        {
                            //if ((name.TrimLeft().Left(5).CompareNoCase(_T("Probe")) == 0) && 
                            //	cData.pvPoints[uIndex].bTriggered)
                            // NOTE: Look for "triggered" text to indicate user initiated measurement.
                            //       The .bTriggered value cannot be used as it is always TRUE
                            if (name.Trim().CompareNoCase(_T("Probe (triggered)")) == 0)
                            {
                                m_bCaptureNextGuidedMeasure = true;
                            }
                        }

                        if ((name.TrimLeft().Left(5).CompareNoCase(_T("Probe")) == 0) ||
                            // NOTE: Need a way to capture a single triggered probe point.  Does not come over
                            //       with a name when in probing measurement mode.
                            (bProbingModeActive && name.IsEmpty() && 
                            //cData.pvPoints[uIndex].bTriggered && 
                            !IsAutoProxActive() && !IsGuidedMeasureActive()))
                        {
                            // Got a point for SA
                            if (ProcessPoint(posInMM, pUncertInMM, pNormal, currentProbeDiameterInMM, cData.pvPoints[uIndex].bTriggered))
                            {

                            }
                            // Always update dialog with last probe point received information...
                            UpdateDialogInfo(posInMM, currentProbeDiameterInMM);
                        }
                    }
                }

                if (!IsAutoProxActive() && !IsGuidedMeasureActive())
                {
					// TODO: Not available in Pipeline
                    //theApp.m_SAConnection.ProcessCompleted(MPResult::DoneSuccess);
                }

                if (pointSet.Pts.size())
                {
                    FlushBufferedMeasurements(); // Safety

                    // https://github.com/nlohmann/json
					// create an empty structure (null)
					nlohmann::json j;

					j = pointSet;

					std::string foobar = j.dump();

					CString jsonPointObjects(foobar.c_str());

					// This is called from the instrument to SA to facilitate
					// trans-track with a group of points.  For photogrammetry systems, for example,
					// where you have a set of points you measure over and over.  The first call
					// SA gets when the SA mode is up establishes the nominal starting condition, then it moves from there.
					m_pipelineSDK.SendPointSetTracking(jsonPointObjects);

                    if ((m_bStorePointsAlways || (m_bStorePointsTriggered && bProbeTriggered)) && m_bAutoGroupIncrement)
                    {
                        IncrementGroup(1);
                    }
                }

                bGotData = true;
            }
		}

		// cleanup
		if( cData.pvPoints != NULL )
		{
			delete[] cData.pvPoints;
			cData.pvPoints = NULL;
			cData.uPointsCapacity = 0;
		}
		if( cData.pv6DoF != NULL )
		{
			delete[] cData.pv6DoF;
			cData.pv6DoF = NULL;
			cData.u6DoFCapacity = 0;
		}
		if( cData.pvCameras != NULL )
		{
			// Mantis #7787... always update camera information, thus
			//  never turn it off
			//gReceiver_RetrieveCameraInformation(false);
			// Save out the camera information we captured...
			//m_pvCameras = cData.pvCameras;
			//m_numCameras = cData.uCameraCapacity;
			cData.pvCameras = NULL;
			cData.uCameraCapacity = 0;
#ifdef false
			{
				CString val;
				TextFileContainer textFile;		
				for (int i = 0; i < m_numCameras; i++)
				{
					tCamera& curCamera = m_pvCameras[i];
					val.Format(_T("Camera Number: %d"), curCamera.iNumber);
					textFile.AddLine(val);
					TRACE(val + _T("\n"));
					val.Format(_T("Loc: (%lf, %lf, %lf) mm"), curCamera.dX, curCamera.dY, curCamera.dZ);
					textFile.AddLine(val);
					TRACE(val + _T("\n"));
					val.Format(_T("Rot: (%lf, %lf, %lf) rad"), curCamera.dOm, curCamera.dPh, curCamera.dKa);
					textFile.AddLine(val);
					TRACE(val + _T("\n"));
				}
				textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
			}
#endif
			ProcessCameras();
		}
	}

	return bGotData;
}

bool DMoveInspectPipelineDlg::ProcessFrame(const CString& frameName, const Vector& posInMM, const Vector& rotInRadians, const BOOL& bUpdateOnly)
{
	bool bSuccess = false;

	CString name(frameName);
	Vector posInches(posInMM);
	ConvertVectorToInches(LU_MILLIMETERS, posInches);
	Vector rot(rotInRadians);
	// Radians to degrees
	rot *= 1.0/DTR;
	//Transform trans;
	//trans.SetZYXRotation(rot);
	//trans.SetTranslation(posInches);
	auto bStatus = m_pipelineSDK.FrameUpdateToSA(m_collectionName, name, posInches.x, posInches.y, posInches.z, rot.x, rot.y, rot.z, bUpdateOnly);
	bSuccess = bStatus ? true : false;

	return bSuccess;
}

void DMoveInspectPipelineDlg::OnTimer(UINT_PTR nIDEvent)
{
	// connection timer
	if( nIDEvent == m_nIDConnection )
	{
		// no more timer
		KillTimer(m_nIDConnection);
		m_nIDConnection = 0;

		// check connection state
		CheckConnection();

		// re-start "connection checker"-timer
		m_nIDConnection = SetTimer(12345, 250, NULL);
	}
	else if( nIDEvent == m_nIDData )
	{
		// no more timer
		KillTimer(m_nIDData);
		m_nIDData = 0;

		// get available data
		GetData();

		// re-start the "data polling/acquisition"-timer
		m_nIDData = SetTimer(54321, 100, NULL);
	}
	else if ( nIDEvent == IGNORE_POINTS_TIMER )
	{
		m_bIgnoreAllPoints = false;
	}
	CDialog::OnTimer(nIDEvent);
}

void DMoveInspectPipelineDlg::OnDestroy()
{
	//PersistentWnd::SaveWndRect(_T("windows"),_T("DMoveInspectPipelineDlg"),this,PersistentWnd::POS_ONLY);

    // Attempt to kill any active timers....
    if (m_nIDConnection > 0)
        KillTimer(m_nIDConnection);

    if (m_nIDData > 0)
        KillTimer(m_nIDData);

    if( gReceiver_IsConnected() == rcSuccess )
    {
        gReceiver_StopConnection();
    }

	//----------------------- release the dll
	UINT uRet = gReceiver_Release();
	//----------------------- not successful?
	if(uRet != rcSuccess)
	{
		//------------------- show the message
		AfxMessageBox(_T("Failed to Release!"));
	}	

	if (ConnectionSetupCompleteEvent != NULL)
	{
		CloseHandle(ConnectionSetupCompleteEvent);
		ConnectionSetupCompleteEvent = NULL;
	}

	CDialog::OnDestroy();
}

void DMoveInspectPipelineDlg::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow, nStatus);

	if (bShow)
	{
		//PersistentWnd::RestoreWndRect(_T("windows"),_T("DMoveInspectPipelineDlg"),this,PersistentWnd::POS_ONLY);
	}
}

typedef struct
{
	CString Collection;
	CString Group;
	CString Target;
} PMSetObservationNames;

void DMoveInspectPipelineDlg::SetObservationNames(const CString& collectionName,const CString& groupName,const CString& targetName)
{
	PMSetObservationNames * m = new PMSetObservationNames;
	m->Collection = collectionName;
	m->Group = groupName;
	m->Target = targetName;

	PostMessage(WM_SETOBSERVATIONNAMES,0,(LPARAM)m);
}

LRESULT DMoveInspectPipelineDlg::OnSetObservationNamesMessage(WPARAM wParam, LPARAM lParam)
{
	PMSetObservationNames * m = (PMSetObservationNames*)lParam;

	m_collectionName	= m->Collection;
	m_groupName			= m->Group;
	m_targetName		= m->Target;
	UpdateData(FALSE);

	delete m;

	return 0;
}

LRESULT DMoveInspectPipelineDlg::OnSetGuidedModeStateChangeMessage(WPARAM wParam, LPARAM lParam)
{
	BOOL* bState = (BOOL*)lParam;

	if (*bState == FALSE)
	{
		SetGuidedMeasureMode(false);

		// TODO: Not supported by Pipeline
		//theApp.m_SAConnection.ProcessCompleted(MPResult::DoneSuccess);

		// Set ignore mode timer so that any extra point data
		//  received after the continuous measurement mode deactivation
		//  will be discarded.
		SetIgnorePointsTimer(3000);
	}
	else
	{
		ClearAlert();
		if (m_pGuidedMeasDlg == NULL)
		{
			SetGuidedMeasureMode(true);
			m_pipelineSDK.Remove6DModel();

			SetTargetingMeasurementMode();

            m_actionTypeMode  = PROBING;

			UpdateData(FALSE);

			UINT uRet = gReceiver_SendStartLiveMeasurement();

			// Send button press to begin measuring
			//theApp.SendExposeButtonPress();

			m_pGuidedMeasDlg = new DGuidedMeasurePipeline;
			m_pGuidedMeasDlg->DoModal();

			gReceiver_SendStopLiveMeasurement(true);

			UpdateData(FALSE);

			delete m_pGuidedMeasDlg;
			m_pGuidedMeasDlg = NULL;

			// Stop the SA ServoView mode...
			m_pipelineSDK.EndGuidedPointServoView();

			BOOL& bState = *new BOOL;
			bState = FALSE; // Turn it OFF
			PostMessage(WM_GUIDEDMODESTATECHANGE, 0, (LPARAM) &bState);
		}
	}

	delete bState;

	return 0;
}

void DMoveInspectPipelineDlg::IncrementGroup(const int& inc)
{
	m_groupComboCtrl.GetWindowText(m_groupName);
	StringHelper::IncrementName(m_groupName, inc);
	m_groupComboCtrl.SetWindowText(m_groupName);
	//m_groupComboCtrl.SetFocus();
}
void DMoveInspectPipelineDlg::IncrementTarget(const int& inc)
{
	m_targetEditCtrl.GetWindowText(m_targetName);
	StringHelper::IncrementName(m_targetName, inc);
	m_targetEditCtrl.SetWindowText(m_targetName);
	//m_targetEditCtrl.SetFocus();
}

#define M2CM 100.0
#define M2MM 1000.0
#define M2FT 3.280839895
#define M2IN 39.37007874

double DMoveInspectPipelineDlg::ConvertLengthToMeters(LengthUnitsType fromUnits, double fromValue) const
{
	double result = fromValue;

	double m = 1;

	switch (fromUnits)
	{
	case LU_METERS:
		m = 1;
		break;
	case LU_CENTIMETERS:
		m = 1 / M2CM;
		break;
	case LU_MILLIMETERS:
		m = 1 / M2MM;
		break;
	case LU_FEET:
		m = 1 / M2FT;
		break;
	case LU_INCHES:
		m = 1 / M2IN;
		break;
	}

	result *= m;

	return result;
}

double DMoveInspectPipelineDlg::ConvertLengthFromMeters(LengthUnitsType toUnits, double fromValue) const
{
	double result = fromValue;

	double m = 1;

	switch (toUnits)
	{
	case LU_METERS:
		m = 1;
		break;
	case LU_CENTIMETERS:
		m = M2CM;
		break;
	case LU_MILLIMETERS:
		m = M2MM;
		break;
	case LU_FEET:
		m = M2FT;
		break;
	case LU_INCHES:
		m = M2IN;
		break;
	}

	result *= m;

	return result;
}

CString DMoveInspectPipelineDlg::GetUnitsLabel(LengthUnitsType units) const
{
	CString s;

	switch (units)
	{
	case LU_METERS:
		s = _T("m");
		break;
	case LU_CENTIMETERS:
		s = _T("cm");
		break;
	case LU_MILLIMETERS:
		s = _T("mm");
		break;
	case LU_FEET:
		s = _T("ft");
		break;
	case LU_INCHES:
		s = _T("in");
		break;
	}

	return s;
}

void DMoveInspectPipelineDlg::ConvertVectorToInches(LengthUnitsType current, Vector& v) const
{
	v.x = ConvertLengthToMeters(current, v.x);
	v.x = ConvertLengthFromMeters(LU_INCHES, v.x);
	v.y = ConvertLengthToMeters(current, v.y);
	v.y = ConvertLengthFromMeters(LU_INCHES, v.y);
	v.z = ConvertLengthToMeters(current, v.z);
	v.z = ConvertLengthFromMeters(LU_INCHES, v.z);
}

double DMoveInspectPipelineDlg::GetDisplayValue(const double& valInMM) const
{
	// TODO: Not Presently available in Pipeline
	LengthUnitsType lenUnits = m_displayLengthUnits;
	double tmp = ConvertLengthToMeters(LU_MILLIMETERS, valInMM);
	// Convert to desired display units
	return ConvertLengthFromMeters(lenUnits,tmp);
}

void DMoveInspectPipelineDlg::UpdateDialogInfo(const Vector& posInMM, const double& probeDiameterInMM)
{
	CString txt;
	int decimalPrecision = 4;

#if FALSE
	{
		TextFileContainer textFile;
		textFile.AddLine(_T("BEGIN DMoveInspectPipelineDlg::UpdateDialogInfo"));
		textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
	}
#endif

	LengthUnitsType lenUnits = m_displayLengthUnits;
	CString unit = GetUnitsLabel(lenUnits);

	double dispVal = GetDisplayValue(posInMM.x);
	m_lastX.Format(_T("%.*f"), decimalPrecision, dispVal);
	//GetDlgItem(IDC_LAST_X_STATIC)->SetWindowText(txt);
	dispVal = GetDisplayValue(posInMM.y);
	m_lastY.Format(_T("%.*f"), decimalPrecision, dispVal);
	//GetDlgItem(IDC_LAST_Y_STATIC)->SetWindowText(txt);
	dispVal = GetDisplayValue(posInMM.z);
	m_lastZ.Format(_T("%.*f"), decimalPrecision, dispVal);
	//GetDlgItem(IDC_LAST_Z_STATIC)->SetWindowText(txt);
	dispVal = GetDisplayValue(probeDiameterInMM);
	m_lastRadius.Format(_T("%.*f"), decimalPrecision, dispVal / 2.0);
	//GetDlgItem(IDC_LAST_RADIUS_STATIC)->SetWindowText(txt);

	GetDlgItem(IDC_XUNITS_STATIC)->SetWindowText(unit);
	GetDlgItem(IDC_YUNITS_STATIC)->SetWindowText(unit);
	GetDlgItem(IDC_ZUNITS_STATIC)->SetWindowText(unit);
	GetDlgItem(IDC_RADIUS_UNITS_STATIC)->SetWindowText(unit);

	UpdateData(FALSE);

#if FALSE
	{
		TextFileContainer textFile;
		textFile.AddLine(_T("END DMoveInspectPipelineDlg::UpdateDialogInfo"));
		textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
	}
#endif
}

void DMoveInspectPipelineDlg::UpdateReferenceDetails(const tData* pCurrentDataSet)
{
    int decimalPrecision = 3;

    bool bReferenceQualityValid = pCurrentDataSet ? pCurrentDataSet->bReferenceQualityValid : false;

    if (bReferenceQualityValid)
    {
        LengthUnitsType lenUnits = m_displayLengthUnits;
        CString unit = GetUnitsLabel(lenUnits);

        GetDlgItem(IDC_REF_SIGMA_UNITS_STATIC)->SetWindowText(unit);

        const tData& currentDataSet = *pCurrentDataSet;
        int iTargets = currentDataSet.iReferenceRayCnt;
        iTargets /= m_numCameras;
        m_sRefTargets.Format(_T("%d"), iTargets);
        double dispVal = GetDisplayValue(currentDataSet.dReferenceSigma0);
        m_sRefSigma0.Format(_T("%.*f"), decimalPrecision, dispVal);
        m_sRefVolume.Format(_T("%.*f"), decimalPrecision, currentDataSet.dReferenceRayPercentage);
    }
    else
    {
        m_sRefTargets.Empty();
        m_sRefSigma0.Empty();
        m_sRefVolume.Empty();
        GetDlgItem(IDC_REF_SIGMA_UNITS_STATIC)->SetWindowText(_T(""));
    }

    UpdateData(FALSE);   
}

bool DMoveInspectPipelineDlg::ProcessProbeFrame(const Vector& posInches, const Vector& normal, const double& probeDiameterInches, const BOOL& bUpdateOnly)
{
	Transform trans;
	Vector zaxis(normal);
	Vector yaxis(-1.0, 0.0, 0.0);
	Vector xaxis;
	xaxis.Cross(yaxis, zaxis);
	// Do one more cross product to take the arbitrary x and make it truly orthogonal
	yaxis.Cross(zaxis, xaxis);

	trans.SetAxes(xaxis, yaxis, zaxis);
	trans.Normalize();
	trans.SetTranslation(posInches);

	Vector rot;
	trans.GetRotation(&rot);
	auto bSuccess = m_pipelineSDK.FrameUpdateToSA2(posInches.x, posInches.y, posInches.z, rot.x, rot.y, rot.z, 
		probeDiameterInches / 2.0, probeDiameterInches / 2.0, TRUE);
	return bSuccess ? true : false;
}

bool DMoveInspectPipelineDlg::ProcessPoint(const Vector& posInMM, const Vector* pUncertaintyInMM, const Vector* pProbeDir, const double& probeDiameterInMM, const bool& bTrigger /*= false*/)
{
	BOOL bStatus = FALSE;

#if FALSE
	{
		TextFileContainer textFile;
		textFile.AddLine(_T("BEGIN DMoveInspectPipelineDlg::ProcessPoint"));
		textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
	}
#endif

	UpdateData(TRUE);

	Vector posInches(posInMM);
	ConvertVectorToInches(LU_MILLIMETERS, posInches);

	Vector uncertInches;
	Vector *pUncertInches = NULL;
	if (pUncertaintyInMM)
	{
		uncertInches = *pUncertaintyInMM;
		ConvertVectorToInches(LU_MILLIMETERS, uncertInches);
		pUncertInches = &uncertInches;
	}

	//Vector normInches(normalInMM);
	//ConvertVectorToInches(LU_MILLIMETERS, normInches);

	double probeInInches = probeDiameterInMM;
	probeInInches = ConvertLengthToMeters(LU_MILLIMETERS, probeInInches);
	probeInInches = ConvertLengthFromMeters(LU_INCHES, probeInInches);

	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// During "Probing" measurement mode, await for triggering to send measurement to SA.  If not triggered,
	//  then process as Update to SA.
	if ( m_actionTypeMode == PROBING )
	{
		bool bUpdateOnly = false;
		bool bAutoProxModeActive	= IsAutoProxActive();
		bool bGuidedMeasureActive	= IsGuidedMeasureActive();
		/////////////////////////////////////////////////////////////////////////////////
		// Auto Proximity Special Processing
		if (bAutoProxModeActive)
		{
			CString targetNameToSend;
			bUpdateOnly = true;
			auto jsonReturn = m_pipelineSDK.CheckPointForInstrument(posInches.x, posInches.y, posInches.z, probeInInches / 2.0);

			auto jj = nlohmann::json::parse(std::wstring(jsonReturn));
			auto checkpoint = jj.get<SACheckPointForInstrument>();

			if (checkpoint.DidFindPoint)
			{
				auto closestPtName = checkpoint.TargetNameToSend;
				bUpdateOnly = false;
				m_targetName = closestPtName.c_str();
			}
			m_pipelineSDK.AudibleSetError(checkpoint.DistanceToClosest);
		}
		/////////////////////////////////////////////////////////////////////////////////
		// Guided Measurement Special Processing
		else if (bGuidedMeasureActive)
		{
			bUpdateOnly = true;
			if (m_pGuidedMeasDlg)
			{
				// If the capture flag has been set, this is not an update only!
				bUpdateOnly = !m_bCaptureNextGuidedMeasure;
				if (!bUpdateOnly)
				{
					m_pGuidedMeasDlg->GetCurrentTarget(m_collectionName, m_groupName, m_targetName);
				}
			}
		}
		else
		{
			bUpdateOnly = !bTrigger;
		}

		unsigned short flags = 0;
		Vector uncert;
		if (pUncertInches != nullptr)
		{
			uncert = *pUncertInches;
			flags |= VALID_UNCERT_PARAM;
		}
		Vector normal;
		if (pProbeDir != nullptr)
		{
			normal = *pProbeDir;
			flags |= VALID_PROBEDIR_PARAM;
		}

		if (!bUpdateOnly)
		{
#if FALSE
			if (pUncertaintyInMM)
			{
				TextFileContainer textFile;
				CString line;

				const Vector& u = *pUncertaintyInMM;
				line.Format(_T("%s::%s: Loc(%lf, %lf, %lf)mm U(%lf, %lf %lf)mm\n"), m_groupName, m_targetName, posInMM.x, posInMM.y, posInMM.z, u.x, u.y, u.z);
				textFile.AddLine(line);
				Vector& uin = *pUncertInches;
				line.Format(_T("%s::%s: Loc(%lf, %lf, %lf)in U(%lf, %lf %lf)in\n"), m_groupName, m_targetName, posInches.x, posInches.y, posInches.z, uin.x, uin.y, uin.z);
				textFile.AddLine(line);
				textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
			}
#endif
			bStatus = m_pipelineSDK.SendSinglePoint(m_collectionName, m_groupName, m_targetName, posInches.x, posInches.y, posInches.z,
				uncert.x, uncert.y, uncert.z, normal.x, normal.y, normal.z, probeInInches / 2.0, probeInInches / 2.0, flags);
		}
		bStatus = m_pipelineSDK.UpdateSinglePoint(posInches.x, posInches.y, posInches.z,
			uncert.x, uncert.y, uncert.z, normal.x, normal.y, normal.z, probeInInches / 2.0, probeInInches / 2.0, flags);

		if (pProbeDir)
		{
			ProcessProbeFrame(posInches, *pProbeDir, probeInInches, TRUE);
		}

		// For updates or during guided measurements, don't want to increment name every time through...
		if (!bGuidedMeasureActive && !bUpdateOnly && bStatus)
		{
			IncrementTarget(1);
			MessageBeep((bStatus)? MB_ICONEXCLAMATION : MB_ICONHAND);
		}

		if (bGuidedMeasureActive)
		{
			if (m_pGuidedMeasDlg)
			{
				int inc = bUpdateOnly ? 0 : 1;
				m_pGuidedMeasDlg->ProcessNewData(posInches, inc);
				// Attempt to advance GUI to next set of names...
				if (!bUpdateOnly && m_pGuidedMeasDlg->GetCurrentTarget(m_collectionName, m_groupName, m_targetName))
				{
					MessageBeep((bStatus)? MB_ICONEXCLAMATION : MB_ICONHAND);
					UpdateData(FALSE);
				}
			}
			m_bCaptureNextGuidedMeasure = false;
		}
	}

#if FALSE
	{
	    TextFileContainer textFile;
	    textFile.AddLine(_T("END DMoveInspectPipelineDlg::ProcessPoint"));
	    textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
	}
#endif

	return bStatus ? true : false;
}

void DMoveInspectPipelineDlg::OnBnClickedDeleteLastPoint()
{
	m_pipelineSDK.RemoveLastMeasurementInSA();
	
	// Decrement the target to reflect last point deletion
	IncrementTarget(-1);
}

void DMoveInspectPipelineDlg::OnBnClickedSingleMeasure()
{
    if (StartMeasure(SINGLE))
    {
        m_bSingleMeasureButtonPressed = true;
    }
}

BOOL DMoveInspectPipelineDlg::StartMeasure(const MeasMode& mode)
{
	BOOL bSuccess = FALSE;

	UpdateData(TRUE);
	switch (mode)
	{
	case SINGLE:
		{
			// single measurement
			UINT uRet = gReceiver_SendDoSingleMeasurement();
			// not successful?
			if(uRet != rcSuccess)
			{
				CString msg;
				msg.Format(_T("%s: %d"), _T("SendDoSingleMeasurement failed"), uRet);
				PostError(msg);
				// TODO: Presently not supported by Pipeline
                //theApp.m_SAConnection.ProcessCompleted(MPResult::DoneFatalError);
			}
			else
			{
				bSuccess = TRUE;
			}
		}
		break;
	case CONTINUOUS:
		{
			// start live mode
			UINT uRet = gReceiver_SendStartLiveMeasurement();
			// not successful?
			if(uRet != rcSuccess)
			{
				CString msg;
				msg.Format(_T("%s: %d"), _T("SendStartLiveMeasurement failed"), uRet);
				PostError(msg);
				// TODO: Presently not supported by Pipeline
                //theApp.m_SAConnection.ProcessCompleted(MPResult::DoneFatalError);
			}
			else
			{
				bSuccess = TRUE;
			}
		}
		break;
	}

	return bSuccess;
}

void DMoveInspectPipelineDlg::OnBnClickedContinuousMeasureOFF()
{
    StopMeasure();
}

void DMoveInspectPipelineDlg::OnBnClickedContinuousMeasureON()
{
	StartMeasure(CONTINUOUS);
}

BOOL DMoveInspectPipelineDlg::StopMeasure()
{
	BOOL bSuccess = FALSE;

	// stop live mode
	UINT uRet = gReceiver_SendStopLiveMeasurement(true);
	// not successful?
	if(uRet != rcSuccess)
	{
		CString msg;
		msg.Format(_T("%s: %d"), _T("SendStopLiveMeasurement failed"), uRet);
		PostError(msg);
	}
	else
	{
		bSuccess = TRUE;
	}

	return bSuccess;
}

void DMoveInspectPipelineDlg::ActivateWatchUpdating(void)
{
	// Don't switch to watch updates when in an auto or guided mode!
	if (IsAutoProxActive() || IsGuidedMeasureActive())
	{
		return;
	}

    m_actionTypeMode = PROBING;

	SetProbingMeasurementMode();

	UpdateData(FALSE);

	// Auto Start live updating!
	UINT uRet = gReceiver_SendStartLiveMeasurement();
}

// ***************************************************************************
bool DMoveInspectPipelineDlg::GetMeasureModes()
{
	bool bSuccess = false;

	ClearMeasurementModes();

	// how many modes do we have
	size_t uModes = gReceiver_GetMeasureModeCount();
	if(uModes != 0 )
	{
		int  iModePos = -1;

		// measure mode info structure
		tMeasureModeInfo cInfo;
		// for each measure mode
		for( size_t iIndex = 0; iIndex < uModes; iIndex++ )
		{
			// get mode info
			UINT uRet = gReceiver_GetMeasureModeInfo(256 + iIndex, cInfo);
			// get the info
			if( uRet == rcSuccess )
			{
				tMeasureModeInfo * pMode = new tMeasureModeInfo[ sizeof(tMeasureModeInfo) ];
				CopyMemory( pMode, &cInfo, sizeof(tMeasureModeInfo) );
                // Request to change TARGETING to TRACKING...
                CString name(pMode->cName);
                if (name.CompareNoCase(_T("TARGETING")) == 0)
                {
                    CString foo(_T("Tracking"));
                    wchar_t* buf = foo.GetBuffer();
                    lstrcpyn(pMode->cName, buf, 128);
                    foo.ReleaseBuffer();
                }

				m_measurementModes.Add(pMode);
				bSuccess = true;
			}
		}
	}

	return bSuccess;
}

void DMoveInspectPipelineDlg::ClearMeasurementModes()
{
	for (INT_PTR i = 0; i < m_measurementModes.GetSize(); i++)
	{
		delete [] m_measurementModes[i];
	}
	m_measurementModes.RemoveAll();
}

bool DMoveInspectPipelineDlg::SetProbingMeasurementMode()
{
	bool bSuccess = false;

	UINT mode = 0;
	for (INT_PTR i = 0; i < m_measurementModes.GetSize(); i++)
	{
		tMeasureModeInfo* pInfo = (tMeasureModeInfo*) m_measurementModes[i];
		if (pInfo)
		{
			CString txt(pInfo->cName);
#if 0
			if (txt.MakeUpper().Find(_T("TIP")) != -1)
			{
				// Assumes only a single entry with the word "tip"
				//  "3d tip"
				mode = pInfo->iNum;
				bSuccess = true;
				break;
			}
#endif
			if (txt.MakeUpper().Find(_T("PROBING")) != -1)
			{
				// Assumes only a single entry with the word "Probing"
				mode = pInfo->iNum;
				bSuccess = true;
				break;
			}
		}
	}

	if (bSuccess)
	{
		UINT uRet = gReceiver_ChangeMeasureMode(mode);
		// check return code
		if( uRet != rcSuccess )
		{
			bSuccess = false;
		}
	}

	return bSuccess;
}

bool DMoveInspectPipelineDlg::SetTargetingMeasurementMode()
{
	bool bSuccess = false;

	UINT mode = 0;
	int comboIndex = 0;
	for (INT_PTR i = 0; i < m_measurementModes.GetSize(); i++)
	{
		tMeasureModeInfo* pInfo = (tMeasureModeInfo*) m_measurementModes[i];
		if (pInfo)
		{
			CString txt(pInfo->cName);
            // Request to change TARGETING to TRACKING...
			if (txt.MakeUpper().Find(_T("TRACKING")) != -1)
			{
				// Assumes only a single entry with the word "Targeting"
				mode = pInfo->iNum;
				comboIndex = i;
				bSuccess = true;
				break;
			}
		}
	}

	if (bSuccess)
	{
		UINT uCurMode = gReceiver_GetCurrentMeasureMode();
		// Change Measure Mode if not the current mode...
		if (uCurMode != mode)
		{
			UINT uRet = gReceiver_ChangeMeasureMode(mode);
			// check return code
			if( uRet != rcSuccess )
			{
				bSuccess = false;
			}
			else
			{
				// Targeting mode does not support probe point normals, remove the probe from
				//  display within SA
				m_pipelineSDK.Remove6DModel();
			}
		}
	}

	return bSuccess;
}

bool DMoveInspectPipelineDlg::IsTargetingModeActive() const
{
    // Request to change TARGETING to TRACKING...
	return CheckActiveMode(_T("TRACKING"));
}

bool DMoveInspectPipelineDlg::IsProbingModeActive() const
{
	return CheckActiveMode(_T("PROBING"));
}

bool DMoveInspectPipelineDlg::CheckActiveMode(const CString& mode) const
{
	UINT uCurMode = gReceiver_GetCurrentMeasureMode();

	bool bActive = false;

	CString m(mode);
	for (INT_PTR i = 0; i < m_measurementModes.GetSize(); i++)
	{
		tMeasureModeInfo* pInfo = (tMeasureModeInfo*) m_measurementModes[i];
		if (pInfo)
		{
			CString txt(pInfo->cName);
			if (txt.MakeUpper().Find(m.MakeUpper()) != -1)
			{
				if (uCurMode == pInfo->iNum)
				{
					bActive = true;
				}
				break;
			}
		}
	}

	return bActive;
}


BOOL DMoveInspectPipelineDlg::OnCommand(WPARAM wParam, LPARAM lParam)
{
	// TODO: Presently not supported by Pipeline
	//BOOL result = theApp.m_SAConnection.GetDockWrapper().OnCommand(*this, wParam, lParam);
	BOOL result = FALSE;

	if (!result)
	{
		result = CDialog::OnCommand(wParam, lParam);
	}

	return result;
}

void DMoveInspectPipelineDlg::OnCbnSelchangeReferenceCombo()
{
	// get current selection
	int iSel = m_referenceComboBox.GetCurSel();
	if( iSel != CB_ERR )
	{
		UINT uRet = 0;

        UpdateReferenceDetails(NULL);

		// get selection data
		int iIndex = (int)m_referenceComboBox.GetItemData(iSel);
		if( iIndex >= 0 )
		{
			// name buffer
			TCHAR cName[128];

			// get mode info
			uRet = gReceiver_GetReferenceName(iIndex, cName, 128);
			if( uRet == rcSuccess )
			{
				// activate this reference
				uRet = gReceiver_SetActiveReference( cName );
			}
		}
		else
		{
			// deactivate reference
			uRet = gReceiver_SetActiveReference( _T("") );
		}
	}
}

// Added Method for the ability to set the active reference via an OpCheck
BOOL DMoveInspectPipelineDlg::SetActiveDynamicReference(const CString& refName)
{
    BOOL bSuccess = FALSE;

    CString foo(refName);
    CString trimmedRefName(foo.Trim());
    if (trimmedRefName.IsEmpty() || DoesDynamicReferenceExist(trimmedRefName))
    {
        TCHAR cName[128];
        lstrcpyn(cName, trimmedRefName, 128);

        UINT uRet = gReceiver_SetActiveReference( cName );

        if (uRet == rcSuccess)
        {
            SendMessage(DMoveInspectPipelineDlg::WM_DYNAMICREFCHANGED, 0, 0);
            bSuccess = TRUE;
        }
    }


    return bSuccess;
}

BOOL DMoveInspectPipelineDlg::DoesDynamicReferenceExist(const CString& refName)
{
    BOOL bFound = FALSE;
    TCHAR sName[256];

    // how many modes do we have
    size_t uAmount = gReceiver_GetReferenceNameCount();
    if(uAmount > 0 )
    {
        // for each system ref
        for( size_t iIndex = 0; !bFound && iIndex < uAmount; iIndex++ )
        {
            // get mode info
            UINT uRet = gReceiver_GetReferenceName((UINT)iIndex, sName, 256);
            if( uRet == rcSuccess )
            {
                CString deviceRefName(sName);
                if (refName.CompareNoCase(deviceRefName.Trim()) == 0)
                {
                    bFound = TRUE;
                }
            }
        }
    }

    return bFound;
}

void DMoveInspectPipelineDlg::SetIgnorePointsTimer(const int& timeInMS)
{
	if (!m_bIgnoreAllPoints)
	{
		m_bIgnoreAllPoints = true;
		SetTimer(IGNORE_POINTS_TIMER, timeInMS, NULL);
	}
}

#include "DOptions.h"
void DMoveInspectPipelineDlg::OnBnClickedOptions()
{
	DOptions dlg;
	dlg.m_codedTargetIDs.Append(m_codedTargetIDs);
    dlg.m_bCoded = m_bCoded;
    dlg.m_bNonCoded = m_bNonCoded;
    dlg.m_bAdapterFrames = m_bAdapterFrames;
    dlg.m_bAdapterPoints = m_bAdapterPoints;
    dlg.m_bSnapShot = m_bSnapShot;
    dlg.m_bStorePointsAlways = m_bStorePointsAlways;
    dlg.m_bStorePointsTriggered = m_bStorePointsTriggered;
    dlg.m_bAutoGroupIncrement = m_bAutoGroupIncrement;
	if (dlg.DoModal() == IDOK)
	{
		m_codedTargetIDs.RemoveAll();
		m_codedTargetIDs.Append(dlg.m_codedTargetIDs);
		// TODO: Presently not supported by Pipeline
		//theApp.m_SAConnection.SetTargetIDs(m_codedTargetIDs);
        m_bCoded = dlg.m_bCoded;
        m_bNonCoded = dlg.m_bNonCoded;
        m_bAdapterFrames = dlg.m_bAdapterFrames;
        m_bAdapterPoints = dlg.m_bAdapterPoints;
        m_bSnapShot = dlg.m_bSnapShot;
        m_bStorePointsAlways = dlg.m_bStorePointsAlways;
        m_bStorePointsTriggered = dlg.m_bStorePointsTriggered;
        m_bAutoGroupIncrement = dlg.m_bAutoGroupIncrement;
	}
}

void DMoveInspectPipelineDlg::OnBnClickedProbingRadio()
{
	UpdateData(TRUE);

	SetProbingMeasurementMode();

    m_actionTypeMode = PROBING;
}

void DMoveInspectPipelineDlg::OnBnClickedTrackingRadio()
{
	UpdateData(TRUE);

	SetTargetingMeasurementMode();

    m_actionTypeMode = TRACKING;
}

void DMoveInspectPipelineDlg::OnBnClickedConnect()
{
	Connect();
}

BOOL DMoveInspectPipelineDlg::Connect(const bool& bSilent /*= false*/)
{
	BOOL bSuccess = FALSE;

	if( gReceiver_IsConnected() == rcSuccess)
	{
        return TRUE;
	}

    if (!StartServer())
    {
        return FALSE;
    }
        
	// get number of sources
	size_t uSources = gReceiver_GetDataSourceCount();
    // Added this logic... found when debugging from my desktop this would
    //  return zero sources the first time.  NOTE: Added Delay in StartServer
    //  to help prevent this too.
    if (uSources == 0)
    {
        uSources = gReceiver_GetDataSourceCount();
    }

	// any source?
	if( uSources > 0 )
	{
        tDataSourceInfo* pInfoVec = new tDataSourceInfo[uSources];

		// get infos
		if( gReceiver_GetDataSourceInfoVec(pInfoVec, uSources) == rcSuccess)
		{
			// for each info
			bool bDone = false;
			for( size_t uIndex = 0; !bDone && uIndex < uSources; uIndex++ )
			{
				CString txt(pInfoVec[uIndex].cName);
				if ((txt.MakeUpper().Find(_T("MOVEINSPECT")) != -1)
#ifdef _DEBUG
					|| (txt.MakeUpper().Find(_T("AD_SOURCE")) != -1)
#endif
					)
				{
					// start connection
					UINT uRet = gReceiver_StartConnection(pInfoVec[uIndex].cSourceID);
					if( uRet == rcSuccess )
					{
						uRet = gReceiver_RetrieveCameraInformation(true);
						bDone = true;
						bSuccess = TRUE;
					}
				}
			}
            delete [] pInfoVec;
		}
	}
	return bSuccess;
}

bool DMoveInspectPipelineDlg::HaveCamerasChanged(NRKObArray<AiconCamera>& curCameras)
{
	bool bChanged = true;

	if ((m_lastCameras.GetSize() == curCameras.GetSize()))
	{
		bool bSameData = false;
		for (int icam = 0; icam < curCameras.GetSize(); icam++)
		{
			AiconCamera& lastCam = m_lastCameras[icam];
			AiconCamera& curCam  = curCameras[icam];
			bSameData = (lastCam == curCam);
			if (!bSameData)
			{
				break;
			}
		}
		bChanged = !bSameData;
	}

	return bChanged;
}

/// <summary>
/// Classes below MUST be used for communicating camera information
///  to Spatial Analyzer via JSON.
/// </summary>
struct InterfaceCamera
{
	int cameraId;

	double loc_x; // Inches
	double loc_y;
	double loc_z;
	double rot_x; // Degrees
	double rot_y;
	double rot_z;
};

static void to_json(nlohmann::json& j, const InterfaceCamera& s)
{
	j["cameraId"] = s.cameraId;
	j["loc_x"] = s.loc_x;
	j["loc_y"] = s.loc_y;
	j["loc_z"] = s.loc_z;
	j["rot_x"] = s.rot_x;
	j["rot_y"] = s.rot_y;
	j["rot_z"] = s.rot_z;
}

struct InterfaceCameras
{
	std::vector<InterfaceCamera> Cameras;
};

static void to_json(nlohmann::json& j, const InterfaceCameras& s)
{
	j["Cameras"] = s.Cameras;
}

bool DMoveInspectPipelineDlg::ProcessCameras()
{
	bool bSuccess = false;

	if (m_numCameras && m_pvCameras != nullptr)
	{
		InterfaceCameras pipelineCameraList;

		// Generate a list of cameras in a more NRK usable format
		NRKObArray<AiconCamera> cameras;
		// These are the cameras of the project
		for (int icam = 0; icam < m_numCameras; icam++)
		{
			AiconCamera& curCam = *new AiconCamera();
			cameras.Add(curCam);

			auto camera = m_pvCameras[icam];

			TRACE1("Camera Number: %d\n", camera.iNumber);
			CString val;
			val.Format(_T("Loc: (%lf, %lf, %lf) mm\n"), camera.dX, camera.dY, camera.dZ);
			TRACE(val);
			val.Format(_T("Rot: (%lf, %lf, %lf) rad\n"), camera.dOm, camera.dPh, camera.dKa);
			TRACE(val);

			//////////////////////////////////////////////////////
			// add camera to array
			Vector rot(camera.dOm, camera.dPh, camera.dKa);

			curCam.SetCameraID(camera.iNumber);
			curCam.SetRot(rot); // Add rotation info in radians

			Vector posInches(camera.dX, camera.dY, camera.dZ);
			curCam.SetLoc(posInches); // Add location info in mm
			ConvertVectorToInches(LU_MILLIMETERS, posInches);

			// Radians to degrees for Pipeline
			rot *= 1.0 / DTR;

			InterfaceCamera& cam = *new InterfaceCamera;
			cam.cameraId = icam;
			cam.loc_x = posInches.x;
			cam.loc_y = posInches.y;
			cam.loc_z = posInches.z;
			cam.rot_x = rot.x;
			cam.rot_y = rot.y;
			cam.rot_z = rot.z;
			pipelineCameraList.Cameras.push_back(cam);
		}

		// Now check to see if camera information has changed.  If not, no 
		//  need to send
		if (HaveCamerasChanged(cameras))
		{
			TRACE("Pushing new camera transforms to SA!\n");

		    // NOTE: Must Use SATrackingPointSet as shown below for SendPointSetTracking
			// https://github.com/nlohmann/json
			// create an empty structure (null)
			nlohmann::json j;

			j = pipelineCameraList;

			std::string foobar = j.dump();

			CString jsonCameraObjects(foobar.c_str());
			m_pipelineSDK.UpdateCameraTransforms(jsonCameraObjects);

			// Update our last cameras list
			m_lastCameras.DeleteAll();
			m_lastCameras.Append(cameras);
			cameras.RemoveAllWithoutDeleting();
		}
	}

	return bSuccess;
}

void DMoveInspectPipelineDlg::OnBnClickedCreateRef()
{
    UpdateData(TRUE);

    if (StringHelper::IsBlank(m_sReferenceName))
    {
        AfxMessageBox(_T("Please Specify a Reference Name."));
    }
    else
    {
        // This will simply send off the request to SpatialAnalyzer to prompt for point selection.
        // The interface will wait for SA to send the selected points back to this instrument interface
        // and then create the reference with the user selected points
		m_pipelineSDK.RequestPointsFromSA();
    }
}

LRESULT DMoveInspectPipelineDlg::OnCreateDynamicReferenceMessage(WPARAM wParam, LPARAM lParam)
{
    TRACE("****** OnCreateDynamicReferenceMessage Handler\n");

    int cnt = (int) wParam;
    tPoint* tpts = (tPoint*)lParam;

    _CreateReferenceFromPoints(m_sReferenceName, cnt, tpts);
    delete [] tpts;

    return 0;
}

void DMoveInspectPipelineDlg::CreateReferenceFromPoints(const CString& name, std::vector<SAGroupPoint>& pts)
{
    if (StringHelper::IsBlank(name))
    {
        PostError(_T("Please Specify a Reference Name."));
		// TODO: Presently not supported by Pipeline
        //theApp.m_SAConnection.ProcessCompleted(MPResult::DoneFatalError);
    }

    m_sReferenceName = name;

    UpdateData(FALSE);

    // Just to be safe, lets kill the data polling timer
    KillTimer(m_nIDData);

    CreateReferenceFromPoints(pts);

    // re-start the "data polling/acquisition"-timer
    m_nIDData = SetTimer(54321, 100, NULL);
}

LRESULT DMoveInspectPipelineDlg::OnRebuildDynamicRefListMessage(WPARAM wParam, LPARAM lParam)
{
    TRACE("****** OnRebuildDynamicRefListMessage Handler\n");
    GetReferences();
    return 0;
}


LRESULT DMoveInspectPipelineDlg::OnDynamicRefChangedMessage(WPARAM wParam, LPARAM lParam)
{
    TRACE("****** OnDynamicRefChangedMessage Handler\n");
    TCHAR sBuf[256];

    UpdateReferenceDetails(NULL);

    // get the current reference mode
    UINT uRet = gReceiver_GetActiveReference(sBuf, 256);
    if(uRet == rcSuccess)
    {
        CString sRef( sBuf );

        // select the 'none' mode
        if( sRef.IsEmpty() )
        {
            // select 'none"
            m_referenceComboBox.SetCurSel(0);
        }
        else
        {
            // select the string
            int retVal = m_referenceComboBox.SelectString(0, sRef);
            if (retVal == CB_ERR)
            {
#ifdef _DEBUG
                CString msg;
                msg.Format(_T("%s: %s\n"), _T("Failed to find reference name"), sRef);
                TRACE(msg);
#endif
            }
        }
    }

    return 0;
}

void DMoveInspectPipelineDlg::SetCodedPointsTypeCheck(const BOOL& state)
{
    m_bCoded = state;
}

void DMoveInspectPipelineDlg::SetNonCodedPointsTypeCheck(const BOOL& state)
{
    m_bNonCoded = state;
}

void DMoveInspectPipelineDlg::SetAdapterTypes(const BOOL& bFrames, const BOOL& bPoints)
{
    m_bAdapterFrames = bFrames;
    m_bAdapterPoints = bPoints;
}

void DMoveInspectPipelineDlg::SetAdapterFrames(const BOOL& bFrames)
{
    m_bAdapterFrames = bFrames;
}

void DMoveInspectPipelineDlg::SetAdapterPoints(const BOOL& bPoints)
{
    m_bAdapterPoints = bPoints;
}

void DMoveInspectPipelineDlg::ToggleSnapshot()
{
    m_bSnapShot = !m_bSnapShot;
}


bool DMoveInspectPipelineDlg::FlushBufferedMeasurements()
{
    bool bSuccess = false;

#if FALSE
	{
		TextFileContainer textFile;
		textFile.AddLine(_T("BEGIN DMoveInspectPipelineDlg::FlushBufferedMeasurements"));
		textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
	}
#endif

	if (m_pipelineSDK.SendMeasurements(m_collectionName, m_groupName, m_targetName, 0.0, 0.0))
	{
		m_pipelineSDK.ClearMeasurements();
		bSuccess = true;
	}

#if FALSE
	{
		TextFileContainer textFile;
		textFile.AddLine(_T("END DMoveInspectPipelineDlg::FlushBufferedMeasurements"));
		textFile.AppendToFile(_T("C:\\AICON\\nrkLog.txt"));
	}
#endif

    return bSuccess;
}

void DMoveInspectPipelineDlg::SetStoreItemsTriggered()
{
    m_bStorePointsAlways = FALSE;
    m_bStorePointsTriggered = TRUE;
}

void DMoveInspectPipelineDlg::SetStoreItemsAlways()
{
    m_bStorePointsAlways = TRUE;
    m_bStorePointsTriggered = FALSE;
}

// Event handler invoked once connected to Spatial Analyzer
void DMoveInspectPipelineDlg::Pipeline_OnConnectToHardware(LPCTSTR selectedSAInstName)
{
	AfxMessageBox(selectedSAInstName);
	m_pipelineSDK.SetInstrumentRemoveAllPriorShotsOverride(TRUE);

	// For testing request for loading simple instrument graphic at instrument base
    // NOTE: Specify an empty filename if no graphic desired for a particular component (instrument or probe).
	CString graphicsPath = theApp.ProgramDirectory;
	graphicsPath += _T("Graphics\\");
	CString instGraphics(graphicsPath + _T("Aicon MoveInspect Camera.STL"));
	CString instProbe(graphicsPath + _T("Aicon MoveInspect Probe.STL"));
	m_pipelineSDK.SetInstrumentGraphics(instGraphics, instProbe);
}

// Event handler invoked to notify instrument to begin live updates
void DMoveInspectPipelineDlg::Pipeline_OnActivateWatchUpdating()
{
	// Should now activate instrument to send live point updates to SA
	ActivateWatchUpdating();
}

// Event handler invoked when the length/temp units within Spatial Analyzer have changed
void DMoveInspectPipelineDlg::Pipeline_OnUnitsChange(long lengthUnits, long temperatureUnits)
{
	// LU_INVALID = 0
	// LU_METERS = 1
	// LU_CENTIMETERS = 2
	// LU_MILLIMETERS = 3
	// LU_FEET = 4
	// LU_INCHES = 5
	m_displayLengthUnits = (LengthUnitsType)lengthUnits;

	// lengthUnits

	// TU_INVALID = 0,
	// TU_DEGS_C,
	// TU_DEGS_F

	// temperatureUnits

	PostMessage(WM_SAUNITSCHANGE, 0, 0);
}

// Event handler invoked via MP Instrument OpCheck.
void DMoveInspectPipelineDlg::Pipeline_OnOpCheck(LPCTSTR command)
{
	BOOL bRet = FALSE;
	//AfxMessageBox(command);

	CString Command(command);

	// Instrument Operational Check MP Command
	if (StringHelper::NameCompare(Command.Left(16), _T("Set MeasureMode ")))
	{
		bRet = TRUE;
		CString modeType = Command.Mid(16);
		if (StringHelper::NameCompare(modeType, _T("Targeting")) ||
			StringHelper::NameCompare(modeType, _T("Tracking")))
		{
			bRet = SetTargetingMeasurementMode() ? TRUE : FALSE;
			if (bRet)
			{
				SetActionType(TRACKING);
			}
		}
		else if (StringHelper::NameCompare(modeType, _T("Probing")))
		{
			bRet = SetProbingMeasurementMode() ? TRUE : FALSE;
			if (bRet)
			{
				SetActionType(PROBING);
			}
		}
		else
		{
			bRet = FALSE;
		}
		UpdateData(FALSE);

		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(bRet ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
	else if (StringHelper::NameCompare(Command.Left(11), _T("Set Filter ")))
	{
		bRet = TRUE;
		CString remainder = Command.Mid(11).Trim();

		int currentPosition = 0;
		CString filterType = remainder.Tokenize(_T(" "), currentPosition);
		CString state = remainder.Tokenize(_T(" "), currentPosition);

		BOOL bState = FALSE;
		if (StringHelper::NameCompare(state, _T("TRUE")))
		{
			bState = TRUE;
		}

		if (StringHelper::NameCompare(filterType, _T("Coded")))
		{
			SetCodedPointsTypeCheck(bState);
		}
		else if (StringHelper::NameCompare(filterType, _T("NonCoded")))
		{
			SetNonCodedPointsTypeCheck(bState);
		}
		else if (StringHelper::NameCompare(filterType, _T("Adapter Frames")))
		{
			SetAdapterFrames(bState);
		}
		else if (StringHelper::NameCompare(filterType, _T("Adapter Points")))
		{
			SetAdapterPoints(bState);
		}
		else
		{
			bRet = FALSE;
		}

		// So filter variables are updated...
		UpdateData(FALSE);

		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(bRet ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
	// XIT-1836
	else if (StringHelper::NameCompare(Command.Left(14), _T("Set StoreMode ")))
	{
		bRet = TRUE;
		CString modeType = Command.Mid(14);
		if (StringHelper::NameCompare(modeType, _T("Triggered")))
		{
			SetStoreItemsTriggered();
			bRet = TRUE;
		}
		else if (StringHelper::NameCompare(modeType, _T("Always")))
		{
			SetStoreItemsAlways();
			bRet = TRUE;
		}
		else
		{
			bRet = FALSE;
		}
		//pDlg->UpdateData(FALSE);

		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(bRet ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
	else if (StringHelper::NameCompare(Command.Left(14), _T("Set Reference ")))
	{
		CString refName = Command.Mid(14);

		bRet = SetActiveDynamicReference(refName);

		UpdateData(FALSE);

		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(bRet ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
	else if (StringHelper::NameCompare(Command, _T("Start Server")))
	{
		bRet = StartServer();
		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(bRet ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
	else if (StringHelper::NameCompare(Command, _T("Stop Server")))
	{
		bRet = StopServer();
		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(bRet ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
	else if (StringHelper::NameCompare(Command, _T("Connect")))
	{
		bRet = Connect(true);
		// Connecting, when successful will fire a timer to acquire 
		//  connection parameters... for MP, await this timer to fire
		//  and setup the interface properly.
		if (bRet == TRUE)
		{
			AwaitConnectionSetupEvent(2000);
		}
		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(bRet && pDlg->IsConnected() ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
	else if (StringHelper::NameCompare(Command, _T("Measure")) ||
		StringHelper::NameCompare(Command, _T("Measure Single")))
	{
		bRet = StartMeasure(SINGLE);
		// ProcessCompleted result handled within dialog GetData() method.
	}
	else if (StringHelper::NameCompare(Command, _T("Measure Continuous")))
	{
		bRet = StartMeasure(CONTINUOUS);
		// ProcessCompleted result handled within dialog GetData() method.
	}
	else if (StringHelper::NameCompare(Command, _T("Stop")))
	{
		bRet = StopMeasure();
		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(bRet && pDlg->IsConnected() ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
	else if (StringHelper::NameCompare(Command, _T("Snapshot")))
	{
		ToggleSnapshot();
		// TODO: Not yet supported by Pipeline
		//ProcessCompleted(pDlg->IsConnected() ? MPResult::DoneSuccess : MPResult::DoneFatalError);
	}
}

// Event handler invoked via MP when the collection / group / target names have changed.
void DMoveInspectPipelineDlg::Pipeline_OnSetColGroupTarget(LPCTSTR collectionName, LPCTSTR groupName, LPCTSTR targetName)
{
	//CString msg;
	//msg.Format(_T("Pipeline_OnSetColGroupTarget: %s::%s::%s"), collectionName, groupName, targetName);
	//AfxMessageBox(msg);

	SetObservationNames(collectionName, groupName, targetName);
}

// This class must be used for MP dynamic reference requests and for points returned from
// SpatialAnalyzer via the initiation of the 'RequestPointsFromSA()' call.
struct SAGroupPoint
{
public:
	std::string colName;
	std::string groupName;
	std::string pointName;

	std::vector<double> PointInInstrument;

	bool bHasUncert;
	std::vector<double> StdDev;
};

static void from_json(const nlohmann::json& j, SAGroupPoint& s)
{
	s.colName = j.at("colName").get<std::string>();
	s.groupName = j.at("groupName").get<std::string>();
	s.pointName = j.at("pointName").get<std::string>();
	s.PointInInstrument = j.at("PointInInstrument").get<std::vector<double>>();
	s.bHasUncert = j.at("bHasUncert").get<double>();
	s.StdDev = j.at("StdDev").get<std::vector<double>>();
}

// When 'RequestPointsFromSA()' called, this is the response from SA after user selects points.
// Normally take specified points and create a dynamic reference.
void DMoveInspectPipelineDlg::Pipeline_OnSAIPRequestPoints(LPCTSTR jsonPointObjects)
{
	auto j = nlohmann::json::parse(std::wstring(jsonPointObjects));
	auto parsed = j.get<std::vector<SAGroupPoint>>();

	//// Use the extracted points to create the desired dynamic reference on the instrument...
	//for (int i = 0; i < parsed.size(); i++)
	//{
	//	SAGroupPoint& pt = parsed[i];
	//	CString foo;
	//	foo.Format(_T("%s::%s::%s : (%lf, %lf, %lf)\n"), CString(pt.colName.c_str()), CString(pt.groupName.c_str()), CString(pt.pointName.c_str()), pt.PointInInstrument[0], pt.PointInInstrument[1], pt.PointInInstrument[2]);
	//	TRACE(foo);
	//}

	CreateReferenceFromPoints(parsed);
}


// This method invoked once points have been received from SA
void DMoveInspectPipelineDlg::CreateReferenceFromPoints(std::vector<SAGroupPoint>& pts)
{
	DWORD_PTR cnt = pts.size();
	if (!StringHelper::IsBlank(m_sReferenceName) && cnt > 0)
	{
		tPoint* tpts = new tPoint[cnt];
		SecureZeroMemory(tpts, cnt * sizeof(tPoint));
		CString namePart;
		__int64 numPart;
		Vector posInMM, stddevInMM;
		for (DWORD_PTR i = 0; i < cnt; i++)
		{
			SAGroupPoint& pt = pts[i];
			tPoint& tpt = tpts[i];
			CString name(pt.pointName.c_str());
			if (StringHelper::DivideNumberedName(name, namePart, numPart))
			{
				tpt.iNum = numPart;
			}
			//_tcscpy(tpt.cName, pt.pointName);
			lstrcpyn(tpt.cName, name, 128);
			tpt.bCoordValid = true;
			posInMM.x = pt.PointInInstrument[0];
			posInMM.y = pt.PointInInstrument[1];
			posInMM.z = pt.PointInInstrument[2];
			posInMM.Scale(25.4); // Convert from Inches to MM
			tpt.dX = posInMM.x;
			tpt.dY = posInMM.y;
			tpt.dZ = posInMM.z;
			if (pt.bHasUncert)
			{
				stddevInMM.x = pt.StdDev[0];
				stddevInMM.y = pt.StdDev[1];
				stddevInMM.z = pt.StdDev[2];
				stddevInMM.Scale(25.4);
				tpt.dSx = stddevInMM.x;
				tpt.dSy = stddevInMM.y;
				tpt.dSz = stddevInMM.z;
				tpt.bDeviationValid = true;
			}
		}

		// Since this method may be called via MPs, make direct call to create
		//  vs posting message and creating later.
		//PostMessage(WM_CREATEDYNAMICREFERENCE,(WPARAM) cnt,(LPARAM) tpts);
		_CreateReferenceFromPoints(m_sReferenceName, cnt, tpts);
		delete[] tpts;
	}
	else
	{
		// TODO: Presently not supported by Pipeline
		//theApp.m_SAConnection.ProcessCompleted(MPResult::DoneFatalError);
	}
}

void DMoveInspectPipelineDlg::_CreateReferenceFromPoints(const CString& sDynRefName, const DWORD_PTR cnt, tPoint* tpts)
{
	//wchar_t* refName = m_sReferenceName.GetBuffer();

	TCHAR refName[256];
	lstrcpyn(refName, sDynRefName, 256);

	//
	// A note from marcel.ellerbrok@aicon.de:
	// A call to gReceiver_CreateReference automatically activates the new reference. 
	// So you dont have to call gReceiver_SetActiveReference. 
	//
	UINT uRet = gReceiver_CreateReference(refName, tpts, cnt);
	if (uRet != rcSuccess)
	{
		PostError(_T("Failed to Create Reference."));
		// TODO: Presently not supported by Pipeline
		//theApp.m_SAConnection.ProcessCompleted(MPResult::DoneFatalError);
	}
	else
	{
		// Experienced situation where active reference wasn't always set
		//  once created... added gReceiver_SetActiveReference.
		//uRet = gReceiver_SetActiveReference( refName );

		// Need to update the reference combo so it will include
		//  the newly created reference.  This will also update
		//  the combo to select the currently active reference
		//  within MoveInspect
		//GetReferences();
		PostMessage(DMoveInspectPipelineDlg::WM_DYNAMICREFLISTREBUILD, 0, 0);
		// TODO: Presently not supported by Pipeline
		//theApp.m_SAConnection.ProcessCompleted(MPResult::DoneSuccess);
	}
}

void DMoveInspectPipelineDlg::Pipeline_OnSAIPCreateDynamicReference(LPCTSTR dynRefName, LPCTSTR jsonPointObjects)
{
	CString name(dynRefName);

	auto j = nlohmann::json::parse(std::wstring(jsonPointObjects));
	auto parsed = j.get<std::vector<SAGroupPoint>>();

	//// Use the extracted points to create the desired dynamic reference on the instrument...
	//for (int i = 0; i < parsed.size(); i++)
	//{
	//	SAGroupPoint& pt = parsed[i];
	//	CString foo;
	//	foo.Format(_T("%s::%s::%s : (%lf, %lf, %lf)\n"), CString(pt.colName.c_str()), CString(pt.groupName.c_str()), CString(pt.pointName.c_str()), pt.PointInInstrument[0], pt.PointInInstrument[1], pt.PointInInstrument[2]);
	//	TRACE(foo);
	//}

	CreateReferenceFromPoints(name, parsed);
}

// This will be invoked when performing the following within SpatialAnalyzer:
//   Instrument >> Automatic Measurement >> Auto-Correspond with Proximity Trigger >> Points
void DMoveInspectPipelineDlg::Pipeline_OnSAIPAutoMeasProximity(BOOL bProcessDone)
{
	if (IsGuidedMeasureActive())
	{
		//OnMeasurementDone(MPResult::DoneFatalError);
		return;
	}

	if (bProcessDone == FALSE) // Begin AutoMeasure
	{
	    SetAutoProxMode(true);

		ClearAlert();

		m_pipelineSDK.Remove6DModel();

		auto jsonReturn = m_pipelineSDK.GetCurAutoMeasureDestination();
		auto j = nlohmann::json::parse(std::wstring(jsonReturn));
		auto destination = j.get<SAAutoMeasureDestination>();

		CString foo(destination.collection.c_str());
		SetCollectionName(foo);
		foo = destination.group.c_str();
		SetGroupName(foo);
		foo = destination.target.c_str();
		SetTargetName(foo);
		UpdateData(FALSE);

		jsonReturn = m_pipelineSDK.GetCurAutoMeasureTolerances();

		j = nlohmann::json::parse(std::wstring(jsonReturn));
		auto tolerances = j.get<SAAutoMeasureTolerances>();

		m_pipelineSDK.AudibleSetParams(tolerances.proximityTolerance, tolerances.toneRampStartTolerance);
		m_pipelineSDK.AudibleStart();

		SetTargetingMeasurementMode();

		UINT uRet = gReceiver_SendStartLiveMeasurement();

		m_actionTypeMode = PROBING;

		UpdateData(FALSE);
	}
	else // End AutoMeasure
	{
		m_pipelineSDK.AudibleStop();
		gReceiver_SendStopLiveMeasurement();

		SetProbingMeasurementMode();

		SetAutoProxMode(false);

		// TODO: Not supported by Pipeline
		//theApp.m_SAConnection.ProcessCompleted(MPResult::DoneSuccess);

		// Set ignore mode timer so that any extra point data
		//  received after the continuous measurement mode deactivation
		//  will be discarded.
		SetIgnorePointsTimer(3000);
	}
}

struct SAMeasGroupPoint
{
	std::string Group;
	std::string Target;
	std::vector<double> Nominal;
};

static void from_json(const nlohmann::json& j, SAMeasGroupPoint& s)
{
	s.Group = j.at("Group").get<std::string>();
	s.Target = j.at("Target").get<std::string>();
	s.Nominal = j.at("Nominal").get<std::vector<double>>();
}

struct SAMeasGroup
{
	std::string TargetSuffix;
	std::string GroupSuffix;
	std::vector<SAMeasGroupPoint> Points;
};

static void from_json(const nlohmann::json& j, SAMeasGroup& s)
{
	s.TargetSuffix = j.at("TargetSuffix").get<std::string>();
	s.GroupSuffix = j.at("GroupSuffix").get<std::string>();
	s.Points = j.at("Points").get<std::vector<SAMeasGroupPoint>>();
}

struct SAMeasGuidePointDeltaInWorking
{
	double DeltaXInches;
	double DeltaYInches;
	double DeltaZInches;
	double DeltaMag;
};

static void from_json(const nlohmann::json& j, SAMeasGuidePointDeltaInWorking& s)
{
	s.DeltaXInches = j.at("DeltaXInches").get<double>();
	s.DeltaYInches = j.at("DeltaYInches").get<double>();
	s.DeltaZInches = j.at("DeltaZInches").get<double>();
	s.DeltaMag = j.at("DeltaMag").get<double>();
}

std::wstring CurGroupTarget;
std::wstring DeltaInWorking;
std::wstring DeltaMagnitude;
double ProbeRadius = 0.025;

bool DMoveInspectPipelineDlg::ServoView(int activeGuidePtIndex)
{
	return m_pipelineSDK.SetGuidedPointServoView(activeGuidePtIndex) ? true : false;
}

void DMoveInspectPipelineDlg::GetGuideDeltaInWorking(int activeGuidePtIndex, const Vector& actual, Vector& DeltaInWorking, double& DeltaMag)
{
	auto deltaInWorkingInfo = m_pipelineSDK.GetGuidedDeltaInWorking(activeGuidePtIndex, actual.x, actual.y, actual.z);
	auto jj = nlohmann::json::parse(std::wstring(deltaInWorkingInfo));
	auto deltaInWorking = jj.get<SAMeasGuidePointDeltaInWorking>();

	DeltaInWorking.x = deltaInWorking.DeltaXInches;
	DeltaInWorking.y = deltaInWorking.DeltaYInches;
	DeltaInWorking.z = deltaInWorking.DeltaZInches;
	DeltaMag = deltaInWorking.DeltaMag;
}

struct SAGuidedPointInfo
{
	std::string collection;
	std::string group;
	std::string target;
	std::string grpSuffix;
	std::string tgtSuffix;
	double NominalXInches;
	double NominalYInches;
	double NominalZInches;
};

static void from_json(const nlohmann::json& j, SAGuidedPointInfo& s)
{
	s.collection = j.at("Collection").get<std::string>();
	s.group = j.at("Group").get<std::string>();
	s.target = j.at("Target").get<std::string>();
	s.grpSuffix = j.at("GroupSuffix").get<std::string>();
	s.tgtSuffix = j.at("TargetSuffix").get<std::string>();
	s.NominalXInches = j.at("NominalXInches").get<double>();
	s.NominalYInches = j.at("NominalYInches").get<double>();
	s.NominalZInches = j.at("NominalZInches").get<double>();
}

void DMoveInspectPipelineDlg::GetGuideInfo(int activeGuidePtIndex, Vector& nominal, CString& col, CString& grp, CString& tgt, CString& grpSuffix, CString& tgtSuffix)
{
	auto jsonReturn = m_pipelineSDK.GetGuidedPointInfo(activeGuidePtIndex);
	auto j = nlohmann::json::parse(std::wstring(jsonReturn));
	auto info = j.get<SAGuidedPointInfo>();

	CString foo(info.collection.c_str());
	col = foo;
	grp = info.group.c_str();
	tgt = info.target.c_str();
	grpSuffix = info.grpSuffix.c_str();
	tgtSuffix = info.tgtSuffix.c_str();
	nominal.x = info.NominalXInches;
	nominal.y = info.NominalYInches;
	nominal.z = info.NominalZInches;
}

SAMeasGroup g_guidGroup;

// Called from: DGuidedMeasurePipeline::ProcessNewData
BOOL DMoveInspectPipelineDlg::AdvanceGuide(const int& inc)
{
	BOOL bLastPoint = FALSE;

	if (inc > 0)
	{
		m_activeGuidePtIndex++;

		//Check if its the last
		if (m_activeGuidePtIndex >= g_guidGroup.Points.size())
		{
			bLastPoint = TRUE;
		}
	}
	else if (inc < 0)
	{
		if (m_activeGuidePtIndex > 0)
		{
			m_pipelineSDK.RemoveLastMeasurementInSA();
			m_activeGuidePtIndex--;
		}
	}
	return bLastPoint;
}

// This will be invoked when performing the following within SpatialAnalyzer:
//   Instrument >> Automatic Measurement >> Measure Batch of Points
void DMoveInspectPipelineDlg::Pipeline_OnSAIPMeasGroup(LPCTSTR jsonPointObjects)
{
	auto j = nlohmann::json::parse(std::wstring(jsonPointObjects));
	g_guidGroup = j.get<SAMeasGroup>();

	if (IsGuidedMeasureActive() || IsAutoProxActive() || (g_guidGroup.Points.size() <= 0))
	{
		//OnMeasurementDone(MPResult::DoneFatalError);
		return;
	}

	m_bCaptureNextGuidedMeasure = false;

	// Send message to begin Guided measurement mode
	BOOL& bState = *new BOOL;
	bState = TRUE; // Turn it on
	PostMessage(WM_GUIDEDMODESTATECHANGE, 0, (LPARAM)&bState);
}

// Event Handler Invoked whenever instrument transform has changed within Spatial Analyzer
void DMoveInspectPipelineDlg::Pipeline_OnInstToWorkingTransformChanged(LPCTSTR workingFrameName, LPCTSTR jsonTransform, double instrumentScaleFactor)
{
	// https://github.com/nlohmann/json
	// create an empty structure (null)
	CString foo(jsonTransform);
	::std::wstring des(foo.GetString());
	nlohmann::json jdata;
	jdata = nlohmann::json::parse(des);
	auto T = jdata.get<std::array<std::array<double, 4>, 4>>();
}