// DMoveInspectPipelineDlg.h : header file
//

#pragma once
#include "afxwin.h"
#include "afxbutton.h"
#include "resource.h"
#include "CPIPEOCX.h"

#include "NRK/vector.h"

class DGuidedMeasurePipeline;
class AiconCamera;
struct SAGroupPoint;
struct SAMeasGroup;


// DMoveInspectPipelineDlg dialog
class DMoveInspectPipelineDlg : public CDialog
{
// Construction
public:
	DMoveInspectPipelineDlg(CWnd* pParent = NULL);	// standard constructor
	virtual ~DMoveInspectPipelineDlg();

// Dialog Data
	enum { IDD = IDD_AICONMOVEINSPECT_DIALOG };

	enum UserMsg
	{
		AFTER_INIT = WM_USER+2,
		WM_SETOBSERVATIONNAMES = WM_USER+3,
		WM_SAUNITSCHANGE = WM_USER+4,
		WM_GUIDEDMODESTATECHANGE = (WM_USER+5),
		WM_AUTOPROXSTATECHANGE = (WM_USER+6),
		IGNORE_POINTS_TIMER = (WM_USER+7),
        WM_CREATEDYNAMICREFERENCE = (WM_USER+8),
        WM_DYNAMICREFLISTREBUILD = (WM_USER+9),
        WM_DYNAMICREFCHANGED = (WM_USER+10),
	};

	void SetObservationNames(const CString& collectionName,const CString& groupName,const CString& targetName);
	void IncrementGroup(const int& inc);
	void IncrementTarget(const int& inc);

	void SetCollectionName(const CString& name) { m_collectionName = name; }
	void SetGroupName(const CString& name) { m_groupName = name; }
	void SetTargetName(const CString& name) { m_targetName = name; }

	CString GetCollectionName(void) const { return m_collectionName; }
	CString GetGroupName(void) const { return m_groupName; }
	CString GetTargetName(void) const { return m_targetName; }

	void ActivateWatchUpdating(void);

	void PostError(const CString& msg);
	void PostAlert(const CString& msg, const COLORREF& txtColor, const COLORREF& bkgndColor);
	void ClearAlert(void);

	void GetTargetIDs(CWordArray& targetIDs) { targetIDs.RemoveAll(); targetIDs.Append(m_codedTargetIDs); }

	void OnReceiverEvent(int iEvent);
	bool ProcessData(void) { return GetData(); }
	bool IsConnected(void) const { return m_bConnection; }
	bool CheckConnection();

	enum Actions
	{
		PROBING	= 0,
		TRACKING	= 1,
	};

	enum MeasMode
	{
		SINGLE	= 0,
		CONTINUOUS = 1
	};

	void SetActionType(const Actions& action) { m_actionTypeMode = (int) action; }

	bool SetProbingMeasurementMode();
	bool SetTargetingMeasurementMode();

    // XIT-1836
    void SetStoreItemsTriggered();
    void SetStoreItemsAlways();

	BOOL StartServer();
	BOOL StopServer();
	BOOL Connect(const bool& bSilent = false);
	BOOL StartMeasure(const MeasMode& mode);
	BOOL StopMeasure();

    BOOL SetActiveDynamicReference(const CString& refName);
    BOOL DoesDynamicReferenceExist(const CString& refName);

	bool AwaitConnectionSetupEvent(DWORD dwMilliseconds);

    void CreateReferenceFromPoints(std::vector<SAGroupPoint>& pts);
    void CreateReferenceFromPoints(const CString& name, std::vector<SAGroupPoint>& pts);

    void SetCodedPointsTypeCheck(const BOOL& state);
    void SetNonCodedPointsTypeCheck(const BOOL& state);
    void SetAdapterTypes(const BOOL& bFrames, const BOOL& bPoints);
    void SetAdapterFrames(const BOOL& bFrames);
    void SetAdapterPoints(const BOOL& bPoints);
    void ToggleSnapshot();

	int  GetGuidePointIndex(void) const { return m_activeGuidePtIndex; }
	void SetGuidePointIndex(const int& idx) { m_activeGuidePtIndex = idx; }

	bool ServoView(int activeGuidePtIndex);
	void GetGuideDeltaInWorking(int activeGuidePtIndex, const Vector& actual, Vector& DeltaInWorking, double& DeltaMag);
	void GetGuideInfo(int activeGuidePtIndex, Vector& nominal, CString& col, CString& grp, CString& tgt, CString& grpSuffix, CString& tgtSuffix);
	BOOL AdvanceGuide(const int& inc);

	typedef enum
	{
		LU_INVALID = 0,
		LU_METERS,
		LU_CENTIMETERS,
		LU_MILLIMETERS,
		LU_FEET,
		LU_INCHES
	} LengthUnitsType;
	double ConvertLengthToMeters(LengthUnitsType fromUnits, double fromValue) const;
	double ConvertLengthFromMeters(LengthUnitsType toUnits, double fromValue) const;
	CString GetUnitsLabel(LengthUnitsType units) const;
	LengthUnitsType GetDisplayUnits() { return m_displayLengthUnits; }

protected:
	CString m_version;
	CEdit m_MessageCtrl;
	DGuidedMeasurePipeline*		m_pGuidedMeasDlg;
	//CDockingBtnCtrl m_dockingBtnCtrl;
	CComboBox m_referenceComboBox;
	HICON m_hIcon;

	CPipelineSDK m_pipelineSDK;

    // Request to always fire in targeting when SINGLE button
    //  in GUI pressed.
    bool m_bSingleMeasureButtonPressed;

	UINT_PTR m_nIDConnection;
	UINT_PTR m_nIDData;
	bool     m_bConnection;

	CString m_collectionName;
	CString m_groupName;
	CString m_targetName;

    CString m_sReferenceName;
    CString m_sRefTargets;
    CString m_sRefSigma0;
    CString m_sRefVolume;

	CEdit m_collectionComboCtrl;
	CEdit m_groupComboCtrl;
	CEdit m_targetEditCtrl;

	int		m_actionTypeMode;
	CString m_lastX;
	CString m_lastY;
	CString m_lastZ;
	CString m_lastRadius;

	// When in "targeting" mode (versus "probing" mode), AICON
	//  does NOT send the probe information.  Thus, the variable
	//  below will cache the last valid probe radius received
	//  and use that when performing automated operations such as
	//  "Measure Batch of Points"
	double	m_lastProbeDiameterInMM;

    BOOL m_bStorePointsAlways;
    BOOL m_bStorePointsTriggered;
	BOOL m_bAutoGroupIncrement;
	BOOL m_bSnapShot;

    BOOL m_bCoded;
    BOOL m_bNonCoded;
    BOOL m_bAdapterFrames;
    BOOL m_bAdapterPoints;

	// Coded Target IDs needed during "Batch of Points" action...
	CWordArray m_codedTargetIDs;

	NRKObArray<AiconCamera> m_lastCameras;
	tCamera* m_pvCameras;
	int m_numCameras;
	bool ProcessCameras();

	bool HaveCamerasChanged(NRKObArray<AiconCamera>& curCameras);

	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	virtual BOOL OnInitDialog();
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);

	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();

	afx_msg void OnDestroy();
	afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);
	afx_msg void OnSetfocusErroredit();
	afx_msg void OnTimer(UINT_PTR nIDEvent);

	afx_msg void OnBnClickedSingleMeasure();
	afx_msg void OnBnClickedContinuousMeasureON();
    afx_msg void OnBnClickedContinuousMeasureOFF();
	afx_msg void OnBnClickedStopServer();
	afx_msg void OnBnClickedConnect();

    afx_msg void OnBnClickedProbingRadio();
    afx_msg void OnBnClickedTrackingRadio();

	afx_msg void OnBnClickedDeleteLastPoint();
	afx_msg void OnBnClickedOptions();

	afx_msg void OnCbnSelchangeReferenceCombo();
    afx_msg void OnBnClickedCreateRef();

    afx_msg LRESULT OnSetGuidedModeStateChangeMessage(WPARAM a, LPARAM b);
	afx_msg LRESULT OnSetObservationNamesMessage(WPARAM a, LPARAM b);
	afx_msg LRESULT OnAfterInit(WPARAM a, LPARAM b);
    afx_msg LRESULT OnCreateDynamicReferenceMessage(WPARAM a, LPARAM b);
    afx_msg LRESULT OnRebuildDynamicRefListMessage(WPARAM a, LPARAM b);
    afx_msg LRESULT OnDynamicRefChangedMessage(WPARAM a, LPARAM b);

	bool GetMeasureModes();
	void ClearMeasurementModes();
	bool IsTargetingModeActive() const;
	bool IsProbingModeActive() const;
	bool CheckActiveMode(const CString& mode) const;
	bool GetData();
	void GetReferences();

	void SetIgnorePointsTimer(const int& timeInMS);

	void Pipeline_OnConnectToHardware(LPCTSTR connectData);
	void Pipeline_OnActivateWatchUpdating();
	void Pipeline_OnUnitsChange(long lengthUnits, long temperatureUnits);
	void Pipeline_OnOpCheck(LPCTSTR command);
	void Pipeline_OnSetColGroupTarget(LPCTSTR collectionName, LPCTSTR groupName, LPCTSTR targetName);
	void Pipeline_OnSAIPRequestPoints(LPCTSTR jsonPointObjects);
	void Pipeline_OnSAIPCreateDynamicReference(LPCTSTR dynRefName, LPCTSTR jsonPointObjects);
	void Pipeline_OnSAIPAutoMeasProximity(BOOL bProcessDone);
	void Pipeline_OnSAIPMeasGroup(LPCTSTR jsonPointObjects);
	void Pipeline_OnInstToWorkingTransformChanged(LPCTSTR workingFrameName, LPCTSTR jsonTransform, double instrumentScaleFactor);

	DECLARE_MESSAGE_MAP()
	DECLARE_EVENTSINK_MAP()

private:
	bool IsAutoProxActive(void) const { return m_bAutoProxActive; }
	void SetAutoProxMode(const bool& bState) { m_bAutoProxActive = bState; }
	bool IsGuidedMeasureActive(void) const { return m_bGuidedMeasActive; }
	void SetGuidedMeasureMode(const bool& bState) { m_bGuidedMeasActive = bState; }
	bool m_bAutoProxActive;
	bool m_bGuidedMeasActive;

	bool m_bCaptureNextGuidedMeasure;
	int m_activeGuidePtIndex;

	LengthUnitsType m_displayLengthUnits;

	HANDLE ConnectionSetupCompleteEvent;

	CPtrArray	m_measurementModes;
	double GetDisplayValue(const double& valInMM) const;
	void ConvertVectorToInches(LengthUnitsType current, Vector& v) const;
	void UpdateDialogInfo(const Vector& posInMM, const double& probeDiameterInMM);
	bool ProcessPoint(const Vector& posInMM, const Vector* pUncertaintyInMM, const Vector* pProbeDir, const double& probeDiameterInMM, const bool& bTrigger = false);	

	bool ProcessFrame(const CString& frameName, const Vector& posInMM, const Vector& rotInRadians, const BOOL& bUpdateOnly);
	bool ProcessProbeFrame(const Vector& posInches, const Vector& normal, const double& probeDiameterInches, const BOOL& bUpdateOnly);

    void _CreateReferenceFromPoints(const CString& sDynRefName, DWORD_PTR cnt, tPoint* tpts);

    void UpdateReferenceDetails(const tData* pCurrentDataSet);

	// Used after auto prox or guided measure modes to ignore any points shortly after
	//  mode ends and continuous measure is deactivated.  Without this, potential for
	//  data to be received and stored as point.
	bool		m_bIgnoreAllPoints;

    // For buffering data 
    bool FlushBufferedMeasurements();
};

class AiconCamera : public CObject
{
public:
	AiconCamera()
	{
		m_iNumber = -1;
		m_loc.x = m_loc.y = m_loc.z = 0.0;
		m_rot.x = m_rot.y = m_rot.z = 0.0;
	}
	AiconCamera(int cameraNo, Vector loc, Vector rot)
	{
		m_iNumber = cameraNo;
		m_loc = loc;
		m_rot = rot;
	}
	AiconCamera(const AiconCamera& source)
	{
		operator=(source);
	}
	void operator=(const AiconCamera& source)
	{
		m_iNumber = source.m_iNumber;
		m_loc = source.m_loc;
		m_rot = source.m_rot;
	}

	bool operator==(const AiconCamera& other) const
	{
		if (m_iNumber != other.m_iNumber)
			return false;
		if (m_loc != other.m_loc)
			return false;
		if (m_rot != other.m_rot)
			return false;

		return true;
	}

	void SetCameraID(const int& id) { m_iNumber = id; }
	void SetLoc(const Vector& loc) { m_loc = loc; }
	void SetRot(const Vector& rot) { m_rot = rot; }

	DECLARE_DYNCREATE(AiconCamera);

protected:
	int m_iNumber;

	Vector m_loc; // In mm
	Vector m_rot; // In rad
};
