// FancyList.cpp : implementation file
//

#include "stdafx.h"
#include "FancyList.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// FancyList

FancyList::FancyList()
{
	m_iSortColumn = -1;
	m_bSortAscending = TRUE;
	hasStoredSelection = false;
	storedSelection = 0;
}

FancyList::~FancyList()
{
}


BEGIN_MESSAGE_MAP(FancyList, CListCtrl)
	//{{AFX_MSG_MAP(FancyList)
	//}}AFX_MSG_MAP
	ON_NOTIFY_REFLECT(NM_RCLICK, OnNMRclick)
	ON_NOTIFY_REFLECT ( NM_CUSTOMDRAW, OnCustomdraw )
	ON_NOTIFY_REFLECT( LVN_COLUMNCLICK, OnColumnClick)
	ON_NOTIFY_REFLECT(LVN_ITEMCHANGED, OnItemChanged)
	ON_NOTIFY_REFLECT(LVN_ENDLABELEDIT, OnEndlabeledit)
	ON_NOTIFY_EX_RANGE(TTN_NEEDTEXTW, 0, 0xFFFF, OnToolTipText)
	ON_NOTIFY_EX_RANGE(TTN_NEEDTEXTA, 0, 0xFFFF, OnToolTipText)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// FancyList message handlers



void FancyList::OnCustomdraw( NMHDR* pNMHDR, LRESULT* pResult )
{
	NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>( pNMHDR );

	// Take the default processing unless we set this to something else below.
	*pResult = 0;

	// First thing - check the draw stage. If it's the control's prepaint
	// stage, then tell Windows we want messages for every item.

	if ( CDDS_PREPAINT == pLVCD->nmcd.dwDrawStage )
	{
		*pResult = CDRF_NOTIFYITEMDRAW;
	}
	else if ( CDDS_ITEMPREPAINT == pLVCD->nmcd.dwDrawStage )
	{
		// This is the notification message for an item.  We'll request
		// notifications before each subitem's prepaint stage.

		*pResult = CDRF_NOTIFYSUBITEMDRAW;
	}
	else if ( (CDDS_ITEMPREPAINT | CDDS_SUBITEM) == pLVCD->nmcd.dwDrawStage )
	{
		// This is the prepaint stage for a subitem. Here's where we set the
		// item's text and background colors. Our return value will tell 
		// Windows to draw the subitem itself, but it will use the new colors
		// we set here.
		// The text color will cycle through red, green, and light blue.
		// The background color will be light blue for column 0, red for
		// column 1, and black for column 2.

		CFont* pFont=NULL;
		DWORD_PTR row = GetItemData(pLVCD->nmcd.dwItemSpec);
		if (GetItemCustomDrawSpecs(row, pLVCD->iSubItem,
			&pLVCD->clrText, &pLVCD->clrTextBk, &pFont))
		{
			if (pFont != NULL)
			{
				CDC* pdc = CDC::FromHandle(pLVCD->nmcd.hdc);
				pdc->SelectObject(*pFont);
				*pResult = CDRF_NEWFONT;
			}
			else
				*pResult = CDRF_DODEFAULT;
		}
		else
			*pResult = CDRF_DODEFAULT;
	}
}
INT_PTR FancyList::OnToolHitTest(CPoint point, TOOLINFO * pTI) const {

   //See if the point falls onto a list item
	//UINT nFlags = 0;

	LVHITTESTINFO lvhitTestInfo;

	lvhitTestInfo.pt	= point;

	int nItem = ListView_SubItemHitTest(
			this->m_hWnd,
			&lvhitTestInfo);
	int nSubItem = lvhitTestInfo.iSubItem;

	UINT nFlags =   lvhitTestInfo.flags;

   //nFlags is 0 if the SubItemHitTest fails
   //Therefore, 0 & <anything> will equal false
   if (nFlags & LVHT_ONITEMLABEL)
	{
      //If it did fall on a list item,
      //and it was also hit one of the
      //item specific sub-areas we wish to show tool tips for

      //Get the client (area occupied by this control
      RECT rcClient;
      GetClientRect( &rcClient );

      //Fill in the TOOLINFO structure
      pTI->hwnd = m_hWnd;
      pTI->uId = (UINT) (nItem * 100 + nSubItem);
      pTI->lpszText = LPSTR_TEXTCALLBACK;
      pTI->rect = rcClient;

      return pTI->uId; //By returning a unique value per listItem,
                       //we ensure that when the mouse moves over another list item,
                       //the tooltip will change
   }
	else	//Otherwise, we aren't interested, so let the message propagate
	{ 
      return -1;
   }
}

BOOL FancyList::OnToolTipText( UINT id, NMHDR * pNMHDR, LRESULT * pResult ){
	//Handle both ANSI and UNICODE versions of the message
	TOOLTIPTEXTA* pTTTA = (TOOLTIPTEXTA*)pNMHDR;
	TOOLTIPTEXTW* pTTTW = (TOOLTIPTEXTW*)pNMHDR;

	//Ignore messages from the built in tooltip, we are processing them internally
	if( (pNMHDR->idFrom == (UINT_PTR)m_hWnd) &&  // #VS2019MigrationTweak
		 ( ((pNMHDR->code == TTN_NEEDTEXTA) && (pTTTA->uFlags & TTF_IDISHWND)) ||
         ((pNMHDR->code == TTN_NEEDTEXTW) && (pTTTW->uFlags & TTF_IDISHWND)) ) ){
      return FALSE;
   }


   *pResult = 0;

	CString strTipText;

	//Get the mouse position
	const MSG* pMessage;
   pMessage = GetCurrentMessage();
	ASSERT ( pMessage );
   CPoint pt;
   pt = pMessage->pt; //Get the point from the message
	ScreenToClient( &pt ); //Convert the point's coords to be relative to this control

   //See if the point falls onto a list item

	LVHITTESTINFO lvhitTestInfo;

	lvhitTestInfo.pt	= pt;

	int nItem = SubItemHitTest(&lvhitTestInfo);
	int nSubItem = lvhitTestInfo.iSubItem;

	UINT nFlags =   lvhitTestInfo.flags;

   //nFlags is 0 if the SubItemHitTest fails
   //Therefore, 0 & <anything> will equal false
   if( nFlags & LVHT_ONITEMLABEL )
	{
	  //If it did fall on a list item,
      //and it was also hit one of the
      //item specific sub-areas we wish to show tool tips for

		// try to get tips from the derived class
	  CString strTipText;
	  if (GetToolTip(GetItemData(nItem),nSubItem,strTipText))
	  {
         //Deal with UNICODE

      #ifndef _UNICODE
	      if (pNMHDR->code == TTN_NEEDTEXTA)
		      lstrcpyn(pTTTA->szText, strTipText, 80);
	      else
			{  
		     // _mbstowcsz(pTTTW->szText, strTipText, 80);
				//MultiByteToWideChar(CP_ACP,0,pTTTW->szText,-1,strTipText.GetBuffer(80),80); // FIXLATER
			//	strTipText.ReleaseBuffer();
			}
      #else
	      if (pNMHDR->code == TTN_NEEDTEXTA)
			{  
		      //_wcstombsz(pTTTA->szText, strTipText, 80);
				//WideCharToMultiByte(CP_ACP,0,pTTTA->szText,-1,strTipText,80,NULL,NULL);  // FIXLATER
			}
	      else
		      lstrcpyn(pTTTW->szText, strTipText, 80);
      #endif
         return FALSE;    //We found a tool tip,
                         //tell the framework this message has been handled

		////////////////////////////////////////////////////////////////////////////////
		// ****** Special note *****
		//
		// Still don't understand why the function must return FALSE for CListCtrl
		// so as not to cause flickering, as opposed to Nate Maynard's derivation
		// from CTreeCtrl.
		// I have experimented with disabling Tooltips for the control
		// and found out that a "ghost" tooltip appears for a fraction of a second...
		//
		// I am completely at a loss...
		// Seems to work, though...
		//
		////////////////////////////////////////////////////////////////////////////////
							
      }
   }

   return FALSE; //We didn't handle the message,
                 //let the framework continue propagating the message
}


void FancyList::OnColumnClick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	const int iColumn = pNMListView->iSubItem;

	// if it's a second click on the same column then reverse the sort order,
	// otherwise sort the new column in ascending order.
	Sort( iColumn, iColumn == m_iSortColumn ? !m_bSortAscending : TRUE );
	
	*pResult = 0;
}
void FancyList::SetSortParameters(int col, BOOL Ascending)
{
	m_iSortColumn = col;
	m_bSortAscending = Ascending;
}

void FancyList::Sort()
{
	Sort(m_iSortColumn, m_bSortAscending);
}
void FancyList::Sort( int iColumn, BOOL bAscending )
{
	m_iSortColumn = iColumn;
	m_bSortAscending = bAscending;

	// show the appropriate arrow in the header control.
	//m_ctlHeader.SetSortArrow( m_iSortColumn, m_bSortAscending );

	StartedSorting();
	VERIFY( SortItems( CompareFunction, reinterpret_cast<DWORD_PTR>( this ) ) ); // sws - XIT-427 (crash when viewing GR's point list)
	FinishedSorting();
}

int CALLBACK FancyList::CompareFunction( LPARAM lParam1, LPARAM lParam2, LPARAM lParamData )
{
	FancyList* pListCtrl = reinterpret_cast<FancyList*>( lParamData );
	ASSERT( pListCtrl->IsKindOf( RUNTIME_CLASS( CListCtrl ) ) );
	
   return pListCtrl->SortCompareOperator
      (pListCtrl->m_iSortColumn,lParam1, lParam2, pListCtrl->m_bSortAscending );
}


void FancyList::OnItemChanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	*pResult = 0;

	if ((pNMListView->uChanged & LVIF_STATE) && (pNMListView->uNewState & LVNI_SELECTED)) 
	{
		OnSelectionChanged();		// so we can handle if needed.
		return;
	}


	if (pNMListView->uOldState == 0 && pNMListView->uNewState == 0)
		return;	// No change

	BOOL bPrevState = (BOOL)(((pNMListView->uOldState & 
				LVIS_STATEIMAGEMASK)>>12)-1);   // Old check box state
	if (bPrevState < 0)	// On startup there's no previous state 
		bPrevState = 0; // so assign as false (unchecked)

	// New check box state
	BOOL bChecked=(BOOL)(((pNMListView->uNewState & LVIS_STATEIMAGEMASK)>>12)-1);   
	if (bChecked < 0) // On non-checkbox notifications assume false
		bChecked = 0; 

	if (bPrevState == bChecked) // No change in check box
		return;

	// call overridable function to handle the change in check state.
	OnCheckChange( GetItemData(pNMListView->iItem),bChecked);
}
int FancyList::GetItemFromRow( const DWORD_PTR row ) const
{
	// the row corresponds to the LParam in the list item... search to find it then use it.
	LVFINDINFO info;
	info.flags = LVFI_PARAM;
	info.lParam = row;
	int res = FindItem(&info);
	return res;
}
bool FancyList::EnsureRowVisible( const DWORD_PTR row)
{
	int itemIndex = GetItemFromRow( row );
	return EnsureVisible(itemIndex,false);
}
BOOL FancyList::GetColumnText(const int i,CString& str)
{
	if(i>=GetHeaderCtrl()->GetItemCount())
		return FALSE;

	LVCOLUMN pc;
	TCHAR text[255];
	pc.mask = LVCF_TEXT;
	pc.fmt = NULL; 
	pc.cx = NULL; 
	pc.pszText = (LPTSTR)&text; 
	pc.cchTextMax = 255; 
	pc.iSubItem = NULL; 
	pc.iImage = NULL;
	pc.iOrder = NULL;
	int ret = GetColumn(i,&pc);
	str = text;
	return ret;
}

void FancyList::GetAsStringArray(ListFmt mode, CStringArray& array)
{
	BOOL checks = GetExtendedStyle() & LVS_EX_CHECKBOXES;
	CString s;
	array.RemoveAll();
	if (checks)
	{
		s += _T("Check");
		if(mode==F_EXCEL)
			s+="\t";
		else if(mode==F_ASCII)
			s+="\t\t";
		else if(mode==F_COMMA)
			s+=",";
		else
			s+=" ";
	}
	for(int k=0;k<GetHeaderCtrl()->GetItemCount();k++)
	{
		CString temp;
		if(GetColumnText(k,temp))
			s+=temp;
		if(k==GetHeaderCtrl()->GetItemCount()-1)
			s+="\n";
		else
		{
			if(mode==F_EXCEL)
				s+="\t";
			else if(mode==F_ASCII)
				s+="\t\t";
			else if(mode==F_COMMA)
				s+=",";
			else
				s+=" ";
		}
	}		
	array.Add(s);
	for(int i =0;i<GetItemCount();i++)
	{
		s="";
		if (checks)
		{
			if (GetCheck(i))
				s += _T("X ");
			else 
				s += _T(" ");
			
			if(mode==F_EXCEL)
				s+="\t";
			else if(mode==F_ASCII)
				s+="\t\t";
			else if(mode==F_COMMA)
				s+=",";
			else
				s+=" ";
		}
		for(int j=0;j<GetHeaderCtrl()->GetItemCount();j++)
		{
			CString temp;
			temp = GetItemText(i,j);
			s+=temp;
			if(j==GetHeaderCtrl()->GetItemCount()-1)
				s+="\n";
			else
			{
				if(mode==F_EXCEL)
					s+="\t";
				else if(mode==F_ASCII)
					s+="\t\t";
				else if(mode==F_COMMA)
					s+=",";
				else
					s+=" ";
			}
		}
		array.Add(s);
	}
	for(int i=0;i<3;i++)
	{
		s="";
		for(int j=0;j<GetHeaderCtrl()->GetItemCount();j++)
		{
			if(j==GetHeaderCtrl()->GetItemCount()-1)
				s+="\n";
			else
			{
				if(mode==F_EXCEL)
					s+="\t";
				else if(mode==F_ASCII)
					s+="\t\t";
				else if(mode==F_COMMA)
					s+=",";
				else
					s+=" ";
			}
		}
		array.Add(s);
	}	
}

//*********************************************************************************************
// olga 2020.03.14 XIT-1927 - The utility converts selected items (64-bit) into CPtrArray of rows 
//*********************************************************************************************
BOOL FancyList::GetSelectedRows(CPtrArray& rows) const 
{
	rows.RemoveAll();
	POSITION pos = GetFirstSelectedItemPosition();
	if (pos == NULL)
		return FALSE; // no selection
	else
	{
		while (pos)
		{
			int nItem = GetNextSelectedItem(pos);

			DWORD_PTR rowid = GetItemData(nItem); // olga XIT-1927 - the rowid is 64-bit value (for example, a pointer to a translated pair)
			BOOL dup = FALSE;
			for (int k = 0; k < rows.GetSize(); k++)
				if ((DWORD_PTR)rows[k] == rowid)
				{
					dup = TRUE;
					break;
				}
			if (!dup)
				rows.Add((void*)rowid); 
		}
	}
	return TRUE;
}

BOOL FancyList::GetSelectedRows(CDWordArray& rows) const // convert selected items into rows... false if no selections.
{
	rows.RemoveAll();
	POSITION pos = GetFirstSelectedItemPosition();
	if (pos == NULL)
		return FALSE;
	else
	{
	  while (pos)
	  {
		  int nItem = GetNextSelectedItem(pos);   
		  
		  DWORD_PTR rowid = GetItemData(nItem); // olga 2020.03.14 XIT-1927 - beware that 64-bit rowid value will be stored in DWORD (32-bit) variable below
												// Use utility above with CPtrArray instead
		  BOOL dup = FALSE;
		  for (int k=0;k<rows.GetSize();k++)
				if (rows[k]==rowid)
				{
					dup=TRUE;
					break;
				}
			if (!dup)
				rows.Add(rowid);
	  }
	}	
	return TRUE;
}
void FancyList::SetSelectedRows(const CDWordArray& rows)
{
	SetRedraw(FALSE);
	for (int i=0;i<GetItemCount();i++)
	{
		SetItemState(i, FALSE, LVIS_SELECTED);
	}
	for (int i=0;i<rows.GetSize();i++)
	{
		int itm = GetItemFromRow(rows[i]);
		if (itm < 0)
			continue;
		SetItemState(itm, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
	}
	SetRedraw(TRUE);
}

void FancyList::SetAllSelections(BOOL select)
{
	SetRedraw(FALSE);
	for (int i=0;i<GetItemCount();i++)
	{
		SetItemState(i, (select)?LVIS_SELECTED:FALSE, LVIS_SELECTED);
	}
	SetRedraw(TRUE);
}

//***************************************************************
// olga 2020.03.14 XIT-1927 - Get 64-bit selection from the list.
// For example, a 64-bit pointer to a translated pair
//***************************************************************
bool FancyList::GetSingleSelectionAsPtr(DWORD_PTR& sel)
{
	CPtrArray sels; // 64-bit
	if (!GetSelectedRows(sels))
		return false;
	sel = (DWORD_PTR)sels[0];
	return true;
}

bool FancyList::GetSingleSelection(DWORD_PTR& sel)
{
	CDWordArray sels; // 32-bit
	if (!GetSelectedRows(sels))
		return false;
	sel = sels[0];
	return true;
}
void FancyList::SetSingleSelection(const DWORD_PTR sel)
{
	CDWordArray sels;
	sels.Add(sel);
	SetSelectedRows(sels);
}

void FancyList::StoreSingleSelection()
{
	// jmc - call before deleting all items to regenerate a list
	hasStoredSelection=false;
	storedSelection = 0;
	if (GetSingleSelection(storedSelection))
		hasStoredSelection = true;
}
void FancyList::RestoreSingleSelection()
{
	// jmc - call after regenerating a list to restore selection.
	if (hasStoredSelection)
	{
		SetSingleSelection( storedSelection );
		EnsureRowVisible( storedSelection );
	}
}
void FancyList::DeleteAllColumns()
{
	int nColumnCount = GetHeaderCtrl()->GetItemCount();
	for (int i=0;i < nColumnCount;i++)
		DeleteColumn(0);
}

// sws - Mantis 0007245 (added delimit argument)
BOOL FancyList::GetWholeItemText(const int item, CString& text, bool delimit) const 
{
	FancyList& this_non_const = const_cast <FancyList&> (*this);  // because we know we are "const"

	for (int j=0;j< this_non_const.GetHeaderCtrl()->GetItemCount();j++)
	{
      bool lastItem = (j == (this_non_const.GetHeaderCtrl()->GetItemCount() - 1));  // sws - Mantis 0007802 (undesired trailing comma)

      CString t(GetItemText(item,j));
      if (delimit  && !lastItem)               // sws - Mantis 0007245 (added delimit argument)
         text += t + _T(", ");
      else
         text += t + _T(" ");
	}
	return TRUE;
}

void FancyList::OnEndlabeledit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// the label is done editing!!
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;

	CString m;
	//	m = list.GetItemText(pDispInfo->item.iItem,0);
	if (pDispInfo->item.pszText==NULL)
		return;
	m = pDispInfo->item.pszText;
	DWORD_PTR index = GetItemData(pDispInfo->item.iItem);

	OnLabelEdited(index,m);

	*pResult = 0;
}

void FancyList::OnNMRclick(NMHDR *pNMHDR, LRESULT *pResult)
{
	DWORD_PTR row, col;
	CPoint dialogPt;

	*pResult = 0;

	if (!GetSelectedRowCol(dialogPt, row, col))
		return;
	// Call virtual function 
	OnRClick(dialogPt, row, col);

}

bool FancyList::GetSelectedRowCol(CPoint& dialogPt_out, DWORD_PTR& row_out, DWORD_PTR& col_out)
{
	DWORD dwpos = GetMessagePos();
	dialogPt_out.x = GET_X_LPARAM(dwpos);
	dialogPt_out.y = GET_Y_LPARAM(dwpos);
	CPoint mappt = dialogPt_out;
	::MapWindowPoints(HWND_DESKTOP, this->m_hWnd, &mappt, 1);

	LVHITTESTINFO lvhti;
	lvhti.pt = mappt;
	if (SubItemHitTest(&lvhti)<0)
		return false;

	if (lvhti.iItem<0 || lvhti.iSubItem<0)
		return false;

	row_out = (DWORD_PTR) GetItemData(lvhti.iItem);		// jmc - the row needs to be based on the ITEMDATA -- not current order in list. (this matches other fx in class)
	col_out = (DWORD_PTR) lvhti.iSubItem;

	return true;
}
