#pragma once
// FancyList.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// FancyList window

class FancyList : public CListCtrl
{
protected:		// OVERRIDES
	// whenever a ROW is given in an override, it is not the actual row in the list control, but
	// instead the value of the ItemData LPARAM (SetItemData) for the row.  This way if the list
	// is sorted, you still get a reference to the actual original data structure (if you set the
	// item data when building.)

	// you must override this function to provide custom highlighting and bolding.
	// You are required to keep your own database of items and respond to questions when
	// asked.
	virtual BOOL GetItemCustomDrawSpecs
		(const DWORD_PTR row, const int col,COLORREF* pTextColor, COLORREF* pBackColor, CFont** pFont)
	{
		*pTextColor = RGB(0,0,0);
		*pBackColor = RGB(255,255,255);
		*pFont = NULL;
		return TRUE;
	}
	// provide a tooltip based on your database.....
	virtual BOOL GetToolTip(const DWORD_PTR row, const int col, CString& tip)
		{return FALSE;}
	// provide a sort order from your database.
	virtual int SortCompareOperator(const int sortcol,const DWORD_PTR row1, const DWORD_PTR row2, const BOOL Ascending )
		{return 0;}

	virtual void StartedSorting() {}
	virtual void FinishedSorting() {}

	// override this to handle when the checked state on an item changes.
	virtual void OnCheckChange(const DWORD_PTR row, const BOOL checked)
	{
		// NOTE:  You may get this event when you are populating the list! So, if
		// this handler causes a redraw, either message isolate it, or put an interlock
		// in the handler so it won't process while drawing.
	}
	// override to handle edited labels (only column 0.)
	virtual void OnLabelEdited(const DWORD_PTR row, const CString& text)	{}

	// jmc - override to know when selection changes.
	virtual void OnSelectionChanged() {}

	// override to handle right clicks
	virtual void OnRClick(const CPoint& dialogPt_in, const DWORD_PTR&row_in, const DWORD_PTR& col_in){}

// Construction
public:
	FancyList();
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(FancyList)
	//}}AFX_VIRTUAL

	// search the item dats for a database row index... then return the current list item that contains it.
	int GetItemFromRow( const DWORD_PTR row ) const;
	BOOL GetColumnText(const int i,CString& str);
	bool EnsureRowVisible( const DWORD_PTR row );

	BOOL GetSelectedRows(CPtrArray& rows) const;  // olga XIT-1927 - fixed SA crash caused by attempt to store a 64-bit pointer into 32-bit variable.
	bool GetSingleSelectionAsPtr(DWORD_PTR& sel); // olga XIT-1927 - Get 64-bit selection from the list

	BOOL GetSelectedRows(CDWordArray& rows) const; // convert selected items into rows... false if no selections.
	void SetSelectedRows(const CDWordArray& rows);
	void SetAllSelections(BOOL select = TRUE);
	bool GetSingleSelection(DWORD_PTR& sel);
	void SetSingleSelection(const DWORD_PTR sel);

	void StoreSingleSelection();		// jmc - so we can repopulate list and restore the selection.
	void RestoreSingleSelection();	// called after repopulating list.  NOTE - these work on the Row (not the list item)

	void DeleteAllColumns();		// jmc - for cases where we reset columns during a draw.

	BOOL GetWholeItemText(const int item, CString& text, bool delimit = false) const;  // sws - Mantis 0007245 (added delimit argument)

	// if you want to output the contents of the listbox, this function gets it in formatted string format:
	typedef enum
	{
		F_ASCII=0,
		F_EXCEL,
		F_COMMA,
		F_SPACE
	} ListFmt;
	void GetAsStringArray(ListFmt format, CStringArray& array);

	void SetSortParameters(int col, BOOL Ascending);
	void Sort();			// sorts using the last set of parameters.

	bool GetSelectedRowCol(CPoint& dialogPt_out, DWORD_PTR& row_out, DWORD_PTR& col_out);

// Implementation
public:
	virtual ~FancyList();

	// Generated message map functions
protected:
	//{{AFX_MSG(FancyList)
	//}}AFX_MSG
	afx_msg void OnCustomdraw(NMHDR*, LRESULT*);
	afx_msg void OnColumnClick(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnItemChanged(NMHDR* pNMHDR, LRESULT* pResult) ;
	afx_msg void OnEndlabeledit(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnNMRclick(NMHDR *pNMHDR, LRESULT *pResult);
	
	//Respondes to the TTN_NEEDTEXT* messages, provides the text of a tooltip
	virtual afx_msg BOOL OnToolTipText( UINT id, NMHDR * pNMHDR, LRESULT * pResult ); 

	//Provide our own logic for HitTests, specifically, make ToolHitTests respond per SubItem
	virtual INT_PTR OnToolHitTest(CPoint point, TOOLINFO * pTI) const;

	DECLARE_MESSAGE_MAP()
private:
	int m_iSortColumn;
	BOOL m_bSortAscending;
 	// provide a compare callback.. this will thunk to a more user-friendly compare function
	// which will be overridden by the derived class.
	static int CALLBACK CompareFunction( LPARAM lParam1, LPARAM lParam2, LPARAM lParamData );
	void Sort( int iColumn, BOOL bAscending );

	bool hasStoredSelection;
	DWORD_PTR storedSelection;

};

