// ForceFitStatic.cpp : implementation file
//

#include "stdafx.h"
#include "ForceFitStatic.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//
// CForceFitStatic
//
// NOTE: To use this class, make sure that the NOTIFY property of the dialog control
//       that this will be used with is set to TRUE.
//
// Also, when using a dialog that also uses a CString variable with the CStatic, you'll
//   need to remember the following:
//
// 1) MUST have DDX_Text before the DDX_Control
//  
//  DDX_Text(pDX, IDC_MYSTATIC, m_staticStr);
//  
//  DDX_Control(pDX, IDC_MYSTATIC, m_staticCtrl);
//  
// 2) If you're going to set the CString associated with a control in OnInitDialog, 
//    the initialization of that string MUST preceed the call to CDialog::OnInitDialog
//  
// BOOL DTargetInfo::OnInitDialog() 
// {
//  m_staticStr = _T("Hello");
//  CDialog::OnInitDialog();
//
CForceFitStatic::CForceFitStatic()
{
	m_bTransparentBk  = TRUE;
	m_rgbText		  = ::GetSysColor(COLOR_WINDOWTEXT);
	m_rgbBkgnd		  = ::GetSysColor(COLOR_BTNFACE);
	m_indicatorText   = _T(" [...] ");
	m_strText         = _T("");
}

CForceFitStatic::~CForceFitStatic()
{
}


BEGIN_MESSAGE_MAP(CForceFitStatic, CMouseAction)
	ON_WM_PAINT()
END_MESSAGE_MAP()

void CForceFitStatic::PreSubclassWindow()
{
	this->GetWindowText(m_strText);
	SetToolTipText(m_strText);
}

// CForceFitStatic message handlers

void CForceFitStatic::OnPaint()
{
	//PAINTSTRUCT ps;
	//CDC& dc = *BeginPaint(&ps);
	CPaintDC	dc(this); // device context for painting
	CBitmap		bmp;
	CBitmap*	pOldBitmap = NULL;

	// GET Client area
	CRect rc;
	GetClientRect(rc);

	// TEXT PROPERTIES
	CFont*   pOldFont = dc.SelectObject( GetFont() );
	COLORREF rgbText  = dc.SetTextColor( m_rgbText );
	COLORREF rgbBkgnd = dc.SetBkColor( m_rgbBkgnd );

	// Erase the background this way, otherwise droppings seem to appear!
	CBrush brush(m_rgbBkgnd);
	dc.FillRect(&rc, &brush);

	if ( ! m_strText.IsEmpty() )
		DrawText(&dc, &rc, m_strText);

	// Restore DC's State
	dc.SelectObject(pOldFont);
	dc.SetTextColor(rgbText);
	dc.SetBkColor(rgbBkgnd);
	//EndPaint(&ps);
}



void CForceFitStatic::DrawText(CDC* pDC, CRect* pRect, const CString& csText)
{
	DWORD dwStyle = GetStyle();
	DWORD dwFlags = 0;

	// Map "Static Styles" to "Text Styles" - WARNING MACROS
#define MAP_STYLE(src, dest) if(dwStyle & (src)) dwFlags |= (dest)
#define NMAP_STYLE(src, dest) if(!(dwStyle & (src))) dwFlags |= (dest)

	MAP_STYLE(	SS_RIGHT,			DT_RIGHT   					);
	MAP_STYLE(	SS_CENTER,			DT_CENTER					);
	MAP_STYLE(	SS_LEFT,			DT_LEFT					    );
	MAP_STYLE(	SS_NOPREFIX,		DT_NOPREFIX					);
	MAP_STYLE(	SS_WORDELLIPSIS,	DT_WORD_ELLIPSIS			);
	MAP_STYLE(	SS_ENDELLIPSIS,		DT_END_ELLIPSIS				);
	MAP_STYLE(	SS_PATHELLIPSIS,	DT_PATH_ELLIPSIS			);


	// TAb expansion
	if (csText.Find( _T('\t') ) != -1)
		dwFlags |= DT_EXPANDTABS;

	CSize		sz;
	::GetTextExtentPoint32(pDC->GetSafeHdc(), csText,csText.GetLength(), &sz);

	CString forcedText;

	// Text doesn't fit
	if (sz.cx > pRect->Width())
	{
		CString localString = csText;
		int indicatorLen = m_indicatorText.GetLength();
		CString prevForcedText;
		CString head, tail;
		bool done = false;

		while (!done)
		{
			// Take a character from the front and the end...
			//  continue to do so until the string fits.
			head = localString.Left(head.GetLength()+1);
			tail = localString.Right(tail.GetLength()+1);
			forcedText = head + m_indicatorText + tail;
			::GetTextExtentPoint32(pDC->GetSafeHdc(), forcedText, forcedText.GetLength(), &sz);
			if ( sz.cx < pRect->Width() ){// We found how many letters fits
				prevForcedText = forcedText;
			}
			else 
			{
				forcedText = prevForcedText;
				done = true;
			}
		}
		// If nothing was set, then widget working area is to small to support
		//  a truncated string... just set it to the complete original string.
		if (forcedText.IsEmpty())
		{
			forcedText = csText;
		}
	}
	else
	{ // No Need For Wrapping 
		forcedText = csText;
	}

	pDC->DrawText(forcedText, pRect, dwFlags);

}

void CForceFitStatic::SetWindowText(const CString& strText)
{
	m_strText = strText;
	SetToolTipText(m_strText);
	RedrawWindow();
}

void CForceFitStatic::SetIndicatorText(const CString& indicatorText)
{
	m_indicatorText = indicatorText;
	RedrawWindow();
}

BOOL CForceFitStatic::RedrawWindow()
{
	InvalidateRect(NULL);	

	return TRUE;
}

void CForceFitStatic::SetDefaultBkColor()
{
	SetBkColor(::GetSysColor(COLOR_BTNFACE));
}
void CForceFitStatic::SetDefaultTextColor()
{
	SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));
}

void CForceFitStatic::SetBkColor(COLORREF crColor)
{
	if (crColor == m_rgbBkgnd)
		return;		// already set.
	m_rgbBkgnd = crColor; // Passing the value passed by the dialog to the member varaible for Backgound Color
	RedrawWindow();

}

void CForceFitStatic::SetTextColor(COLORREF crColor)
{
	if (crColor == m_rgbText)
		return;
	m_rgbText = crColor; // Passing the value passed by the dialog to the member varaible for Text Color
	RedrawWindow();
}

COLORREF CForceFitStatic::GetTextColor()
{
	return m_rgbText;
}

COLORREF CForceFitStatic::GetBkColor()
{
	return m_rgbBkgnd;
}
