#pragma once

template <class T>		
class NRKObArray
{
	// JMC - 2003.07.02
	// usage:
	//		NRKObArray<Blob> blbs;
   //		int idx = blbs.Add(*new Circle());
	//		int size = blbs.GetSize();
	//		blbs[idx].DrawMe = true;.....
	// also -- you can dual derive from this and cobject to extend wrapper functionality:
	//		class MyWrapper : public CObject, public NRKObArrak<MyObj>
	//
	// REQUIREMENTS for <T>:  
	//			T::T(const T& other);			// copy constructor required for all the "Copy" functions.
	//			DECLARE_DYNCREATE / IMPLEMENT_DYNCREATE (or serial) required for copy operations
	//			bool T::operator==(const T& other) const;	// == operator required for Find functions (and remove duplicates).
public:
	NRKObArray(void) {}
	virtual ~NRKObArray(void);
	NRKObArray(const NRKObArray<T>& source);
	void operator=(const NRKObArray<T>& source);

	T& operator[](DWORD_PTR i) const;
	T& Get(DWORD_PTR i) const;
	T& GetLastElement() const;		// eliminates the need for .GetUpperBound each time.
	DWORD_PTR GetSize() const;
	DWORD_PTR GetUpperBound() const; // returns "InvalidIndex()" if getsize<1
	bool Find(const T& obj) const;		// calls objects == operator.
	bool Find(const T& obj, DWORD_PTR& index) const;	// calls objects == operator.
	bool FindReference(const T& obj) const;		// checks for pointer match
	bool FindReference(const T& obj, DWORD_PTR& index) const;	// checks for pointer match.
	bool FindReferenceAndRemove(const T& obj);	// checks for pointer match - returns true if it was found and removed.
	bool FindReferenceAndDelete(const T& obj);	// checks for pointer match - returns true if it was found and DELETED.

	bool IsIndexValid(DWORD_PTR i) const;	// if you need to bounds check your index, call this BEFORE Get or []

	DWORD_PTR Add(const T& newHeapObj);
	DWORD_PTR AddCopy(const T& sourceObj);	// will copy construct new item.
	void InsertAt(DWORD_PTR idx, const T& newHeapObj);
	void InsertCopyAt(DWORD_PTR idx, const T& sourceObj);
	void RemoveAt(DWORD_PTR idx, DWORD_PTR nCount=1);
	void SetAt(DWORD_PTR idx, const T& heapObj);
	void SetCopyAt(DWORD_PTR idx, const T& heapObj);
	void SetAtGrow(DWORD_PTR idx, const T& heapObj);

	void SwapElements(DWORD_PTR idxA, DWORD_PTR idxB);

	void DeleteAndRemoveAt(DWORD_PTR idx);	// delete [idx] then remove.
	void DeleteAll();		// delete and remove all items
	void RemoveAllWithoutDeleting();		// wipe out all pointers -- don't delete objects.
	void SetSize(DWORD_PTR size, INT_PTR growBy = -1);		// thunks through to obarray function - doesn't delete object memory.
	DWORD_PTR DeleteAndRemoveDuplicates();		// uses find() to determine match.  returns number removed.

	void RemoveMatchingItems(const NRKObArray<T>& matchList);

	void Append(const NRKObArray<T>& source);	// just append the pointers...
	void AppendCopy(const NRKObArray<T>& source);	// append by copy-constructing all objects.

	static DWORD_PTR InvalidValue()		// invalid index value.
	{	return ((DWORD_PTR)(-1)); }

	virtual void SerializeObArray(CArchive& ar);  // not standard "serialize" because this template is not cobject derived.

	void HandOffElements(NRKObArray<T>& destination);

	T* MakeCopyOfObject(const T& sourceObj);  // jmc -- so insert, add, and setat copy fx have common copier.
	//
	// Method to allow direct access to array memory...  added to allow array pointer to be used by 
	//  QSort.
	//
	CObject** GetData() { return array.GetData(); }

private:
	CTypedPtrArray <CObArray, T*> array;	// private so we can restructure connection to CTypedPtrArray easily...

};

////// implementation of the class --- would be in CPP except that this is a template!
template <class T>
NRKObArray<T>::~NRKObArray(void)
{
	DeleteAll();
}

template <class T>
T& NRKObArray<T>::operator[](DWORD_PTR i) const
{
	return *array[i];
}
template <class T>
NRKObArray<T>::NRKObArray(const NRKObArray<T>& source)
{
	operator=(source);
}
template <class T>
void NRKObArray<T>::operator=(const NRKObArray<T>& source)
{
	DeleteAll();
	AppendCopy(source);
}
template <class T>
T& NRKObArray<T>::Get(DWORD_PTR i) const
{
	return *array[i];
}
template <class T>
T& NRKObArray<T>::GetLastElement() const
{
	ASSERT(array.GetUpperBound()>=0);
	return *array[array.GetUpperBound()];
}
template <class T>
bool NRKObArray<T>::IsIndexValid(DWORD_PTR i) const
{
	if (GetSize()<1)		// prevents problem with get upper bound returning invalid valid.
		return false;
	return (i!=InvalidValue() && i>=0 && i<=GetUpperBound());
}
template <class T>
bool NRKObArray<T>::Find(const T& obj) const
{
	DWORD_PTR index=0;
	return Find(obj,index);
}
template <class T>
bool NRKObArray<T>::Find(const T& obj, DWORD_PTR& index) const
{
	for (DWORD_PTR i=0;i<GetSize();i++)
		if (obj==Get(i))
		{
			index = i;
			return true;
		}
	return false;
}
template <class T>
bool NRKObArray<T>::FindReference(const T& obj) const
{
	DWORD_PTR index=0;
	return FindReference(obj,index);
}
template <class T>
bool NRKObArray<T>::FindReferenceAndRemove(const T& obj)
{
	DWORD_PTR index=0;
	if (FindReference(obj,index))
	{
		RemoveAt(index);
		return true;
	}
	return false;
}
template <class T>
bool NRKObArray<T>::FindReferenceAndDelete(const T& obj)
{
	if (FindReferenceAndRemove(obj))
	{
		delete &obj;
		return true;
	}
	return false;
}

template <class T>
bool NRKObArray<T>::FindReference(const T& obj, DWORD_PTR& index) const
{
	for (DWORD_PTR i=0;i<GetSize();i++)
		if (&obj==&Get(i))
		{
			index = i;
			return true;
		}
		return false;
}

template <class T>
DWORD_PTR NRKObArray<T>::Add(const T& newHeapObj)
{
	return array.Add(& const_cast<T&>(newHeapObj) );
}
template <class T>
DWORD_PTR NRKObArray<T>::AddCopy(const T& sourceObj)	// will copy construct new item.
{
	return array.Add( MakeCopyOfObject(sourceObj) );
}

template <class T>
T* NRKObArray<T>::MakeCopyOfObject(const T& sourceObj)
{
	T* newObj = (T *) sourceObj.GetRuntimeClass()->CreateObject();
	*newObj = sourceObj;
	return newObj;
}


template <class T>
void NRKObArray<T>::InsertAt( DWORD_PTR idx,const T& newHeapObj)
{
	array.InsertAt(idx, & const_cast<T&>(newHeapObj) );
}
template <class T>
void NRKObArray<T>::InsertCopyAt( DWORD_PTR idx,const T& sourceObj)
{
	array.InsertAt(idx, MakeCopyOfObject(sourceObj) );
}
template <class T>
void NRKObArray<T>::SetAt(DWORD_PTR idx, const T& heapObj)
{
	array.SetAt(idx, & const_cast<T&>(heapObj) );
}
template <class T>
void NRKObArray<T>::SetCopyAt(DWORD_PTR idx, const T& sourceObj)
{
	array.SetAt(idx, MakeCopyOfObject(sourceObj) );
}
template <class T>
void NRKObArray<T>::SetAtGrow(DWORD_PTR idx, const T& heapObj)
{
	array.SetAtGrow(idx, & const_cast<T&>(heapObj) );
}
template <class T>
void NRKObArray<T>::RemoveAt(DWORD_PTR idx, DWORD_PTR nCount /*= 1*/)
{
	array.RemoveAt(idx, nCount);
}

template <class T>
DWORD_PTR NRKObArray<T>::GetSize() const
{
	return array.GetSize();
}
template <class T>
DWORD_PTR NRKObArray<T>::GetUpperBound() const
{
	return array.GetUpperBound();
}
template <class T>
void NRKObArray<T>::DeleteAndRemoveAt(DWORD_PTR idx)
{
	delete &Get(idx);
	array.RemoveAt(idx);
}
template <class T>
void NRKObArray<T>::DeleteAll()
{
#ifdef _DEBUG
	//
	// AFP - NOTE: Logic added to ensure that during debug builds the class contained in the array 
	//  is properly defined.  Due to using forward declarations for speedy compiles, it appears that
	//  the true class defn is often lost.  This will ensure that everything is defined.
	//
	if (array.GetSize())
	{
		T* pItem = array.GetAt(0);
		if (pItem)
		{
			CRuntimeClass *pRuntime = pItem->GetRuntimeClass();
			if (pRuntime && pRuntime->m_pfnCreateObject)
			{
				T* pTest = (T *) pRuntime->CreateObject();
				delete pTest;
			}
		}
	}
#endif

	for (INT_PTR i=0;i<array.GetSize();i++)
	{
		T* pItem = array.GetAt(i);
		delete pItem;
	}
	array.RemoveAll();
}
template <class T>
void NRKObArray<T>::RemoveAllWithoutDeleting()
{
	array.RemoveAll();
}
template <class T>
DWORD_PTR NRKObArray<T>::DeleteAndRemoveDuplicates()
{
	if (GetSize()<2)
		return 0;
	DWORD_PTR count = 0;
	for (DWORD_PTR i=0;i<GetSize()-1;i++)
	{
		T& obj = Get(i);
		for (DWORD_PTR j=i+1; j<GetSize();j++)
		{
			if (obj==Get(j))
			{
				DeleteAndRemoveAt(j);
				count++;
				if (j>0)
					j--;
				else
					break;
			}
		}
	}
	return count;
}
template <class T>
void NRKObArray<T>::SetSize(DWORD_PTR size, INT_PTR growBy /*=-1*/)
{
	array.SetSize(size, growBy);
}
template <class T>
void NRKObArray<T>::SerializeObArray(CArchive& ar)
{		
	// no special "schema" or anything since this template class is NOT
	// Cobject derived.
	if (ar.IsLoading())
		DeleteAll();		// jmc 2010.06.17 -  added so loading array starts clean
	array.Serialize(ar);
}

template <class T>
void NRKObArray<T>::HandOffElements(NRKObArray<T>& destination)
{
	// transfer pointers and clean out the source array.
	// don't assume dest is empty... just append.
	for (DWORD_PTR i=0;i<GetSize();i++)
	{
		destination.Add(Get(i));
	}
	array.RemoveAll();
}

template <class T>
void NRKObArray<T>::Append(const NRKObArray<T>& source)	// just append the pointers...
{
	for (DWORD_PTR i=0;i<source.GetSize();i++)
		Add(source[i]);
}
template <class T>
void NRKObArray<T>::AppendCopy(const NRKObArray<T>& source)	// append by copy-constructing all objects.
{
	for (DWORD_PTR i=0;i<source.GetSize();i++)
		AddCopy(source[i]);
}
template <class T>
void NRKObArray<T>::SwapElements(DWORD_PTR idxA, DWORD_PTR idxB)
{
	T* pT=array[idxA];
	array.SetAt(idxA, array[idxB]);
	array.SetAt(idxB, pT);
}

template <class T>
void NRKObArray<T>::RemoveMatchingItems(const NRKObArray<T>& matchList)
{
	for (DWORD_PTR i=0;i<matchList.GetSize();i++)
	{
		T& searchItem = matchList[i];
		for (DWORD_PTR j=0;j<array.GetSize();)
		{
			T& curObj = Get(j);
			if (curObj == searchItem)
			{
				DeleteAndRemoveAt(j);
			}
			else
			{
				j++;
			}
		}
	}
}