// StringHelper.cpp: implementation of the StringHelper class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "StringHelper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

StringHelper::StringHelper()
{

}

StringHelper::~StringHelper()
{

}
BOOL StringHelper::GetLastErrorTrace()
{
#ifdef _DEBUG
	CString Str;
	int Err = 0;
	Err = GetLastError();
	if(!GetLastErrorString(Str))
		return FALSE;
	TRACE(_T("Error %d-%s\n"),Err,Str);
#endif
	return TRUE;
}
BOOL StringHelper::GetLastErrorString(CString& Str)
{
	BOOL ret = FALSE;
	DWORD error = GetLastError();
	if(error)
	{
		LPVOID lpMsgBuf;
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			error,
			0, // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
				);

		if(lpMsgBuf)
		{
			Str = (LPCTSTR)lpMsgBuf;
			ret = TRUE;
		}
		else
		{
			TRACE1("Error message 0x%lx not found",error);
		}
		LocalFree( lpMsgBuf );
	}
	return ret;
}
BOOL StringHelper::GetStringBetween(const CString& InStr,const CString& s1, const CString& s2,CString& OutStr, BOOL Backwards,int* plastindex)
{

	//Forard gets the first set of chars and looks for the next second set
	//Backwards finds the first occurance of the second set and reverse searchs from there to find the first set of chars
	//Whew! 
	if(!Backwards)
	{
		int begin = InStr.Find(s1);
		if(begin==InStr.GetLength())
			return FALSE;
		int end = InStr.Find(s2,begin+1);
		if(begin<0||end<0)
		{
			return FALSE;
		}
		begin = begin+s1.GetLength();
		if(end-begin<1)
			return FALSE;
		OutStr = InStr.Mid(begin,end-begin);
		if(plastindex)
			*plastindex = end;
		return TRUE;
	}
	else
	{
		int end = InStr.Find(s2);
		if(end<=0)
			return FALSE;
		int begin = 0;
		int lastbegin = -1;
		while(begin<end)
		{
			//special oneshot
			begin = InStr.Find(s1,lastbegin+1);
			if(begin<0)
				break;
			if(begin>=end)
				break;
			lastbegin = begin;
		}
		if(lastbegin < 0)
			return FALSE;
		begin = lastbegin+s1.GetLength();
		if(end-begin<1)
			return FALSE;
		OutStr = InStr.Mid(begin,end-begin);
		if(plastindex)
			*plastindex = end;
		return TRUE;

	}
}
BOOL StringHelper::RemoveStringBetweenIncludingBounds(const CString& InStr,const CString& s1, const CString& s2,CString& OutStr,int& index)
{
	//This does forward searches only
	int begin = InStr.Find(s1);
	if(begin==InStr.GetLength())
		return FALSE;
	int end = InStr.Find(s2,begin+1);
	if(begin<0||end<0)
	{
		return FALSE;
	}
	CString Composite1,Composite2;
	Composite2 = InStr.Right(InStr.GetLength()-end-1);
	Composite1 = InStr.Left(begin);
	OutStr = Composite1 + Composite2;
	index = begin;
	return TRUE;
}

BOOL StringHelper::SplitString(const CString& InStr,const CString& split, CString& OutStr1,CString& OutStr2)
{
	int sp = InStr.Find(split);
	if(sp<0)
		return FALSE;
	OutStr1 = InStr.Mid(0,sp);
	OutStr2 = InStr.Mid(sp+split.GetLength());
	return TRUE;
}
BOOL StringHelper::IsAsciiString(const LPCTSTR str) // olga XIT-1696 - added validation for English key in translation map
{
	if (!str) return FALSE;
	int l = lstrlen(str);
	for (int i = 0; i < l; i++)
	{
		if ((str[i] > 127))
			return FALSE;
	}
	return TRUE;
}
void StringHelper::Tokenize(const LPCTSTR inStr, LPCTSTR inSeparatorChars, CStringArray& outTokens)
{
	ASSERT(inStr != NULL);
	if (inStr == NULL)
		return;

	ASSERT(inSeparatorChars != NULL);
	if (inSeparatorChars == NULL)
		return;

	CString in(inStr);

	outTokens.RemoveAll();

	int curPos = 0;
	CString nextToken = in.Tokenize(inSeparatorChars,curPos);
	while (nextToken != _T(""))
	{
		outTokens.Add(nextToken);
		nextToken = in.Tokenize(inSeparatorChars,curPos);
	}
}

BOOL StringHelper::DissectPointName(const CString& InStr, CString& CN, CString& GN, CString& TN)
{
	CString GNTN;
	if(SplitString(InStr,CString("::"),CN,GNTN))
	{
		CString cn2;
		StripSpaces(CN,cn2.GetBuffer(300));
		cn2.ReleaseBuffer();
		if(cn2.IsEmpty()) //allowed
			CN = cn2;
		//	return FALSE;
	}
	else 
		return FALSE;
	if(SplitString(GNTN,CString("::"),GN,TN))
	{
		CString tn2,gn2;
		StripSpaces(GN,gn2.GetBuffer(300));
		StripSpaces(TN,tn2.GetBuffer(300));
		gn2.ReleaseBuffer();
		tn2.ReleaseBuffer();
		if(gn2.IsEmpty())
			return FALSE;
		if(tn2.IsEmpty())
			return FALSE;
		return TRUE;
	}
	return FALSE;
}
BOOL StringHelper::DissectPointName(const CString& InStr,  CString& GN, CString& TN)
{
	if(SplitString(InStr,CString("::"),GN,TN))
	{
		CString tn2,gn2;
		StripSpaces(GN,gn2.GetBuffer(300));
		StripSpaces(TN,tn2.GetBuffer(300));
		gn2.ReleaseBuffer();
		tn2.ReleaseBuffer();
		if(gn2.IsEmpty())
			return FALSE;
		if(tn2.IsEmpty())
			return FALSE;
		return TRUE;
	}
	return FALSE;
}

// splits coll::group or coll:group into coll and group strings, where coll is optional
BOOL StringHelper::DissectOptionalCollectionAndGroupName(LPCTSTR inStr, CString& CN, CString& GN)
{
	CN = _T("");
	GN = _T("");

	ASSERT(inStr != NULL);
	if (inStr == NULL)
		return FALSE;

	CString s(inStr);

	int sepIdx1 = s.Find(_T(':'));
	int sepIdx2 = s.ReverseFind(_T(':'));

	if ((sepIdx1 >= 0) && (sepIdx2 >= 0))
	{
		CN = s.Left(sepIdx1);
		GN = s.Right(s.GetLength() - sepIdx2 - 1);
	}
	else
		GN = s;

	return TRUE;
}

void StringHelper::AssembleFullTargetName(const CString& col, const CString& grp, const CString& tgt, CString& composite)
{
	// jmc 2013.08.08 - easy, but nice to centralize.
	if (StringHelper::IsBlank(col))
		composite.Format(_T("%s::%s"), grp, tgt);		// because you can have a blank collection
	else
		composite.Format(_T("%s::%s::%s"), col, grp, tgt);
}

void StringHelper::MakeUpperAndRemoveSpaces(CString& txt)
{
	// jmc - if you want to make a CMapString... and you want it to act like NameCompare
	// which is space and case insensitive, use this fx before popping the string in the map.
	// Also be sure to use it on the string you pass in to search the map. :)
	txt.MakeUpper();
	txt.Remove(_T(' '));
}

BOOL StringHelper::NameCompare(LPCTSTR ps1, LPCTSTR ps2)
{
	// modified 9-5-2000 by jmc -- made it search backwards and not copy any memory.
	int i = _tcslen(ps1)-1;
	int j = _tcslen(ps2)-1;
	BOOL have1 = TRUE;
	BOOL have2 = TRUE;

	while(have1||have2)
	{
		have1 = have2 = FALSE;
		for(;i>=0;i--)   //get next non space char of s1
		{
			if(ps1[i] != ' ')
			{
				have1 = TRUE;
				i--;
				break;
			}
		}

		for(;j>=0;j--)   //get next non space char of s2
		{
			if(ps2[j] != ' ')
			{
				have2 = TRUE;
				j--;
				break;
			}
		}
			
		if(have1 != have2)
			return FALSE;

		if (!have1 && !have2)
			return TRUE;

		if((have1)&&(tolower(ps1[i+1]) != tolower(ps2[j+1])))
			return FALSE;

	}
	return TRUE;
}

void StringHelper::GetCurrentTimeString( CString& s )
{
	TCHAR buf[20];
	_tstrdate( buf );
	s = buf;
	_tstrtime( buf );
	s += "  ";
	s += buf;
}

void StringHelper::GetTimeIntervalString( double time, CString& s )
{
	double remain = time;

	int days = (int) (remain/60.0/60.0/24.0);
	remain -= days*24*60*60;
	int hrs = (int)(remain/60.0/60.0);
	remain -= hrs*60*60;

	int mins = (int)(remain/60.0);
	remain -= mins*60;

	if (days > 0)
		s.Format(_T("%d days, %d hrs"), days, hrs);
	else if (hrs > 0)
		s.Format(_T("%d hrs, %d min"),hrs,mins);
	else if (mins > 0)
		s.Format(_T("%d min, %d sec"),mins,(int)remain);
	else
		s.Format(_T("%.1f sec"),remain);
}
//the following does extended checking for scalebar identifiers $0_0, $0_1
BOOL StringHelper::IncrementTargetName(CString &name, int inc) 
{
	if (inc==0)		// jmc- so we don't change the name.
		return TRUE;	

	if(name.Find(_T('$'))>=0)
	{
		CString Other,SBStr;
		if(SplitString(name,CString("$"),Other,SBStr))
		{
			CString barNum,targNum;
			if(SplitString(SBStr,CString("_"),barNum,targNum))
			{
				if(targNum == _T("0"))
				{
					targNum = _T("1");
				}
				else
				{
					targNum = _T("0");
					IncrementName(Other,inc);
				}

				//reassemble 
				CString NewName;
				NewName.Format(_T("%s$%s_%s"),Other,barNum,targNum);
				name = NewName;
				return TRUE;
			}
		}
	}
	return IncrementName(name,inc);
}

BOOL StringHelper::IncrementName(CString &name, int inc) 
{
	if (inc==0)		// jmc- so we don't change the name.
		return TRUE;	

	__int64 GNum = 0;
	int GDigits = 0;
	CString GName =_T("");
	CString GNumber =_T("");

	if(!DivideNumberedName(name, GName,GNum,GDigits))
		return FALSE;
	int nonDigitLoc = 0; 
	double endNum = 0.0; //to throw away
	if(GDigits<0)
	{
		GDigits =0;
	}
	GNum += inc;
	bool rollover = false;
	if(GNum<0)
	{
		rollover=true;
		GNum=0;
	}
	// reassemble the targetname
	GNumber.Format(_T("%0*I64d"),GDigits,GNum);
	//_i64tot(GNum,GNumber.GetBuffer(300),10);
	//if(GNum == 0)
	//	GNumber = "";
	//GNumber.ReleaseBuffer();
	name = GName;
	name += GNumber;
	if(rollover)
	{
		name+= _T(".1000");
	}
	return TRUE;
}
void StringHelper::IncrementAlpha(CString& str)
{
	if (StringHelper::IsBlank(str))
	{
		str = _T("A");
		return;
	}
	int lastCharIndex = str.GetLength()-1;
	// A = 65 dec, Z = 90 dec
	TCHAR ch = str[lastCharIndex];		// get the last character.
	if (ch < 65 || ch > 90)	// A-Z
		ch = 65;
	else if (ch == 90)		// Z
		str += _T("A");		// append an A
	else
		ch += 1;
	str.SetAt(lastCharIndex,ch);
}
BOOL StringHelper::DivideNumberedName(const CString& in, CString& namebase,__int64& num)
{
	if(in.IsEmpty())
		return FALSE;
	//search to find first non-number or '.' character from end
	int FirstDigit = 0;
	int CharCount = in.GetLength();		

	for(int i=0;i<CharCount;i++)
	{
	//	char c = in[CharCount-i-1];
        wint_t c = in[CharCount - i - 1];  // sws/olga - mantis #8216 (fixes wchar digit check in non-ASCII languages)

		if	(((!iswdigit(c))&&
				(c != '-'))||
				(c == '.'))
		{
			FirstDigit = CharCount-i;				   
			break;
		}		  
	}
	//disassemble the name
	CString sNumber;
	sNumber  = in.Right(CharCount-FirstDigit);
	namebase = in.Left(FirstDigit);

	num = _ttoi64(sNumber);

	return TRUE;
}

BOOL StringHelper::DivideNumberedName(const CString& in, CString& namebase,__int64& num,int& digits)
{
	if(in.IsEmpty())
		return FALSE;
	//search to find first non-number or '.' character from end
	int CharCount = in.GetLength();		
	int FirstDigit = 0;
	digits = CharCount;   //RJS 02/27/14   To initialize for only numeral values

	for(int i=0;i<CharCount;i++)
	{
		// char c = in[CharCount-i-1];
        wint_t c = in[CharCount - i - 1];  // sws- Mantis 0008216 (fixes wchar digit check in non-ASCII languages)
		if	(((!iswdigit(c))/*&&(c != '-'))||(c == '.')*/))
		{
			digits = i;
			FirstDigit = CharCount-i;				   
			break;
		}		  
	}
	//disassemble the name
	CString sNumber;
	sNumber  = in.Right(CharCount-FirstDigit);
	namebase = in.Left(FirstDigit);

	num = _ttoi64(sNumber);

	return TRUE;
}

// Returns FALSE if the input string is empty, TRUE otherwise.
// nonDigitLocation: the location of the first non-digit value found from the end of the input string - set to -1 if entirely a number
// endNumber: the numeric value found on the end of the given string - set to 0.0 if no digits found
BOOL StringHelper::DivideNumberedName(const CString& inputString, int& nonDigitLocation, double& endNumber)
{
	BOOL success = FALSE;
	nonDigitLocation = -1;
	endNumber = 0.0;

	if(!inputString.IsEmpty())
	{
		// Copy the original
		CString workingString(inputString);

		while(!workingString.IsEmpty())
		{
			if(IsValidNumString(workingString))
			{
				// Check for a dash
				if(workingString.FindOneOf(_T("-")) == 0)
				{
					// The previous algorithm didn't account for negative numbers so we won't either
					nonDigitLocation++;
					if(workingString.GetLength() > 1)
					{
						// Keep moving
						workingString = workingString.Mid(1);
					}
					else
					{
						// Dash only - go no further
						break;
					}
				}

				// Check for a period
				if(workingString.FindOneOf(_T(".")) == 0)
				{
					// If the first character is a '.' we will consider it a period and not a decimal point.
					// So, p.1 = 1 and p0.1 = .1
					nonDigitLocation++;
					if(workingString.GetLength() > 1)
					{
						// Keep moving
						workingString = workingString.Mid(1);
					}
					else
					{
						// Period only - go no further
						break;
					}
				}

				// We're done
				_stscanf(workingString, _T("%lf"), &endNumber);
				break;
			}

			// Drop the first character from the string and try again
			nonDigitLocation++;
			workingString = workingString.Mid(1);
		}

		success = TRUE;
	}

	return success;
}

int StringHelper::NumberedNameCompare(LPCTSTR a, LPCTSTR b)
{
	double anum, bnum;

	int aloc, bloc;

	// Returning a is less than b if either a or b is empty
	if(!DivideNumberedName(a, aloc, anum))
		return -5;
	if(!DivideNumberedName(b, bloc, bnum))
		return -5;

	// Both strings start with a non-digit
	if(aloc != -1 && bloc != -1)
	{
		int result = Compare(a, aloc, b, bloc);
		if (result != 0)
		{
			return result;
		}
	}
	else
	{
		// If aloc and/or bloc is -1, then the name is entirely a number
		//  If only one is -1, then return the name that is entirely a number
		//  as the lesser.  If both are -1, then need to compare values.

		// Only one string starts with a non-digit
		if (!(aloc == -1 && bloc == -1))
		{
			if (aloc == -1)
				return -1;
			else
				return 1;
		}

	}

	// Both strings start with a digit
	if(anum > bnum)
		return 1;
	else if(anum < bnum)
		return -1;

	return 0;
}

//
// Compares String A to String B, up to the specified offsets in each.
// Returns:    0 if strings are equal
//          -100 if String A is less than String B
//          +100 if String A is greater than String B
//
int StringHelper::Compare(LPCTSTR a, const int& aoffset, LPCTSTR b, const int& boffset)
{
	BOOL have1 = TRUE;
	BOOL have2 = TRUE;

	register int i = 0;
	register int j = 0;

	while(have1||have2)
	{
		have1 = have2 = FALSE;
		for(;i<=aoffset;i++)   //get next non space char of s1
		{
			if(a[i] != ' ')
			{
				have1 = TRUE;
				break;
			}
		}

		for(;j<=boffset;j++)   //get next non space char of s2
		{
			if(b[j] != ' ')
			{
				have2 = TRUE;
				break;
			}
		}

		if (have1 && have2)
		{
			int lowerA = tolower(a[i]);
			int lowerB = tolower(b[j]);
			if (lowerA != lowerB)
			{
				return ( (lowerA < lowerB) ? -100 : 100);
			}
			// increment location pointers
			i++;
			j++;
		}
		else // Someone ran out of characters
		{
			// If first string still has characters, then
			//  the first string is larger then the second
			if (have1)
				return 100;
			else if (have2) // Second still has characters, first is smaller
				return -100;
		}
	}

	return 0;
}

void StringHelper::StripSpaces( LPCTSTR a, LPTSTR b )
{
	int j=0;
	for (int i=0;i<(int)_tcslen(a);i++)
		if (a[i] != ' ')
			b[j++] = a[i];
	b[j] = NULL;
}

CString StringHelper::ExtractFilenameFromPathname(const CString& Path)
{
	if (Path.GetLength() == 0)
		return Path;

	int slash = Path.ReverseFind('\\');

	if (slash < 0)
		return Path;

	return Path.Right(Path.GetLength() - slash -1);
}
BOOL StringHelper::ExtractDirectoryAndFilenameFromPath(const CString& Path, CString& Directory,CString& Filename)
{
	Filename = ExtractFilenameFromPathname(Path);
	int pathchars = Path.GetLength();
	int filenamechars = Filename.GetLength();

	if(pathchars-filenamechars>0)
	{
		Directory = Path.Left(pathchars-filenamechars);
		return TRUE;
	}
	return FALSE;

}
bool StringHelper::ExtractExtensionFromPathname(const CString& path, CString& extension)
{
	int slash = path.ReverseFind('.');
	if (slash<0)
		return false;
	extension = path.Right(path.GetLength() - slash -1);
	return true;
}

CString StringHelper::ExtractBasenameFromPath(const CString& path)
{
	CString result;

	result = ExtractFilenameFromPathname(path);

	int dpos = result.ReverseFind(_T('.'));
	if (dpos >= 0)
	{
		CString tmp = result;
		result = tmp.Left(dpos);
	}

	return result;
}

BOOL StringHelper::GetCSVItem(const CString& whole, const int index, CString* pout)
{
	CStringArray pieces;
	GetCSVItems(whole, pieces);
	if (index < pieces.GetSize())
	{
		*pout = pieces[index];
		return TRUE;
	}
	return FALSE;
}
BOOL StringHelper::GetCSVItems(const CString& whole, CStringArray& pieces)
{
	// jmc -- csv parsing -- added sparts to deal with quotes in columns and commas in quotes.
	int count = 0;
	CString piece;
	bool inQuotes=false;		// true while we are inside quotations and in search of an end quote.
	for (int i=0;i<whole.GetLength();i++)
	{
		if (whole[i]==',' && !inQuotes)
		{
			count ++;
			if (piece.GetLength()>1)
			{
				if (piece[0]=='"' && piece[piece.GetLength()-1]=='"')
				{
					CString temp(piece);
					piece = temp.Mid(1,temp.GetLength()-2);
				}
			}
			pieces.Add( piece );
			piece.Empty();		// clear it.
			// Handle case where the data ends with a comma...
			// For example, "5,," or ",5,"
			if (i+1 == whole.GetLength())
			{
				count++;
				pieces.Add(piece);
			}
		}
		else if (whole[i]=='"')
		{
			if (inQuotes)
			{
				if (i+1 >= whole.GetLength())		// at the end.  do nothing.  Ignore the " -- it ends a quotation at the end of the line.
					;
				else
				{
					// is this a double quote?
					if (whole[i+1]=='"')
						piece += whole[i++];		// want to keep the quote.	INCREMENT to skip the next.
					else
						inQuotes = false;		// end of the quote... do nothing.  Should hit a comma next.
				}
			}
			else
			{			// not in a quotation -- should we be??  lets make sure its not a double.
				if (i+1 >= whole.GetLength())		// at the end.  do nothing.  Ignore the " -- it ends a quotation at the end of the line.
					;
				else
				{
					// is this a double quote?
					if (whole[i+1]=='"')
						piece += whole[i++];		// want to keep the quote.
					else
						inQuotes = true;		// we're starting a quote!
				}
			}
		}
		else
			piece += whole[i];
	}
	if (!IsBlank(piece))
		pieces.Add(piece);		// jmc -- make sure to get the last bit!
	return true;
}

BOOL StringHelper::GetCSVItemsEx(LPCTSTR line, CStringArray& cells)
{
	cells.RemoveAll();

	ASSERT(line != NULL);
	if (line == NULL)
		return FALSE;

	CString localLine = line;

	localLine.Trim();
	if (localLine.IsEmpty())
		return FALSE;

	if ((localLine.GetLength() >= 2) && (localLine.Left(2) == _T("//")))
		return FALSE;

	CStringArray tmp;
	StringHelper::GetCSVItems(localLine,tmp);

	BOOL allEmpty = TRUE;
	for (INT_PTR i = 0; i < tmp.GetSize(); i++)
	{
		CString itm = tmp[i];

		itm.Trim();

		if ((itm.GetLength() >= 2) && (itm[0] == _T('"')) && (itm[itm.GetLength()-1] == _T('"')))
		{
			CString itmTmp = itm.Mid(1,itm.GetLength()-2);
			itm = itmTmp;
			itm.Trim();
		}

		if (!itm.IsEmpty())
			allEmpty = FALSE;

		cells.Add(itm);
	}

	if (allEmpty)
	{
		cells.RemoveAll();
		return FALSE;
	}

	if ((cells.GetSize() > 0) && (cells[0].GetLength() >= 2) && (cells[0].Left(2) == _T("//")))
		return FALSE;

	return TRUE;
}

void StringHelper::SetCSVItemsIntoRow(const CStringArray& pieces, CString& whole)
{
	// jmc - push strings in CSV format.  
	// special cases -- if a , is in the string, put quotes around the whole string.
	//							if there are " 's in the string, prefix each with a " -- double quotes are interpreted as a " that does not end the cell.
	for (DWORD_PTR i=0;i<pieces.GetSize();i++)
	{
		CString p( pieces[i] );
		// make double quotes.
		for (int j=0;j<p.GetLength();j++)
		{
			if (p[j]==_T('"'))
			{
				p.Insert(j,_T('"'));
				j++;
			}
		}
		// find internal commas
		if (p.Find(_T(','))>=0)
		{
			CString temp(p);
			p = _T('"') + temp + _T('"');
		}

		whole += p;
		if (i<pieces.GetSize()-1)
			whole += _T(",");
	}

}

// sws - XIT-1044 (Add Frame Set entity to tree for extended high rate 6D measurement tasks)
void StringHelper::SetCSVItemsIntoRow(const CStringArray & pieces, CString & whole, CString & delimiter)
{
   for (DWORD_PTR i=0;i<pieces.GetSize();i++)
   {
      CString p( pieces[i] );

      for (int j = 0; j < p.GetLength(); j++)
      {
         if (p[j]==_T('"'))
         {
            p.Insert(j, _T('"'));
            j++;
         }
      }

      if (p.Find(delimiter) >= 0)
      {
         CString temp(p);
         p = _T('"') + temp + _T('"');
      }

      whole += p;

      if (i < pieces.GetUpperBound())
         whole += delimiter;
   }
}

BOOL StringHelper::isPrintableString(LPCTSTR str)
{
	if (!str) return FALSE;
	int l = lstrlen(str);
	for (int i = 0; i < l; i++)
	{
		if ((str[i] < 32) || (str[i] > 127))
			return FALSE;
	}
	return TRUE;
}

void StringHelper::AppendString(CString& s, DWORD_PTR x)
{      
	TCHAR a[20];
	_stprintf(a,_T("%llu"),x);
	s += a;
}  

void StringHelper::AppendString(CString& s, WORD x)
{      
	TCHAR a[20];
	_stprintf(a,_T("%hu"),x);
	s += a;
}

void StringHelper::AppendString(CString& s, int x)  
{	
	TCHAR a[20];
	_stprintf(a,_T("%d"),x);
	s += a;
}

void StringHelper::AppendString(CString& s, double x)
{
	TCHAR a[20];
	_stprintf(a,_T("%f"),x);
	s += a;
}  

void StringHelper::AppendStringE(CString& s, double x)  // sws - XIT-1399 (Euler Parameters and Equivalent Angle Axis don't copy)
{
   CString a;
   a.Format(_T("%0.12e"), x);
   s += a;
}

BOOL StringHelper::IsValidNumString(CString& s)
{
	BOOL decimal = FALSE;
	for(int i = 0; i < s.GetLength(); i++)
	{
		//TCHAR c = s[i];
        wint_t c = s[i];  // sws/olga - 2016.04.06 mantis #8216 (fixes wchar digit check in non-ASCII languages)

        //if(isdigit(c))
        if(iswdigit(c)) // sws/olga - 2016.04.06 mantis #8216
		{
			continue;
		}
		if(c == '.')
		{
			if(decimal)
			{
				return FALSE;
			}
			decimal = TRUE;
			continue;
		}
		if(c == '-')
		{
			// Don't allow a '-' unless it's at the beginning of the string.
			// Might be valid if we were expecting exponents - but then we
			// would also have to account for '+' and 'e' or 'E'.
			if(i != 0)
			{
				return FALSE;
			}
			continue;
		}
		return FALSE;
	}

	return TRUE;
}

// sws - XIT-125 (GD&T feature checks custom summary table)
BOOL StringHelper::ExtractNumberString(CString inStr, CString & leftStr, CString & rightStr, CString & numStr, int & sigDig)
{
   int start(0);
   int marker(0);
   int dp(-1);
   wint_t c;
   bool stringValid(false);

   while (!stringValid && (marker < inStr.GetLength()))
   {
      // The first character of a number could be any of several as shown

      for (start = marker; start < inStr.GetLength(); start++)
      {
         c = inStr[start];

         if (iswdigit(c) || (c == _T('+')) || (c == _T('-')) || (c == _T('.')))
         {
            marker = start;  // Note character position of start for numeric string
            break;
         }
      }

      // Make sure the entire string did not fail to produce a numeric string start point

      if (start == inStr.GetLength())
         break;

      stringValid = iswdigit(c);

      if (iswdigit(c) || (c == _T('+')) || (c == _T('-')))
      {
         // Find digits up to the decimal point (if there is one)

         for (marker++; marker < inStr.GetLength(); marker++)
         {
            c = inStr[marker];

            if (!iswdigit(c))
               break;

            stringValid = true;
         }

         // Find the remaining digits after the decimal point (if there are any)

         if (c == _T('.'))
         {
            dp = marker;

            for (marker++; marker < inStr.GetLength(); marker++)
            {
               c = inStr[marker];

               if (!iswdigit(c))
                  break;

               stringValid = true;
            }
         }
      }
      else if (c == _T('.'))
      {
         dp = marker;

         for (marker++; marker < inStr.GetLength(); marker++)
         {
            c = inStr[marker];

            if (!iswdigit(c))
               break;

            stringValid = true;
         }
      }
   }

   stringValid = ((start < marker) && (marker <= inStr.GetLength()));

   if (stringValid)
   {
      numStr    = inStr.Mid(start, marker - start);
      leftStr  += inStr.Left(start);
      rightStr  = inStr.Mid(marker, inStr.GetLength() - marker);

      if (dp != -1)
         sigDig = marker - dp - 1;
   }
   else
   {
      leftStr += rightStr;
      numStr.Empty();
   }

   return (stringValid ? TRUE : FALSE);
}

BOOL StringHelper::WriteHTMLTable(const CStringArray& csa, CStdioFile& f)
{
	int TablePadding = 2;	// could make these arguments, but who ever needs to set them??
	int TableBorder = 1;

	CString tmp;
	tmp.Format(_T("<br><p> <big><strong>%s</strong></big></p><div align=\"left\"><table border=\"%d\" cellpadding=\"%d\">\n"),csa[0],TableBorder,TablePadding);
	f.WriteString(tmp);

	for (int i=1;i<csa.GetSize();i++)
	{
		CString data = csa[i];
		CString str;
		f.WriteString(_T("<tr>")); //begin row

		int stuff = 0;
		str = data;
		BOOL Go = TRUE;
		while(Go)
		{	
			int pos1 = data.FindOneOf(_T(",\n"));
			while(pos1==0)
			{
				data.TrimLeft(_T(","));
				data.TrimLeft(_T("\n"));
				pos1 = data.FindOneOf(_T(",\n"));
			}
			if(pos1==-1)
				break;
			str= data.Left(pos1);
			CString m;
			m.Format(_T("<td><p align=\"right\">%s</td>"),str);
			f.WriteString(m);
			data.TrimLeft(str);
		}
		f.WriteString(_T("</tr>")); //end row
	}
	CString footer(_T("</table>\n"));
	f.WriteString(footer);
	return TRUE;
}
int StringHelper::RemoveCharacters(const CString& charlist, CStringArray& array)
{
	int total = 0;
	for (int i=0;i<array.GetSize();i++)
		total += RemoveCharacters(charlist,array[i]);

	return total;
}
int StringHelper::RemoveCharacters(const CString& charlist, CString& txt)
{
	int total=0;
	for (int j=0;j<charlist.GetLength();j++)
		total += txt.Remove(charlist[j]);

	return total;
}
int StringHelper::ReplaceCharacters(const CString& oldCharList, const CString& newChar, CString& txt)
{
	int total=0;
	for (int j=0;j<oldCharList.GetLength();j++)
		total += txt.Replace(oldCharList[j], newChar[0]);

	return total;
}
//***************************************************************************************************
// olga XIT-855 - Replace all matching strings (case sensitive/insensitive)
// It returns the number of replaced instances
//***************************************************************************************************
int StringHelper::FindAndReplaceText(CString& itemText, const CString searchCriteria, const CString replaceWith, BOOL bMatchCaseCheck)    // olga XIT-855
{
    CString inText = itemText;
    int count = 0; // The number of replaced instances

    if(searchCriteria.IsEmpty())
        return count;

    if(bMatchCaseCheck)
    {
        count = itemText.Replace(searchCriteria, replaceWith); // case sensitive
    }
    else // case insensitive
    {       
        CString copyText(itemText);
        copyText.MakeUpper();

        CString searchStr(searchCriteria);
        searchStr.MakeUpper();

        // find location of a search criteria string
        int loc = -1;
        int start = 0;

        // replace all matches
        while( (start < copyText.GetLength()) && ((loc = copyText.Find(searchStr, start))> -1))
        {
            // replace substring in the argument value 
            itemText.Delete(loc, searchCriteria.GetLength());
            itemText.Insert(loc, replaceWith);

            // olga XIT-1393 - reset string for search
            copyText = itemText;
            copyText.MakeUpper();

            start = loc + replaceWith.GetLength();  // adjust start
            count++;                    
        }
    } 
    return count; 
}

bool StringHelper::IsBlank(LPCTSTR str)
{
	int len = _tcslen(str);
	if (len<1)
		return true;
	for (int i=0;i<len;i++)
	{
		//if (str[i] != (TCHAR)_T(" "))
		if (str[i] != ' ')
			return false;
	}
	return true;
}

bool StringHelper::IsBlank(const CStringArray& arr)
{
	if (arr.GetSize()<1)
		return true;
	for (int i=0;i<arr.GetSize();i++)
		if (!StringHelper::IsBlank(arr[i]))
			return false;
	return true;
}
void StringHelper::ArrayToMultiLineString(const CStringArray& array, CString& out)
{
	out.Empty();
	for (int i=0;i<array.GetSize(); i++)
		out += array.GetAt(i) + _T("\r\n");
}
void StringHelper::ArrayToSingleString(const CStringArray& array, CString& out, LPCTSTR sep /*= _T("; ")*/)
{
	ASSERT(sep != NULL);
	if (sep == NULL)
		sep = _T("; ");

	out.Empty();
	for (int i=0;i<array.GetSize(); i++)
	{
		if (i>0)
			out += sep;
		out += array[i];
	}
}
void StringHelper::MultiLineStringToArray(LPCTSTR str, CStringArray& out)
{
	out.RemoveAll();

	ASSERT(str != NULL);
	if (str == NULL)
		return;

	StringHelper::Tokenize(str,_T("\r\n"),out);
}
bool StringHelper::GetCTimeFromString(const CString& formattedTime, CTime& time)
{
	int hour, min, sec, month, day, year;
	hour = min = sec = month = day = year = 0;

	// string assumed to be created with: .Format("%m/%d/%y %H:%M:%S");
	int count=_stscanf(formattedTime,_T("%d/%d/%d %d:%d:%d"),
		&month, &day, &year, &hour, &min, &sec);

	if (count<5)		// jmc - used to be 6.
		return false;
	if (count==5)		// excel, for example, drops off seconds.....
		sec = 0;

	int fullYear = 0;

	if (year > 1900)
	{
		fullYear = year;
	}
	else
	{
		if (year>96)
			fullYear = 1900 + year;
		else
			fullYear = 2000 + year;
	}

	try
	{
		time = CTime(fullYear, month, day, hour, min, sec);
	}
	catch (CException* e)
	{
		return false;
	}

	return true;
}

CString StringHelper::GetFilenameCompatibleDateAndTime(const CTime& time)
{
	CString filename;
	filename.Format(_T("%1d-%1d-%1d_%1d.%1d.%1d"),
		time.GetYear(),time.GetMonth(),time.GetDay(),
		time.GetHour(),time.GetMinute(),time.GetSecond());
	return filename;
}

bool StringHelper::PurifyPathname(CString& path)
{
	CString invalidChars(":*?\"<>|");
	return (RemoveCharacters(invalidChars,path)>0) ? true : false;
}

bool StringHelper::PurifyFilename(CString& path)
{
	bool result = PurifyPathname(path);

	CString invalidChars("\\/");
	if (RemoveCharacters(invalidChars,path) > 0)
		result = true;

	return result;
}

CString StringHelper::PurifyXMLString(LPCTSTR str)
{
	CString result;

	ASSERT(str != NULL);
	if (str == NULL)
		return result;
	CString _str(str);

	for (int strIdx = 0; strIdx < _str.GetLength(); strIdx++)
	{
		_TCHAR c = _str[strIdx];
		if ((c < 32) || (c > 127))
		{
			if ((c != _T('\n')) && (c != _T('\r')))
				result += _T('?');
		}
		else
			result += _str[strIdx];
	}

	result.Replace(_T("<"),_T("&lt;"));
	result.Replace(_T(">"),_T("&gt;"));
	result.Replace(_T("&"),_T("&amp;"));
	result.Replace(_T("'"),_T("&apos;"));
	result.Replace(_T("\""),_T("&quot;"));

	return result;
}

int StringHelper::ComparePaths(LPCTSTR pathA, LPCTSTR pathB)
{
	ASSERT(pathA != NULL);
	if (pathA == NULL)
		return 0;
	ASSERT(pathB != NULL);
	if (pathB == NULL)
		return 0;

	CString longPathA = EnsureLongPath(pathA);
	CString longPathB = EnsureLongPath(pathB);

	return longPathA.CompareNoCase(longPathB);
}

CString StringHelper::EnsureLongPath(LPCTSTR path)
{
	CString result;

	ASSERT(path != NULL);
	if (path == NULL)
		return result;
	result = path;

	_TCHAR buf[MAX_PATH+1];
	ZeroMemory(buf,sizeof(_TCHAR)*(MAX_PATH+1));
	if (GetLongPathName(path,buf,MAX_PATH) != 0)
	{
		buf[MAX_PATH] = 0;
		result = buf;
	}

	return result;
}

void StringHelper::EnsureTrailingBS(CString& s)
{
	if ((s.GetLength() == 0) ||
		(s[s.GetLength()-1] != _T('\\')))
	{
		s += _T('\\');
	}
}

void StringHelper::EnsureNoTrailingBS(CString& s)
{
	if ((s.GetLength() > 0) &&
		(s[s.GetLength()-1] == _T('\\')))
	{
		s = s.Left(s.GetLength()-1);
	}
}


/**
 * optimized serialization for CString objects; the default CString serialization stores an identifier
 * in the stream letting you know there's a CString there; if you know you're storing/loading a CString
 * you can use this instead to avoid those identifiers.
 *
 * this writes the string length followed by the string characters.  the # of bytes used for the string
 * length depends on the length of the string, allowing you to serialize very large strings while
 * still only using the necessary # bytes in your stream for length information.
 *
 * String Length       Binary Stream Representation
 * ---------------------------------------------------------------------------------------
 * 0 ... 253           lengthBYTE char1 ... charN-1
 * 254 ... 65535       254 lengthWORD char1 ... charN-1
 * 65536 ...           255 lengthDWORD char1 ... charN-1
 */
void StringHelper::Serialize(CArchive & ar, CString & str)
{
	if (ar.IsStoring())
	{
		int len = str.GetLength();

		if (len < 254)
		{
			BYTE blen = (BYTE)len;
			ar << blen;
		}
		else if (len <= 65535)
		{
			BYTE mark = 254;
			ar << mark;

			WORD wlen = (WORD)len;
			ar << wlen;
		}
		else
		{
			BYTE mark = 255;
			ar << mark;

			DWORD dlen = (DWORD)len;
			ar << dlen;
		}

		ar.Write((LPCTSTR)str,sizeof(TCHAR)*len);
	}
	else
	{
		int len = 0;

		BYTE blen;
		ar >> blen;

		if (blen < 254)
		{
			len = (int)blen;
		}
		else
		{
			if (blen == 254)
			{
				WORD wlen;
				ar >> wlen;

				len = (int)wlen;
			}
			else if (blen == 255)
			{
				DWORD dlen;
				ar >> dlen;

				len = (int)dlen;
			}
		}

		TCHAR * sbuf = str.GetBuffer(len);
		ar.Read(sbuf,sizeof(TCHAR)*len);
		str.ReleaseBuffer(len);
	}
}

StringHelper::CStringANSI StringHelper::ConvertToANSI(LPCTSTR str)
{
	CStringANSI result(str);
	return result;
}

CString StringHelper::MakeStringFromChar(_TCHAR c)
{
	CString result;
	result += c;
	return result;
}

int StringHelper::CStringCompareAscending(const void *a, const void *b)
{
	CString *pA = (CString*)a;
	CString *pB = (CString*)b;
	return (pA->Compare(*pB));
}

int StringHelper::CStringCompareDescending(const void *a, const void *b)
{
	CString *pA = (CString*)a;
	CString *pB = (CString*)b;
	return (-1 * (pA->Compare(*pB)));
}

void StringHelper::SortStringArray(CStringArray& csa, BOOL bDescending)
{
	int iArraySize = csa.GetSize();
	if (iArraySize <= 0)
		return;

	int iCSSize = sizeof (CString*);
	void* pArrayStart = (void *)&csa[0];

	if (bDescending)
		qsort (pArrayStart, iArraySize, iCSSize, CStringCompareDescending);
	else
		qsort (pArrayStart, iArraySize, iCSSize, CStringCompareAscending);
}

void StringHelper::RemoveDuplicateStrings(CStringArray& csa, BOOL caseSensitive)
{
	CMapStringToOb dupMap;
	for (INT_PTR i = 0; i < csa.GetSize(); )
	{
		CString str = csa[i];
		if (!caseSensitive)
			str.MakeUpper();
		
		CObject* pObj = NULL;
		if (dupMap.Lookup(str,pObj))
		{
			csa.RemoveAt(i);
		}
		else
		{
			dupMap.SetAt(str,NULL);
			i++;
		}
	}
}

BOOL StringHelper::CompareStringArrays(CStringArray& lhs, CStringArray& rhs, BOOL caseSensitive)
{
	if (lhs.GetSize() != rhs.GetSize())
		return FALSE;

	for (INT_PTR i = 0; i < lhs.GetSize(); i++)
	{
		CString& lhsStr = lhs[i];
		CString& rhsStr = rhs[i];

		if (caseSensitive)
		{
			if (lhsStr.Compare(rhsStr) != 0)
				return FALSE;
		}
		else
		{
			if (lhsStr.CompareNoCase(rhsStr) != 0)
				return FALSE;
		}
	}

	return TRUE;
}

void StringHelper::AddToStringArrayIfNotAlreadyPresent(CStringArray& arr, LPCTSTR stringToAdd, BOOL caseSensitiveCompare)
{
	ASSERT(stringToAdd != NULL);
	if (stringToAdd == NULL)
		return;

	BOOL stringPresent = FALSE;
	for (INT_PTR i = 0; (i < arr.GetSize()) && !stringPresent; i++)
	{
		if (caseSensitiveCompare)
		{
			if (arr[i].Compare(stringToAdd) == 0)
			{
				stringPresent = TRUE;
			}
		}
		else
		{
			if (arr[i].CompareNoCase(stringToAdd) == 0)
			{
				stringPresent = TRUE;
			}
		}
	}

	if (stringPresent)
		return;

	arr.Add(stringToAdd);
}

BOOL StringHelper::StringConvertUCS2toUTF8(LPCTSTR inStr, CStringA& outStr)
{
	outStr.Empty();

	ASSERT(inStr != NULL);
	if (inStr == NULL)
		return FALSE;

	int reqdBufSz = WideCharToMultiByte(CP_UTF8,0,inStr,-1,NULL,0,NULL,NULL);
	if (reqdBufSz == 0)
		return FALSE;

	char* outBuf = outStr.GetBuffer(reqdBufSz);
	int res = WideCharToMultiByte(CP_UTF8,0,inStr,-1,outBuf,reqdBufSz,NULL,NULL);
	outStr.ReleaseBuffer(reqdBufSz-1);
	if (res == 0)
	{
		outStr.Empty();
		return FALSE;
	}

	return TRUE;
}

BOOL StringHelper::StringConvertUTF8toUCS2(LPCSTR inStr, CString& outStr)
{
	outStr.Empty();

	ASSERT(inStr != NULL);
	if (inStr == NULL)
		return FALSE;

	int reqdBufSz = MultiByteToWideChar(CP_UTF8,0,inStr,-1,NULL,0);
	if (reqdBufSz == 0)
		return FALSE;

	TCHAR* outBuf = outStr.GetBuffer(reqdBufSz);
	int res = MultiByteToWideChar(CP_UTF8,0,inStr,-1,outBuf,reqdBufSz);
	outStr.ReleaseBuffer(reqdBufSz-1);
	if (res == 0)
	{
		outStr.Empty();
		return FALSE;
	}

	return TRUE;
}

void StringHelper::MakeStringValidCFunctionName(CString& s, TCHAR replacementChar /*= _T('_')*/)
{
	CString ns;
	int len = s.GetLength();
	for (int i = 0; i < len; i++)
	{
		TCHAR c = s[i];
		BOOL valid = FALSE;

		if ((i > 0) && ((c >= _T('0')) && (c <= _T('9'))))
		{
			valid = TRUE;
		}
		else if ((c >= _T('A')) && (c <= _T('Z')))
		{
			valid = TRUE;
		}
		else if ((c >= _T('a')) && (c <= _T('z')))
		{
			valid = TRUE;
		}
		else if (c == _T('$'))
		{
			valid = TRUE;
		}
		else if (c == replacementChar)
		{
			valid = TRUE;
		}

		if (valid)
			ns += c;
		else
			ns += replacementChar;
	}

	s = ns;
}

CString StringHelper::LimitStringLengthUsingEllipsesInTheMiddle(LPCTSTR str, DWORD maxLength)
{
	CString result;

	ASSERT(str != NULL);
	if (str == NULL)
		return result;

	result = str;

	// no action needed if the string does not exceed the max length
	if (result.GetLength() > maxLength)
	{
		// boundary case: too few chars to use ellipses, just take leftmost chars
		if (maxLength < 5)
		{
			CString tmp = result.Left(maxLength);
			result = tmp;
		}
		// shorten string to max length using ellipses in the middle
		else
		{
			int numCharsPerSide = (maxLength - 3) / 2;
			CString tmp;
			tmp += result.Left(numCharsPerSide);
			tmp += _T("...");
			tmp += result.Right(numCharsPerSide);
			result = tmp;
		}
	}

	return result;
}

CString StringHelper::FormatNumBytes(ULONGLONG numBytes)
{
	CString result;

	double numBytesD = (double)numBytes;

	const ULONGLONG kB = 1024;
	const ULONGLONG MB = kB * kB;
	const ULONGLONG GB = MB * 1000;

	if (numBytes < kB)
	{
		result.Format(_T("%I64u %s"), numBytes, _T("bytes"));
	}
	else if (numBytes < MB)
	{
		result.Format(_T("%0.2f %s"), numBytesD / (double)kB, _T("kB"));
	}
	else if (numBytes < GB)
	{
		result.Format(_T("%0.2f %s"), numBytesD / (double)MB, _T("MB"));
	}
	else
	{
		result.Format(_T("%0.2f %s"), numBytesD / (double)GB, _T("GB"));
	}

	return result;
}

void StringHelper::CopyCMapStringToString(const CMapStringToString& source, CMapStringToString& dest)
{
	// jmc 2013.06.30 -- needed and easy way to copy Cstring maps.....

	// Copy the CMapStringToString Keys and Values
	// Note the use of CPair avoids the need to create temporary strings for the Key/Value pairs!

	if (source.GetSize()<1)		// jmc 2013.09.09 - can't call get first on an empty map!
		return;

	const CMapStringToString::CPair* pCurVal = source.PGetFirstAssoc();
	while (pCurVal != NULL)
	{
		dest.SetAt(pCurVal->key, pCurVal->value);
		pCurVal = source.PGetNextAssoc(pCurVal);
	}
}

CString StringHelper::ExtractInclusiveSubstring(const CString& source, const CString& startMarker, const CString& endMarker)
{
	CString result;

	if (source.IsEmpty() || startMarker.IsEmpty() || endMarker.IsEmpty())
		return result;

	if (source.GetLength() < (startMarker.GetLength() + endMarker.GetLength()))
		return result;

	int startIdx = -1;
	int endIdx = -1;
	LPCTSTR sourcePtr = source;
	for (int i = 0; i < source.GetLength(); i++)
	{
		if (startIdx == -1)
		{
			if (_tcsnicmp(&sourcePtr[i],startMarker,startMarker.GetLength()) == 0)
				startIdx = i;
		}
		else if (endIdx == -1)
		{
			if (_tcsnicmp(&sourcePtr[i],endMarker,endMarker.GetLength()) == 0)
			{
				endIdx = i + endMarker.GetLength() - 1;
				break;
			}
		}
	}

	if ((startIdx >= 0) &&
		(endIdx > startIdx) && (endIdx < source.GetLength()))
	{
		result  = source.Mid(startIdx,endIdx - startIdx + 1);
	}

	return result;
}
