#pragma once

class StringHelper  
{
public:
	StringHelper();
	virtual ~StringHelper();
	static BOOL DivideNumberedName(const CString& in, CString& namebase,__int64& num);
	static BOOL DivideNumberedName(const CString& inputString, int& nonDigitLocation, double& endNumber);
	static BOOL StringHelper::DivideNumberedName(const CString& in, CString& namebase,__int64& num,int& digits);
	static int NumberedNameCompare(LPCTSTR a, LPCTSTR b);
	static int Compare(LPCTSTR a, const int& aoffset, LPCTSTR b, const int& boffset);

	static bool IsBlank(LPCTSTR str);	// empty or only containing spaces.
	static bool IsBlank(const CStringArray& arr);
	static BOOL isPrintableString(LPCTSTR str);
	static BOOL IsValidNumString(CString& s);
	static BOOL IsAsciiString(const LPCTSTR inStr); // olga XIT-1696 - check if string has only English chars

	static BOOL GetLastErrorTrace();
	static BOOL GetLastErrorString(CString& Str);

	static BOOL GetStringBetween(const CString& InStr,const CString& s1, const CString& s2,CString& OutStr,BOOL Backwards = FALSE,int* plastindex = NULL);
	static BOOL RemoveStringBetweenIncludingBounds(const CString& InStr,const CString& s1, const CString& s2,CString& OutStr,int& index);
	static BOOL SplitString(const CString& InStr,const CString& split, CString& OutStr1,CString& OutStr2);
	static void Tokenize(const LPCTSTR inStr, LPCTSTR inSeparatorChars, CStringArray& outTokens);
	static BOOL DissectPointName(const CString& InStr, CString& CN, CString& GN, CString& TN);
	static BOOL DissectPointName(const CString& InStr, CString& GN, CString& TN);
	static BOOL DissectOptionalCollectionAndGroupName(LPCTSTR inStr, CString& CN, CString& GN);
	static void AssembleFullTargetName(const CString& col, const CString& grp, const CString& tgt, CString& composite);		// jmc 2013.08.08
	static BOOL NameCompare(LPCTSTR ps1, LPCTSTR ps2); // string comparison which ignores white space and case differences
	static void MakeUpperAndRemoveSpaces(CString& txt);	// jmc - use if making a MAP and you want same compare as "NameCompare"

	static BOOL IncrementName(CString &name, int inc=1) ;
	static BOOL IncrementTargetName(CString &name, int inc=1) ;
	static void IncrementAlpha(CString& str);	// A B C D ... Z  ZA  ZB  ZC
	static void StripSpaces( LPCTSTR a, LPTSTR b );

	static BOOL GetCSVItem(const CString& whole, const int index, CString* pout);		// efficiency warning -- just calls getItems, then only returns the one that is requested.
	static BOOL GetCSVItems(const CString& whole, CStringArray& pieces);
	static BOOL GetCSVItemsEx(LPCTSTR line, CStringArray& cells);
	static void SetCSVItemsIntoRow(const CStringArray& pieces, CString& whole);
    static void SetCSVItemsIntoRow(const CStringArray & pieces, CString & whole, CString & delimiter);  // sws - XIT-1044 (Add Frame Set entity to tree for extended high rate 6D measurement tasks)

	static CString ExtractFilenameFromPathname(const CString& Path);
	static bool ExtractExtensionFromPathname(const CString& path, CString& extension);
	static BOOL ExtractDirectoryAndFilenameFromPath(const CString& Path, CString& Directory,CString& Filename);
	static CString ExtractBasenameFromPath(const CString& path);
	static bool PurifyPathname(CString& path);	// get rid of invalid characters. TRUE if any removed.
	static bool PurifyFilename(CString& path);	// get rid of invalid characters for the filename portion of a path. TRUE if any removed.
	static CString PurifyXMLString(LPCTSTR str); // clean up the string for XML validity
	static int ComparePaths(LPCTSTR pathA, LPCTSTR pathB); // case in-sensitive and long/short in-sensitive comparison of two paths
	static CString EnsureLongPath(LPCTSTR path); // converts (if necessary) the provided arg to a long path
	static void EnsureTrailingBS(CString& s); // ensure the string ends with a backslash '\\'
	static void EnsureNoTrailingBS(CString& s); // ensure the string does not end with a backslash '\\'
	static bool GetCTimeFromString(const CString& formattedTime, CTime& time);
	static CString GetFilenameCompatibleDateAndTime(const CTime& time);
	static CString GetStringFromCTime(const CTime& time)
	{
		return time.Format("%m/%d/%y %H:%M:%S");
	};

    static BOOL ExtractNumberString(CString inStr, CString & leftStr, CString & rightStr, CString & numStr, int & sigDig);  // sws - XIT-125 (GD&T feature checks custom summary table)

	static void GetCurrentTimeString( CString& s );
	static void GetTimeIntervalString( double time, CString& s );

	static void AppendString(CString& s, DWORD_PTR x);
	static void AppendString(CString& s, WORD x);
	static void AppendString(CString& s, int x);
	static void AppendString(CString& s, double x);
    static void AppendStringE(CString& s, double x);  // sws - XIT-1399 (Euler Parameters and Equivalent Angle Axis don't copy)

	static int RemoveCharacters(const CString& charlist, CString& text);
	static int RemoveCharacters(const CString& charlist, CStringArray& array); 
						// calls cstring::remove for all chars in list and all strings in array.
	static int  ReplaceCharacters(const CString& oldCharList, const CString& newChar, CString& txt);
    static int  FindAndReplaceText(CString& inText, const CString searchCriteria, const CString replaceWith, BOOL bMatchCaseCheck);    // olga XIT-855
	static void ArrayToMultiLineString(const CStringArray& array, CString& out); // for multi line edits
	static void ArrayToSingleString(const CStringArray& array, CString& out, LPCTSTR sep = _T("; ")); // ; sep:  for single text blocks.
	static void MultiLineStringToArray(LPCTSTR str, CStringArray& out); // break up a string by line breaks into an array of lines
	static BOOL WriteHTMLTable(const CStringArray& csa,CStdioFile& f); // given comma deliminated data, make HTML table to file.	

	static void Serialize(CArchive & ar, CString & str); // serializes a CString in a much more compact fashion than the default CString serialization uses

	// provides a way to force a string (literal or CString) to an ANSI string
	typedef CStringT<char,StrTraitMFC<char,ChTraitsCRT<char> > > CStringANSI;
	static CStringANSI ConvertToANSI(LPCTSTR str);

	static CString MakeStringFromChar(_TCHAR c); // creates a single-char string

	static void SortStringArray(CStringArray& csa, BOOL bDescending);
	static int CStringCompareAscending(const void *a, const void *b);
	static int CStringCompareDescending(const void *a, const void *b);
	static void RemoveDuplicateStrings(CStringArray& csa, BOOL caseSensitive);
	static BOOL CompareStringArrays(CStringArray& lhs, CStringArray& rhs, BOOL caseSensitive);
	static void AddToStringArrayIfNotAlreadyPresent(CStringArray& arr, LPCTSTR stringToAdd, BOOL caseSensitiveCompare);

	// converts between UCS-2 Unicode (2 bytes per char - TCHAR) and
	// UTF-8 Unicode (variable # bytes per char - char).
	// UCS-2 is the Windows default Unicode encoding, but UTF-8 is more popular
	// on unix environments since with UTF-8 you can use normal string handling
	// routines since there aren't embedded NULL chars in the string.  both can
	// encode the Unicode chars, they just work a little differently.
	static BOOL StringConvertUCS2toUTF8(LPCTSTR inStr, CStringA& outStr);
	static BOOL StringConvertUTF8toUCS2(LPCSTR inStr, CString& outStr);

	static void MakeStringValidCFunctionName(CString& s, TCHAR replacementChar = _T('_'));

	// limits the string length by removing chars from the middle of the string and replacing them with ellipses "..."
	// if you specify maxLength < 5 then ellipses can not be used, instead it just keeps the leftmost chars.
	// if maxLength is odd, strings will shorten to be exactly maxLength
	// if maxLength is even, strings will shorten to be 1 char less than maxLength - maxLength will never be exceeded
	static CString LimitStringLengthUsingEllipsesInTheMiddle(LPCTSTR str, DWORD maxLength);

	// format the given # of bytes in a readable form (bytes, kB, MB, or GB)
	static CString FormatNumBytes(ULONGLONG numBytes);

	static void CopyCMapStringToString(const CMapStringToString& source, CMapStringToString& dest);		// jmc 2013.06.30

	/**
	 * Extract a substring identified by the content between (and including) the specified start and end
	 * markers (case insensitive).  The search first finds the start marker (searching from the beginning
	 * of the string), then from that location looks for the first occurrence of the end marker.
	 * If the substring can not be found, an empty string is returned.
	 */
	static CString ExtractInclusiveSubstring(const CString& source, const CString& startMarker, const CString& endMarker);

};

// macros to do Unicode string conversion in a concise manner
#define STR_T2A(inStr,outStr) StringHelper::StringConvertUCS2toUTF8(inStr,outStr)
#define STR_A2T(inStr,outStr) StringHelper::StringConvertUTF8toUCS2(inStr,outStr)

