#include "stdafx.h"
#include "Transform.h"
/////////////////////////////////////////////////////////////////////////////
// Transform       
// Transform constructor.  Note:  this function is overloaded to allow for several
//	modes of initialization

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define SMALL_TRIG (1e-14)		// for checks on trig results

static double DTR = acos(-1.0) / 180.0;

double SafeSqrt(double x)
{
	if (x < 0)
		return 0.0;
	return sqrt(x);
}

double SafeACos(double x)
{
	if (x > 1.0)		// trap dot out of range.
	{
		x = 1.0;
	}
	if (x < -1.0)
	{
		x = -1.0;
	}
	return  acos(x);
}

void Transform::Scale( double s )
{
	T[0][3] *= s;
	T[1][3] *= s;
	T[2][3] *= s;
}
void Transform::SetTranslationRotation(const Vector& Trans, const Vector& Rot)
{
	SetTranslation(Trans);
	SetRotation(Rot);
}
void Transform::GetTranslationRotation(Vector& trans, Vector& rot) const
{
	GetTranslation(&trans);
	GetRotation(&rot);
}
Vector Transform::GetTranslation() const
{
	Vector v;
	GetTranslation(&v);
	return v;
}
Vector Transform::GetRotation() const
{
	Vector v;
	GetRotation(&v);
	return v;
}
void Transform::SetRotation( const Transform& R )		// just set the rotation from a source
{
	for (int i=0;i<3;i++)
		for (int j=0;j<3;j++)
			T[i][j] = R.T[i][j];
}

bool Transform::operator==(const Transform& other) const
{
	for (int i=0;i<4;i++)
		for (int j=0;j<4;j++)
			if (T[i][j]!=other.T[i][j])
				return false;
	return true;
}
bool Transform::operator!=(const Transform& other) const
{
	return !operator==(other);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

Transform Transform::GetMotionInAltFrame( const Transform& AltFrameInCurrentMotionReference ) const
{
	Transform DeltaInAlt;

	Transform CurrentMotionReferenceInAltFrame = !AltFrameInCurrentMotionReference;

	// this is really AltPrimeInAlt

	Transform AltPrimeInCurrent = *this * AltFrameInCurrentMotionReference;

	DeltaInAlt = CurrentMotionReferenceInAltFrame * AltPrimeInCurrent;
	DeltaInAlt.Normalize();		// jmc 2006.02.09 - safety -- added after reportinthis discovery.

	return DeltaInAlt;
}

void Transform::ReportMotionComponentsInThisFrame( const Transform& T1, const Transform& T2, Vector& AltTranslation, Vector& AltRxyzRotation, double& AltAngle) const
{
	// for translation, look at the origin of the ALT frame before and after the transform.
	// use the translation between the points as the answer.
	Vector PostRotOrg = T2.GetTranslation();
	Vector PreRotOrg = T1.GetTranslation();		

	Vector DeltaTrans = PostRotOrg - PreRotOrg;//In "Ground" reference

	Vector DeltaAxisInT1;
	double DeltaAngle=0.0;
	Transform T2toT1 = !T1 * T2;		// jmx 2008.05.29 - reworked multiplication order to fix issue, and remove the need for an inverse.
	T2toT1.Normalize();		// jmc 2006.02.09 -- fixed frame query reporting issue... inputs were normalized already!
 	T2toT1.GetRotation( &DeltaAxisInT1, &DeltaAngle );  // describes how to get from T2 to T1 -- so really its T2 described in T1.
	AltAngle = DeltaAngle;		// jmc - pass out for reporting purposes.
//	TRACE0("T2 in T1 - rotation axis:  "); DeltaAxisInT1.Debug();

	Vector DeltaAxisInGround;
	T1.MultiplyRot(DeltaAxisInT1, &DeltaAxisInGround);		// jmc 2008.05.29 get axis relative to ground. -- map from T1 reference 
//	TRACE0("axis in world:  "); DeltaAxisInGround.Debug();

	Vector DeltaTransMap,DeltaAxisMap;	// map the vectors from ground to working.

	Transform CurrentMotionReferenceInGround  = (*this);
	Transform GroundInCurrentMotionReference  = !CurrentMotionReferenceInGround;

	GroundInCurrentMotionReference.MultiplyRot( DeltaTrans, &DeltaTransMap );
	GroundInCurrentMotionReference.MultiplyRot( DeltaAxisInGround, &DeltaAxisMap );
//	TRACE0("axis in WORK:  "); DeltaAxisMap.Debug();

	AltTranslation = DeltaTransMap;
	Transform rotcontainer;
	rotcontainer.SetRotation(DeltaAxisMap, DeltaAngle);
	AltRxyzRotation = rotcontainer.GetRotation();
}


void Transform::ConvertBeginNormalToThetaPhiXY(Vector &Begin, Vector &Normal, double* Theta, double* Phi, double* iX, double* iY)
{
	// make a frame where z points in line normal direction.
	Vector rtp;
	Normal.XYZtoRTP(&rtp);
	Transform trans;
	trans.SetRotation(Vector(0.0,rtp.z,rtp.y));
	// now, determine the intercept of the line and the XY plane.
	Transform transinv;
	trans.Invert(&transinv);
	Vector BeginInTrans;
	transinv.Multiply(Begin,&BeginInTrans);
	// assign the variables.
	*Theta = rtp.y;
	*Phi = rtp.z;
	*iX = BeginInTrans.x;
	*iY = BeginInTrans.y;
}
void Transform::ConvertThetaPhiXYToBeginNormal(double Theta, double Phi, double iX, double iY, Vector* Begin, Vector* Normal)
{
	// make a transform using the angles.
	Transform trans;
	trans.SetRotation(Vector(0.0,Phi,Theta));
	trans.Multiply( Vector(iX,iY,0.0), Begin );
	Vector ax,ay;
	trans.GetAxes(&ax,&ay,Normal);
}

Transform::Transform( )     // identity matrix   w/zero translation         jmc
{  
	SetIdentity();
}  

void Transform::Normalize()
{
	// VERY IMPORTANT FUNCTION!
	// When there are multiple stacking multiplications it is IMPERATIVE that the
	// resulting matrix be forced to be orthonormal.  Otherwise the matrix will have
	// disturbing and unexpected effects on the data it so willingly operates on.....
	//
	Vector rot;
	GetRotation( &rot );
	SetRotation( rot );
}
#define IDENT_TOL (1.0e-6)
#define IDENT_TRANS_TOL (1.0e-8)
BOOL Transform::IsIdentity() const
{
	double tol = (1.0-IDENT_TOL);
	if ((T[0][0] > tol) && (T[1][1] > tol) && (T[2][2] > tol) &&
		(fabs(T[0][3]) < IDENT_TRANS_TOL) && (fabs(T[1][3]) < IDENT_TRANS_TOL) && (fabs(T[2][3]) < IDENT_TRANS_TOL) )
		return TRUE;
	return FALSE;
}

double Transform::MeasureOrthoNormal() const
{
	Transform temp;
	for (int i=0;i<3; i++)
		for (int j=0;j<3;j++)
			temp.T[i][j] = T[j][i];				// transpose rotation.

	Transform result;
	result.MultiplyRot( temp, *this ); 
	double total = 0.0;
	for (int i=0;i<3;i++)
		result.T[i][i] -= 1.0;
	for (int i=0;i<3;i++)
		for (int j=0;j<3;j++)
			total += fabs(result.T[i][j]);
	return total;
}


// build a rotation only matrix  		jmc
Transform::Transform( double rx, double ry, double rz )
{
	Vector v;	
	v.x = rx;
	v.y = ry;
	v.z = rz;
	SetRotation( v );
	v.x = v.y = v.z = 0.0;	
	SetTranslation( v );	// add a zero translation
	T[3][0] = 0;	T[3][1] = 0;	T[3][2] = 0;	T[3][3] = 1;     // carry row
} 
// build all 6 parameters into transformation    	jmc
Transform::Transform( double rx, double ry, double rz, double xi, double yi, double zi )
{
	Vector v;	
	v.x = rx;
	v.y = ry;
	v.z = rz;
	SetRotation( v );
	v.x = xi;
	v.y = yi;
	v.z = zi;	
	SetTranslation( v );	
	T[3][0] = 0;	T[3][1] = 0;	T[3][2] = 0;	T[3][3] = 1;     // carry row
}
// given a matrix, insert it into the class data member
Transform::Transform( double a[4][4] )
{
	Set( a );
	T[3][0] = 0;	T[3][1] = 0;	T[3][2] = 0;	T[3][3] = 1;     // carry row
} 
Transform::Transform( const Transform& a)
{
	Set( a );
	T[3][0] = 0;	T[3][1] = 0;	T[3][2] = 0;	T[3][3] = 1;     // carry row
}
//////////////////////////////////////////////////////////////////////////////////////////
// member functions 
void Transform::SetIdentity()
{
	T[0][0] = 1;	T[0][1] = 0;	T[0][2] = 0;	T[0][3] = 0;
	T[1][0] = 0;	T[1][1] = 1;	T[1][2] = 0;	T[1][3] = 0;
	T[2][0] = 0;	T[2][1] = 0;	T[2][2] = 1;	T[2][3] = 0;
	T[3][0] = 0;	T[3][1] = 0;	T[3][2] = 0;	T[3][3] = 1; 
}
void Transform::SetRotation(const  Vector& X,const  Vector& Y,const  Vector& Z )
{
	// set rotation matrix given three coordinate axis vectors.
	// assume these are normalized and orthogonal.
	T[0][0] = X.x;
	T[1][0] = X.y;
	T[2][0] = X.z;
	
	T[0][1] = Y.x;
	T[1][1] = Y.y;
	T[2][1] = Y.z;
	
	T[0][2] = Z.x;
	T[1][2] = Z.y;
	T[2][2] = Z.z; 

	Normalize();
}
void Transform::Set(const Vector& Origin,const Vector& ZDirection )
{
	// sws - XIT-643 (Filter Point Clouds radially to a vector)
   //   Previously, it was assumed that the "ZDirection" input vector was normalized.
   //   It is safer not to make this assumption and normalize an instance of "ZDirection"
   //   ("zAxis") and use that to create a well-formed transform.  This is the only
   //   significant change to this function relative to its legacy form.

	// Place a frame at the origin, with z along the "ZDirection",
	// and place x using an arbitrary cross product. 

	Vector xaxis, yaxis;

	Vector pinf(1,2,3);  // pulled out of hat.
	pinf.Normalize();		// jmc - have to normalize or dot product is meaningless....

	Vector zAxis(ZDirection);            // sws - XIT-643 (Filter Point Clouds radially to a vector)
	zAxis.Normalize();                   // sws - XIT-643 (Filter Point Clouds radially to a vector)
	
	if(fabs(zAxis.Dot(pinf))>.95)        // sws - XIT-643 (Filter Point Clouds radially to a vector)
		pinf = Vector(1,-1,-1);

	xaxis.Cross( zAxis, pinf );          // sws - XIT-643 (Filter Point Clouds radially to a vector)
	yaxis.Cross( zAxis, xaxis );         // sws - XIT-643 (Filter Point Clouds radially to a vector)
	SetRotation( xaxis, yaxis, zAxis );  // sws - XIT-643 (Filter Point Clouds radially to a vector)
	SetTranslation( Origin );
}

void Transform::SetRotation( const Vector& r )
{
   double ca,sa,cb,sb,cg,sg;    // store intermediate in double precision!
   sa = r.z*DTR;
   ca = cos(sa);	// pre-calculate the trig stuff to save re-calculation 
   sa = sin(sa);
   sb = r.y*DTR;
   cb = cos(sb);
   sb = sin(sb);
   sg = r.x*DTR;
   cg = cos(sg);
   sg = sin(sg);
   T[0][0]= ca*cb;	T[0][1]=ca*sb*sg-sa*cg; T[0][2]=ca*sb*cg+sa*sg;
   T[1][0]= sa*cb; 	T[1][1]=sa*sb*sg+ca*cg;	T[1][2]=sa*sb*cg-ca*sg;
   T[2][0]= -sb; 		T[2][1]=cb*sg;  			T[2][2]=cb*cg;  		
}
// given a Vector, the appropriate rotation matrix is inserted into the transform   jmc
void Transform::SetZYXRotation(const Vector& r )
{
	// Fixed angle ZYX convention.
   double cx,sx,cy,sy,cz,sz;    // store intermediate in double precision!
   sz = r.z*DTR;
   cz = cos(sz);	// pre-calculate the trig stuff to save re-calculation 
   sz = sin(sz);
   sy = r.y*DTR;
   cy = cos(sy);
   sy = sin(sy);
   sx = r.x*DTR;
   cx = cos(sx);
   sx = sin(sx);

   T[0][0]= cy*cz;						T[0][1]=-cy*sz;						T[0][2]=sy;
   T[1][0]= sx*sy*cz + cx*sz; 		T[1][1]=-sx*sy*sz + cx*cz;			T[1][2]=-sx*cy;
   T[2][0]= -cx*sy*cz + sx*sz; 		T[2][1]=cx*sy*sz + sx*cz;  		T[2][2]=cx*cy;  		
}

// given a Vector pointer, the point is inserted into the last row of T      jmc
void Transform::SetTranslation( const Vector& p )
{
	T[0][3] = p.x;          
	T[1][3] = p.y;
	T[2][3] = p.z;
} 
// convert a vector and an angle into a rotation matrix using angle axis convention		..jmc..
void Transform::SetRotation(const  Vector& k,const double theta)
{   
	double ThetaRad = theta*DTR;  
	double vt = (1.0 - cos(ThetaRad));
	double st = sin(ThetaRad);
	double ct = cos(ThetaRad);
	
	T[0][0]=k.x*k.x*vt+ct;		T[0][1]=k.x*k.y*vt-k.z*st;	T[0][2]=k.x*k.z*vt+k.y*st;
	T[1][0]=k.x*k.y*vt+k.z*st;	T[1][1]=k.y*k.y*vt+ct;		T[1][2]=k.y*k.z*vt-k.x*st;
	T[2][0]=k.x*k.z*vt-k.y*st;	T[2][1]=k.y*k.z*vt+k.x*st;	T[2][2]=k.z*k.z*vt+ct;	
}  
void Transform::SetDenavitHartenberg(const double& alpha, const double& a, const double& d, const double& theta)
{
	// jmc -- used for robot kinematics.
	// alpha i-1 deg
	// a i-1 length
	// d i   length
	// theta i  deg
	double ct, st, ca, sa;
	st = theta*DTR;		
	ct = cos( st );      // pre-calculate the trig stuff to save time
	st = sin( st );
	sa = alpha*DTR;		
	ca = cos( sa );
	sa = sin( sa );
	T[0][0]=ct;		T[0][1]=-st;	T[0][2]=0.0;	T[0][3]=a;
	T[1][0]=st*ca; T[1][1]=ct*ca; T[1][2]=-sa;	T[1][3]=-sa*d;
	T[2][0]=st*sa; T[2][1]=ct*sa; T[2][2]=ca;		T[2][3]=ca*d;

}

// take a matrix and insert it into the class format   ..jmc..
void Transform::Set(const  double a[4][4] )
{
	for (int i=0;i<4;i++)          // only needs to go up to 3 because last row is 0 0 0 1
		for (int j=0;j<4;j++)
			T[i][j] = a[i][j];
}   
// copy the given matrix over the current classes T		jmc  
void Transform::Set( const Transform& a )
{
	for (int r=0; r<3; r++)    		// only transfer the used rows.  if, the bottom row is used
										// 	later, the r<3 should become r<4
		for (int c=0; c<4; c++)
			T[r][c] = a.T[r][c];
}
//////////////////////////////////////////////////////////////////////////////////////////////  
// New River Kinematics   Foundation Classes v4.2   
// 
// Overloaded Set function will set the current transform to be located at point 0, with the
// X axis along the 01 vector.  Then, X is crossed with 02 to get Z.  Then Z is crossed with
// X to get the Y axis.
// The member transform is from the plane frame to the origin of the coordinate system that
// the points were given in.
void Transform::Set(const Vector p[3])
{
	// guts moved to SetOrigPtXAxisPtXYPlanePt -- clearer. jmc 2003.06.29
	SetOrigPtXAxisPtXYPlanePt(p[0],p[1],p[2]);
}  
void Transform::SetOrigPtXAxisPtXYPlanePt(const Vector& origPt, const Vector& xAxisPt, const Vector& xyPlanePt)
{
	Vector Y,Z;     // these will define the axes of the frame.  
	Vector X (xAxisPt);
	X.Sub(origPt); 
	X.Normalize();
	
	Vector v02(xyPlanePt); 
	v02.Sub(origPt);   
	Z.Cross(X, v02);
	
	Y.Cross(Z,X);

	T[0][0] = X.x;
	T[1][0] = X.y;
	T[2][0] = X.z;
	
	T[0][1] = Y.x;
	T[1][1] = Y.y;
	T[2][1] = Y.z;
	
	T[0][2] = Z.x;
	T[1][2] = Z.y;
	T[2][2] = Z.z; 

	Normalize();  // jmc 2003.01.15 -- some cases of 3 points would produce a matrix that was
						// non homogeneous (prob. identical pts).  This messed up scalingtransform when it 
						// extracted scale.....
	
	SetTranslation(origPt); 
}

// extract the rotations in the matrix and place them in a Vector  		jmc
void Transform::GetRotation( Vector *r ) const
{  
	//
	// Fixed angle XYZ
	//
	r->y = atan2( -T[2][0], SafeSqrt( T[0][0]*T[0][0] + T[1][0]*T[1][0] ) );
	double cos_ry = cos(r->y);
	if  (fabs(cos_ry) <= SMALL_TRIG)	/** prevent divide by zero. **/
	{
		if (r->y > 0.0)
		{
			r->z = 0.0;
			r->x = atan2(T[0][1] , T[1][1]);
		}
		else										 
		{
			r->z = 0.0;							 
			r->x = -atan2(T[0][1] , T[1][1]);  
		}													 
	}
	else
	{
		r->z = atan2( T[1][0] , T[0][0]  );
		r->x = atan2( T[2][1] , T[2][2]  );
	}     
	r->Scale(1.0/DTR);		// convert to degrees
}

// extract the rotations in the matrix and place them in a Vector  		jmc
void Transform::GetZYXRotation( Vector *r ) const
{  
	//
	// FIXED angle ZYX convention.  Using the NRK certified derivation.
	//
	r->y = atan2( T[0][2], SafeSqrt( T[1][2]*T[1][2] + T[2][2]*T[2][2] ) );
	double cos_ry = cos(r->y);
	if  (fabs(cos_ry) <= SMALL_TRIG)	/** prevent divide by zero. prec check rjs**/
	{
	//	TRACE(_T("in the zone!\n"));
		if (r->y > 0.0)
		{
			r->z = 0.0;
			r->x = atan2(T[1][0] , T[1][1]);
		}
		else										 ////////////////////////////////////
		{
			r->z = 0.0;							 /////////////////////////////////////////
			r->x = -atan2(T[1][0] , T[1][1]);  ///////////////////////////////////
		}													 ///////////////////////////////
	}
	else
	{
		r->z = atan2( -T[0][1] , T[0][0]  );
		r->x = atan2( -T[1][2] , T[2][2]  );
	}     
	r->Scale(1.0/DTR);		// convert to degrees
}

//////////////////////////
//Euler GetRotations
void Transform::GetXYZEulerRotation( Vector *r ) const
{
	Vector q;
	GetZYXRotation(&q);
	//q is now scaled to degrees
	 r->x = q.z;
	 r->y = q.y;
	 r->z = q.x;
}

void Transform::GetZYXEulerRotation( Vector *r ) const
{
	Vector q;
	GetXYZRotation(&q);
	//q is now scaled to degrees
	 r->x = q.z;
	 r->y = q.y;
	 r->z = q.x;
}

void Transform::GetZYZEulerRotation( Vector *r ) const
{  
	r->y = atan2( SafeSqrt( T[2][0]*T[2][0] + T[2][1]*T[2][1] ), T[2][2] );   // determine beta
	double sin_b = sin(r->y);   // determine the sin of beta.
	if  (fabs(sin_b) <= SMALL_TRIG)	// prevent divide by zero.
	{  
		r->x = 0.0;
//		if (180.0*DTR - r->y < r->y)		// if beta is closer to 180
		if(r->y-90.0*DTR<0)	// if beta is closer to 0
			r->z = atan2(-T[0][1] , T[0][0]);
		else
			r->z = atan2(T[0][1] , -T[0][0]);
	}
	else
	{
		r->x = atan2( T[1][2] , T[0][2]  );
		r->z = atan2( T[2][1] , -T[2][0]  );
	}     
	r->Scale(1.0/DTR);		// convert to degrees
}

// sws - Mantis 0007518 (add euler ZXZ)
void Transform::GetZXZEulerRotation(Vector * r)	const
{
   r->y = atan2(SafeSqrt(T[2][0]*T[2][0] + T[2][1]*T[2][1]), T[2][2]);   // Determine beta (Rx)
   double sin_b = sin(r->y);                                             // Determine the sin of beta (Rx)
   if  (fabs(sin_b) <= SMALL_TRIG)	                                     // Prevent divide by zero.
   {  
      r->x = 0.0;

      if(T[2][2] < 0)	// Check Rx for 180 (-180)
         r->z = atan2(-T[1][0], T[0][0]);
      else
         r->z = atan2(T[1][0], T[0][0]);
   }
   else
   {
      r->x = atan2( T[0][2] , -T[1][2]  );
      r->z = atan2( T[2][0] , T[2][1]  );
   }  

   r->Scale(1.0 / DTR);		// Convert to degrees
}


//////////////////////////
//Euler SetRotations
void Transform::SetXYZEulerRotation(const  Vector& r )
{
	Vector q(r.z,r.y,r.x);
	SetZYXRotation(q);
}
void Transform::SetZYXEulerRotation(const  Vector& r )
{
	Vector q(r.z,r.y,r.x);
	SetXYZRotation(q);
}
void Transform::SetZYZEulerRotation(const  Vector& r )
{
   double ca,sa,cb,sb,cg,sg;    // store intermediate in double precision!
   sa = r.x*DTR;
   ca = cos(sa);	// pre-calculate the trig stuff to save re-calculation 
   sa = sin(sa);
   sb = r.y*DTR;
   cb = cos(sb);
   sb = sin(sb);
   sg = r.z*DTR;
   cg = cos(sg);
   sg = sin(sg);
   T[0][0]= ca*cb*cg - sa*sg;	T[0][1]=-ca*cb*sg-sa*cg;	T[0][2]=ca*sb;
   T[1][0]= sa*cb*cg + ca*sg;	T[1][1]=-sa*cb*sg+ca*cg;	T[1][2]=sa*sb;
   T[2][0]= -sb*cg; 				T[2][1]=sb*sg;  				T[2][2]=cb;  		
}

void Transform::SetZXZEulerRotation(const  Vector& r )	// blt 02.28.01
{
   double ca,sa,cb,sb,cg,sg;    // store intermediate in double precision!
   sa = r.x*DTR;
   ca = cos(sa);	// pre-calculate the trig stuff to save re-calculation 
   sa = sin(sa);
   sb = r.y*DTR;
   cb = cos(sb);
   sb = sin(sb);
   sg = r.z*DTR;
   cg = cos(sg);
   sg = sin(sg);
   T[0][0]=-sa*cb*sg + ca*cg;	T[0][1]=-sa*cb*cg-ca*sg;	T[0][2]=sa*sb;
   T[1][0]= ca*cb*sg + sa*cg;	T[1][1]=ca*cb*cg-sa*sg;	T[1][2]=-ca*sb;
   T[2][0]= sb*sg; 				T[2][1]=sb*cg;  				T[2][2]=cb;  		
}

void Transform::SetZXYEulerRotation(const Vector& r )
{
	double ca,sa,cb,sb,cg,sg;    // store intermediate in double precision!
	sa = r.x*DTR;
	ca = cos(sa);	// pre-calculate the trig stuff to save re-calculation 
	sa = sin(sa);
	sb = r.y*DTR;
	cb = cos(sb);
	sb = sin(sb);
	sg = r.z*DTR;
	cg = cos(sg);
	sg = sin(sg);
	T[0][0]=-sa*sb*sg + ca*cg;	T[0][1]=-sa*cb;			T[0][2]= sa*sb*cg + ca*sg;
	T[1][0]= ca*sb*sg + sa*cg;	T[1][1]= ca*cb;			T[1][2]=-ca*sb*cg + sa*sg;
	T[2][0]=-cb*sg; 			T[2][1]= sb;  			T[2][2]= cb*cg;  	
}

Vector Transform::GetXAxis() const
{
	Vector v(T[0][0], T[1][0], T[2][0]);
	return v;
}
Vector Transform::GetYAxis() const
{
	Vector v(T[0][1], T[1][1], T[2][1]);
	return v;
}
Vector Transform::GetZAxis() const
{
	Vector v(T[0][2], T[1][2], T[2][2]);
	return v;
}


void Transform::GetAxes( Vector* x, Vector* y, Vector* z ) const
{
	x->x = T[0][0];
	x->y = T[1][0];
	x->z = T[2][0];

	y->x = T[0][1];
	y->y = T[1][1];
	y->z = T[2][1];

	z->x = T[0][2];
	z->y = T[1][2];
	z->z = T[2][2];
}
void Transform::SetAxes(const  Vector& x,const  Vector& y,const  Vector& z)
{
	T[0][0] = x.x;
	T[1][0] = x.y;
	T[2][0] = x.z;

	T[0][1] = y.x;
	T[1][1] = y.y;
	T[2][1] = y.z;

	T[0][2] = z.x;
	T[1][2] = z.y;
	T[2][2] = z.z;
	Normalize();
}


// extract the translations in the matrix and place them in a Vector		jmc
void Transform::GetTranslation( Vector *p ) const
{
	p->x = T[0][3];
	p->y = T[1][3];
	p->z = T[2][3];	
}  
void Transform::GetRTZTranslation( Vector *p ) const
{  
	Vector Temp; 
	GetTranslation( &Temp );		// get cartesian XYZ representation.
	p->z = Temp.z;		// z maps directly
	Temp.z = 0.0;
	p->x = Temp.Length();			// xy magnitude.	
	p->y = (p->x >0) ? atan2( Temp.y, Temp.x )/DTR : 0;			// theta!  NOTE: if x and y are zero, we will have problems!
}
void Transform::SetRTZTranslation(const  Vector &p )
{
	T[0][3] = p.x*cos( p.y*DTR );
	T[1][3] = p.x*sin( p.y*DTR ); 
	T[2][3] = p.z;				// maps directly.
}
void Transform::GetRTPTranslation( Vector *p ) const
{     
	Vector Temp; 
	GetTranslation( &Temp );		// get cartesian XYZ representation.
	p->x = Temp.Length();			// the radius to the point from the origin.
	p->y = (p->x > 0) ? atan2( Temp.y, Temp.x )/DTR : 0;		//    theta --  both should and better not be zero.
	p->z = (p->x > 0) ? acos( Temp.z / p->x )/DTR : 0;			//  PHI    --- divide by possibly zero!!!!
}
void Transform::SetRTPTranslation(const  Vector &p )
{
	double theta = p.y * DTR;
	double phi = p.z * DTR;
	T[0][3] = p.x * sin( phi ) * cos( theta );
	T[1][3] = p.x * sin( phi ) * sin( theta );
	T[2][3] = p.x * cos( phi ); 
}

// convert a rotation matrix into angle axis representation 		..jmc..
void Transform::GetRotation( Vector* k, double *theta) const
{
	// jmc 8-8-97 added safeacos....
	*theta = (SafeACos( (T[0][0] + T[1][1] + T[2][2] - 1.0) / 2.0) / DTR );
	if (fabs(*theta) < SMALL_TRIG)
	{
		k->x = k->y = 0.0;		// jmc - 2008.05.29 -- more properly handle undefined 0 deg case.  Just pick Z.
		k->z = 1.0;
	}
	else
	{
		k->x = T[2][1] - T[1][2];
		k->y = T[0][2] - T[2][0];
		k->z = T[1][0] - T[0][1];
		k->Normalize();  
	}
}        
   
// take the T  matrix and insert it into a 2d array   ..jmc..
void Transform::Get( double a[4][4] ) const
{
	for (int i=0;i<4;i++)        // only must go to 3... last row is 0 0 0 1
		for (int j=0;j<4;j++)
			a[i][j] = T[i][j];
}
// multiply the two input matrices together, and place result in the current class T     jmc
void Transform::Multiply( const Transform& a, const Transform& b )
{ 
	int r,c,i;
	// for each coordinate of the result matrix... 
	for (r=0; r<3; r++)   // NOTE:  does not multiply the last row.  change 3 to 4 if needed....
		for (c=0; c<4; c++)
		{	// move across cols of 1st input while moving down the rows of 2nd input
			T[r][c] = 0.0;
			for (i=0; i<4; i++)
				T[r][c] += a.T[r][i] * b.T[i][c];
		}
} 
// multiply only the two rotations together.... ignore the translations.
void Transform::MultiplyRot( const Transform& a, const Transform& b )
{ 
	int r,c,i;
	// for each coordinate of the result matrix... 
	for (r=0; r<3; r++)   // NOTE:  does not multiply the last row.  change 3 to 4 if needed....
		for (c=0; c<3; c++)
		{	// move across cols of 1st input while moving down the rows of 2nd input
			T[r][c] = 0.0;
			for (i=0; i<3; i++)
				T[r][c] += a.T[r][i] * b.T[i][c];
		}
}
// multiply vector a by the current transform and return a result vector  
// this violates the encapsulation ideal which c++ stresses.   It does, however,
//	eliminate the need for confusing the issue with friend classes.       jmc
void Transform::Multiply( const Vector& a, Vector *b ) const
{
	b->x = a.x*T[0][0] + a.y*T[0][1] + a.z*T[0][2] + T[0][3];
	b->y = a.x*T[1][0] + a.y*T[1][1] + a.z*T[1][2] + T[1][3];
	b->z = a.x*T[2][0] + a.y*T[2][1] + a.z*T[2][2] + T[2][3]; 
}
void Transform::MultiplyRot( const Vector& a, Vector *b ) const
{         
	// ignore the translation... rotate vector only!
	b->x = a.x*T[0][0] + a.y*T[0][1] + a.z*T[0][2] ;
	b->y = a.x*T[1][0] + a.y*T[1][1] + a.z*T[1][2] ;
	b->z = a.x*T[2][0] + a.y*T[2][1] + a.z*T[2][2] ; 
}
double Transform::MultiplyPartial( const Vector& a, Axis ax ) const
{
	// compute only one component of the transform-- for speed reasons.
	return ( a.x*T[ax][0] + a.y*T[ax][1] + a.z*T[ax][2] + T[ax][3] );
}

// multiply vector a by the current transform and return a result vector  
// skip the z coordinate when it is not needed.				jmc
void Transform::Multiply2D( const Vector& a, Vector *b ) const
{
	b->x = a.x*T[0][0] + a.y*T[0][1] + a.z*T[0][2] + T[0][3];
	b->y = a.x*T[1][0] + a.y*T[1][1] + a.z*T[1][2] + T[1][3];
} 
void Transform::Multiply2DRot( const Vector& a, Vector *b ) const
{
	b->x = a.x*T[0][0] + a.y*T[0][1] + a.z*T[0][2];
	b->y = a.x*T[1][0] + a.y*T[1][1] + a.z*T[1][2];
}


void Transform::Invert( Transform* Inv ) const
{
	int r,c;
	for (r=0;r<3;r++)       /* transpose rotation component of transform */
		for (c=0;c<3;c++)
	   	Inv->T[r][c] = T[c][r];
	
	/* rotate the xyz translations in the transform */
	Inv->T[0][3] = - (T[0][3]*Inv->T[0][0] + T[1][3]*Inv->T[0][1] + T[2][3]*Inv->T[0][2]);
	Inv->T[1][3] = - (T[0][3]*Inv->T[1][0] + T[1][3]*Inv->T[1][1] + T[2][3]*Inv->T[1][2]);
	Inv->T[2][3] = - (T[0][3]*Inv->T[2][0] + T[1][3]*Inv->T[2][1] + T[2][3]*Inv->T[2][2]);
	Inv->T[3][0] = Inv->T[3][1] = Inv->T[3][2] = 0;        Inv->T[3][3] = 1;
}
void Transform::TransposeRot( )
{
	Transform temp;
	temp.Set( *this );
	int r,c;
	for (r=0;r<3;r++)       /* transpose rotation component of transform */
		for (c=0;c<3;c++)
	   	T[r][c] = temp.T[c][r];
}
  
// rotate the transform by the specified amount.  

void Transform::Rotate( double x, double y, double z )
{
	Transform r(x,y,z);		// build a rotation matrix for the adjustment.
	Vector pos;
	GetTranslation( &pos );   // save translation.
	Transform old;
	old.Set( *this );      // make a copy of transform   
	Multiply( r, old );		// multiply by rotation matrix.    
	SetTranslation( pos );        // replace with origional translation.
}

void Transform::Rotate(  Vector& k, double theta)
{
	Transform r;
	r.SetRotation(k,theta);		// build a rotation matrix for the adjustment.
	Vector pos;
	GetTranslation( &pos );   // save translation.
	Transform old;
	old.Set( *this );      // make a copy of transform   
	Multiply( r, old );		// multiply by rotation matrix.    
	SetTranslation( pos );        // replace with origional translation.
}

Transform& Transform::operator=(const Transform& r)
{  
	Set(r);
	return *this;
}

Transform Transform::operator*(const Transform& other) const
{
	Transform Trans;
	Trans.Multiply(*this,other);
	return Trans;
}
Vector Transform::operator*(const Vector& p) const
{
	Vector TP;
	Multiply(p,&TP);
	return TP;
}

Transform Transform::operator!() const
{
	Transform Trans;
	Invert(&Trans);
	return Trans;
}

  
void Transform::Debug() const
{
#ifndef _DEBUG
	return;
#endif
	Vector pos,rot;
	GetTranslation(&pos);
	GetRotation(&rot);
	TRACE(_T("Trans (%f, %f, %f)  Rotation (%f, %f, %f)\n"),
		pos.x,pos.y,pos.z,rot.x,rot.y,rot.z);
	
}
// sws - allow a tag string in the debug output
void Transform::Debug(const CString & idStr) const
{
#ifndef _DEBUG
   return;
#endif
   Vector pos,rot;
   GetTranslation(&pos);
   GetRotation(&rot);
   TRACE(_T("%s: Trans (%f, %f, %f)  Rotation (%f, %f, %f)\n"),
      idStr, pos.x,pos.y,pos.z,rot.x,rot.y,rot.z);

}
void Transform::DebugMatrix() const
{    
#ifndef _DEBUG
	return;
#endif
	int dec = 12;
	for (int r=0;r<4;r++)
	{
		for (int c=0;c<4;c++) TRACE(_T("  %.*f  "),dec,T[r][c]);
		TRACE(_T("\n"));
	}         
}
// sws - allow a tag string in the debug output
void Transform::DebugMatrix(const CString & idStr) const
{    
#ifndef _DEBUG
   return;
#endif
   int dec = 4;
   TRACE(_T("%s:\n"), idStr);
   for (int r=0;r<4;r++)
   {
      for (int c=0;c<4;c++) TRACE(_T("  %.*f  "),dec,T[r][c]);
      TRACE(_T("\n"));
   }         
}
void Transform::Perturb( BYTE index, double dPos, double dRot )
{
	// perturb the generator transform 
	Transform Move;
	Vector rot;
	switch( index )
	{
	case 0:	  // translation perturb
		T[0][3] += dPos;
		break;
	case 1:
		T[1][3] += dPos;
		break;
	case 2:
		T[2][3] += dPos;
		break;
	case 3:		// neg translation perturb,
		T[0][3] += -dPos;
		break;
	case 4:
		T[1][3] += -dPos;
		break;
	case 5:
		T[2][3] += -dPos;
		break;
	case 6:
		rot.x = dRot;
		break;
	case 7:
		rot.y = dRot;
		break;
	case 8:
		rot.z = dRot;
		break;
	case 9:
		rot.x = -dRot;
		break;
	case 10:
		rot.y = -dRot;
		break;
	case 11:
		rot.z = -dRot;
		break;
	}
	if (index > 5 )
	{
		Move.SetRotation( rot );
		// now, we have the move transform.
		Transform Old( *this );
		Multiply( Move, Old );		// perturb.
		Normalize();
	}
}
  
void Transform::operator+=(const Transform& trans)
{
	for (int i=0;i<4;i++)
		for (int j=0;j<4;j++)
			T[i][j] += trans.T[i][j];
}
void Transform::ScaleAllElements(const double& scale)
{
	for (int i=0;i<4;i++)
		for (int j=0;j<4;j++)
			T[i][j] *= scale;
}

bool Transform::IsSameOrientation(const Transform& otherFrame, const double toleranceDeg) const
{
	if (operator==(otherFrame))
		return true;

	Transform delta = otherFrame * !(*this);
	Vector axis;
	double angle=0;
	delta.GetRotation(&axis, &angle);
	if (fabs(angle) < toleranceDeg)
		return true;
	return false;
}

bool Transform::IsSameAxesIgnoringSign(const Transform& otherFrame, const double toleranceDeg) const
{
	if (fabs( GetXAxis().AngleBetweenIgnoreSign( otherFrame.GetXAxis() ) ) > toleranceDeg)
		return false;
	if (fabs( GetYAxis().AngleBetweenIgnoreSign( otherFrame.GetYAxis() ) ) > toleranceDeg)
		return false;
	if (fabs( GetZAxis().AngleBetweenIgnoreSign( otherFrame.GetZAxis() ) ) > toleranceDeg)
		return false;

	return true;
}

void Transform::MakeMatrixLeftHanded()
{
	// jmc - flip Y column so that we have a left handed frame.
	// note - if you do a get rotation, and a set rotation, the results
	//   will not be left handed.  Multiplication, inversion, etc are valid, however.
	//   Throughout SA, frame are serialized right handed, then temporairly converted
	//   to left hand notation (with this function) when needed.
	T[0][1] = -T[0][1];
	T[1][1] = -T[1][1];
	T[2][1] = -T[2][1];
}

void Transform::MirrorAboutXYPlaneButKeepRightHanded()
{
	// jmc - when we mirror about XY plane, we just reverse the Z sign on the translation.
	// Then, we flip the Z sign on all of the axes.  This has the effect of making the frame left handed.
	// since we don't every want to store left handed frames, we then reverse the Y axis to switch it back to right handed.
	// applications like SA can then call the MakeMatrixLeftHanded call to get an operable matrix.
	T[2][0] = -T[2][0];	// z component of X axis
	T[2][1] = -T[2][1];	// z component of Y axis
	T[2][2] = -T[2][2];	// z component of Z axis
	// now, flip the Y back:
	MakeMatrixLeftHanded();

	// now, deal with the translation:
	T[2][3] = -T[2][3];
}
void Transform::FlipZAxisWhilePreservingXAxis()
{
	// jmc - used for flipping planes, for example while keeping them right handed.
	Vector x,y,z;
	GetAxes(&x,&y,&z);
	z.Reverse();
	y.Reverse();	// keep it right handed.
	SetAxes(x,y,z);
}

// sws - XIT-355 (ellipse geometry fit not handling target offset correctly)
void Transform::FlipZAxisWhilePreservingYAxis()
{
   Vector x,y,z;
   GetAxes(&x,&y,&z);
   z.Reverse();
   x.Reverse();	// keep it right handed.
   SetAxes(x,y,z);
}; 

void Transform::GetRotationDeltasAsAngleAxisDeg(Vector& rotationErrorsDeg) const
{
	// jmc - for optimization where we need to drive out a rotation delta.  Convert to angle-axis,
	// then multiply each axis component by the total angle of rotation
	double angleDeg = 0;
	GetRotation(&rotationErrorsDeg,&angleDeg);
	rotationErrorsDeg.Scale(angleDeg);
}

void Transform::SetRotationZThenClockX(const Vector& zAxis, const Vector& xAxisClock)
{
	Vector x,y,z;
	z = zAxis;
	z.Normalize();
	x = xAxisClock;
	x.Normalize();
	y = z * x;
	x = y * z;
	SetAxes(x,y,z);
}
void Transform::SetRotationZThenClockY(const Vector& zAxis, const Vector& yAxisClock)
{
	Vector x,y,z;
	z = zAxis;
	z.Normalize();
	y = yAxisClock;
	y.Normalize();
	x = y * z;
	y = z * x;
	SetAxes(x,y,z);
}
