#pragma once


class Transform 
{                                       
public:
	double T[4][4];	// holds a transform.  public for quick accesss if necessary 
	Transform( );		// constructors
	Transform( double rx, double ry, double rz ) ;
	Transform( double rx, double ry, double rz, double xi, double yi, double zi );  
	Transform( double a[4][4] ); 
	Transform( const Transform& a);
	bool operator==(const Transform& other) const;
	bool operator!=(const Transform& other) const;
	virtual ~Transform() {}

	Transform& operator=(const Transform& trans);
	Transform operator*(const Transform& other) const;  // beware! Returns by value... efficiency alert... but its Sooooo convienient
	Vector operator*(const Vector& p) const; // beware! Returns by value... efficiency alert... but its Sooooo convienient
	Transform operator!() const;  // beware! Returns by value... efficiency alert... but its Sooooo convienient
	void operator+=(const Transform& trans);		// just add all cells!!  (used for transformarray average function)


	Vector GetTranslation() const;		// inefficient -- returns by value
	Vector GetRotation() const;			// inefficient -- returns by value
	void SetTranslationRotation(const Vector& Trans, const Vector& Rot);
	void GetTranslationRotation(Vector& trans, Vector& rot) const;
	bool IsSameOrientation(const Transform& otherFrame, const double toleranceDeg=1.0) const;
	bool IsSameAxesIgnoringSign(const Transform& otherFrame, const double toleranceDeg=1.0) const;	// OK if opposite directions (LEFT HANDED FRAMES)

	static void ConvertBeginNormalToThetaPhiXY(Vector &Begin, Vector &Normal, double* Theta, double* Phi, double* iX, double* iY);
	static void ConvertThetaPhiXYToBeginNormal(double Theta, double Phi, double iX, double iY, Vector* Begin, Vector* Normal);

	virtual void Normalize();
   virtual void Invert( Transform* Inv ) const;

	enum Axis
	{
		X = 0,
		Y = 1,
		Z = 2
	};
	double MultiplyPartial(const Vector& input, Axis ax) const;		// compute either X,Y,Z -- for efficiency.

	void Scale( double s);
	BOOL IsIdentity() const;
	void Perturb( BYTE index, double , double );
	double MeasureOrthoNormal() const;
	void TransposeRot( );
	void SetIdentity();		// clear the matrix!
	void Set(const Vector& Origin,const Vector& ZDirection );
	void SetTranslation( const Vector& p );  
	void SetRTZTranslation(const Vector& p ); 
	void SetRTPTranslation(const Vector& p ); 
	void Set(const double a[4][4] );		// take a matrix and insert it into the class
	void Set(const Vector p[3]);
	void SetDenavitHartenberg(const double& alpha, const double& a, const double& d, const double& theta);
	void SetHayatiRoberts(const double& alpha, const double& beta, const double& l, const double& m, const double& theta);

	void SetOrigPtXAxisPtXYPlanePt(const Vector& origPt, const Vector& xAxisPt, const Vector& xyPlanePt);
	void GetAxes( Vector* x, Vector* y, Vector* z ) const;
	Vector GetXAxis() const;
	Vector GetYAxis() const;
	Vector GetZAxis() const;
	void SetAxes(const Vector& x,const Vector& y,const Vector& z);
	void Rotate(  Vector& k, double theta);
	void GetTranslation( Vector* p ) const; 
	void GetRTZTranslation( Vector* p ) const; 
	void GetRTPTranslation( Vector* p ) const; 
	/////////
	void GetRotation( Vector* k, double* theta) const;	   // for angle axis convention
	void GetRotation( Vector* r )	const;	
	void GetXYZRotation( Vector *r ) const//inline funct to do xyz getrotation
		{GetRotation(r);	}	
	void GetZYXRotation( Vector *r ) const;
	void GetXYZEulerRotation( Vector *r ) const;
	void GetZYXEulerRotation( Vector *r ) const;
	void GetZYZEulerRotation( Vector *r ) const;
   void GetZXZEulerRotation( Vector *r ) const;  // sws - Mantis 0007518 (add euler ZXZ)

	void SetRotation(const Vector& k,const double theta);	// for angle axis convention.

	void SetRotation( const Vector& r )	;
	void SetXYZRotation(const Vector &r ) //inline funct to do xyz setrotation
		{SetRotation(r);	}
	void SetZYXRotation(const Vector &r );
	void SetRotation(const Vector& X,const Vector& Y,const Vector& Z );  //be careful, no ortho check
	void SetRotation( const Transform& R);

	void SetRotationZThenClockX(const Vector& zAxis, const Vector& xAxisClock);	// z matches z, xClock and z are crossed to make y, then y and z clocked to make proper x
	void SetRotationZThenClockY(const Vector& zAxis, const Vector& yAxisClock);	// z matches z, yClock and z are crossed to make x, then x and z clocked to make proper y

	void SetXYZEulerRotation(const Vector& r );
	void SetZYXEulerRotation(const Vector& r );
	void SetZYZEulerRotation(const Vector& r );
	void SetZXZEulerRotation(const Vector& r );	// blt 02.28.01
	void SetZXYEulerRotation(const Vector& r );
	
	// function for expressing and applying delta transformation in various coordinate frames
	// note:  this will normalize the matrix, so you cannot pass a ScalingTransform in!
	Transform GetMotionInAltFrame( const Transform& AltFrameInCurrentMotionReference ) const; // "this" assumed to be a delta transformation in current, not a frame

	// function for reporting delta transformations in other frames... you CANNOT used the trans and rot this creates to
	// apply transformations.. this is only to make things clear to the user.  Angle passed out for reporting -- but no different in this frame or that.
	void ReportMotionComponentsInThisFrame( const Transform& T1, const Transform& T2, Vector& AltTranslation, Vector& AltRxyzRotation, double& AltAngle) const;

	void Get( double a[4][4] ) const;	// fill a with the T data
	void Multiply(const Transform& a, const Transform& b ); // multiply two other transforms -- result is T
	void MultiplyRot(const Transform& a,const Transform& b ); // mult rotations of two other transforms -- result is T
	void Set( const Transform& a );
	void Multiply( const Vector& a, Vector *b ) const;	// multiply a point by a transform
	void Multiply2D( const Vector& a, Vector *b ) const;	// skip the z coord -- not used on screen
	void MultiplyRot( const Vector& a, Vector *b ) const;	// multiply a point by a ROTATION
	void Multiply2DRot( const Vector& a, Vector *b ) const;	// 2D by a ROTATION.
   void Rotate( double x, double y, double z );
	void DebugMatrix()const;
   void DebugMatrix(const CString & idStr)const;    // sws - allow a tag string in the debug output
   void ScaleAllElements(const double& scale);
	void MakeMatrixLeftHanded();		// jmc - negate the Y column.  NOTE: get and set rotations will undo this...
	void MirrorAboutXYPlaneButKeepRightHanded();		// jmc - for mirror function in SA.
	void GetRotationDeltasAsAngleAxisDeg(Vector& rotationErrorsDeg) const;	// jmc - used for optimization when we need rotation error components in a residual column
	void FlipZAxisWhilePreservingXAxis();		// jmc - for things like flipping plane normals.
	void FlipZAxisWhilePreservingYAxis();     // sws - XIT-355 (ellipse geometry fit not handling target offset correctly)


	void Debug()const;
   void Debug(const CString & idStr) const;    // sws - allow a tag string in the debug output
	
};  
