#include "stdafx.h"
#include "Vector.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static double DTR = acos(-1.0) / 180.0;

/////////////////////////////////////////////////////////////////////////////
// Vector

// Vector constructor.  Note:  this function is overloaded to allow for several
//	modes of initialization                     jmc
#define SMALL_TRIG (1e-14)		// for checks on trig results

void Vector::SetMax(Vector &v)
{
	x = max( x, v.x );
	y = max( y, v.y );
	z = max( z, v.z );
}
void Vector::SetMin(Vector &v)
{
	x = min( x, v.x );
	y = min( y, v.y );
	z = min( z, v.z );
}
void Vector::Abs()
{
	x = fabs(x);
	y = fabs(y);
	z = fabs(z);
}
void Vector::Square()
{
	x = x*x;
	y = y*y;
	z = z*z;
}

void Vector::Zero()
{
	x=y=z=0;
}
bool Vector::IsZero() const
{
	if (x==0.0 && y==0.0 && z==0.0)
		return true;
	return false;
}
// member functions 
double Vector::Length()const         // magnitude calculation    jmc  // possibly add a close to zero number!
{               
	double temp =  x*x + y*y + z*z;
	if (temp==0.0) return 0.0;
	return (double) sqrt( temp );	
} 
void Vector::SetMid(Vector& A, Vector& B)
{
	operator=(A);
	Add(B);
	Scale(0.5);
}

void Vector::Scale( const double& s )       // multiply all axes by scalar    jmc
{
	x *= s;
	y *= s;
	z *= s;
}
void Vector::ScaleFromOrigin(const double& scale, const Vector& scalingOrigin)
{
	if (scale==1.0)
		return;

	x = (x - scalingOrigin.x)*scale + scalingOrigin.x;
	y = (y - scalingOrigin.y)*scale + scalingOrigin.y;
	z = (z - scalingOrigin.z)*scale + scalingOrigin.z;
}

bool Vector::operator==(const Vector& other) const
{
	if (x != other.x)
		return false;
	if (y != other.y)
		return false;
	if (z != other.z)
		return false;

	return true;
}
bool Vector::operator!=(const Vector& other) const
{
	return !operator==(other);
}

double Vector::DistanceAlongAxisToPoint(const Vector& begin,const Vector& end) const
{
// Calculates distance to point from an infinite line defined by begin and end points
	Vector VLine(end);
	VLine.Sub(begin);                   // get vector along line
	VLine.Normalize();						// Normalize it
	Vector VPoint(*this);
	VPoint.Sub(begin);                  // get vector from pt on line to point in question
	return(VLine.Dot(VPoint));     		// return length of VPoint in VLine direction   may be neg
}
                       
void Vector::Normalize()
{
	double l = Length();
	if (l!=0.0)
	{
		x /= l;
		y /= l;
		z /= l;
	}
	else		// jmc 8-8-97
	{			// pick something!
		x=0.0;
		y=0.0;
		z=1.0;
	}

}
void Vector::CrossProduct(const Vector& a, const Vector& b)
{  
	x = a.y*b.z - b.y*a.z;
	y = -(a.x*b.z - b.x*a.z);
	z = a.x*b.y - b.x*a.y; 
}   
// return the normalized cross procudt of two vectors.
void Vector::Cross(const Vector& a, const Vector& b)
{  
	CrossProduct( a, b );
	Normalize();
}   
void Vector::Cross(const Vector& a, const Vector& b, double* pAngle)
{  
	CrossProduct( a, b );
	*pAngle = (asin(Length())/(a.Length()*b.Length())) / DTR;
	Normalize();
}  
double Vector::AngleBetween( Vector& v )
{
	Vector axis;
	return AngleBetween( v, &axis );
}
double Vector::AngleBetweenIgnoreSign( Vector& v )
{
	Vector axis;
	Vector temp(v);
	if (Dot(temp)<0)
		temp.Scale(-1.0);

	return AngleBetween( temp, &axis );
}
double Vector::AngleBetween( Vector& v, Vector* axis )
{
	Vector a( *this );
	Vector b( v );
	a.Normalize();
	b.Normalize();
	axis->CrossProduct( a, b );
	double dot = a.Dot( b );

	double temp =  (atan2( axis->Length(), dot ) / DTR);
	axis->Normalize();
	return temp;
}
void Vector::AnglesBetween(const Vector& other, double& horizontalAngleDelta, double& verticalAngleDelta) const
{
	// jmc - same as the theodolite eval code - compare 2 free vectors and return the components of angle between them.
	Vector aRTP, bRTP;
	XYZtoRTP(&aRTP);
	other.XYZtoRTP(&bRTP);

	verticalAngleDelta = aRTP.z - bRTP.z;		// for vertical, straight subtraction.

	// for horizontal, need to planize the vectors first then compare.
	Vector aInPlane(x, y, 0.0);
	Vector bInPlane(other.x, other.y, 0.0);
	Vector up;
	horizontalAngleDelta = aInPlane.AngleBetween(bInPlane,&up);
	if (up.z<0.0)
		horizontalAngleDelta *= -1.0;
}

double Vector::Dot(const Vector& a) const
{                                     
	return (a.x*x + a.y*y + a.z*z);	
} 
// subtract a vector from the current value
void Vector::Sub( const Vector &a)
{
	x -= a.x;
	y -= a.y;
	z -= a.z;
}
// add a vector to the current value
void Vector::Add( const Vector &a)
{
	x += a.x;
	y += a.y;
	z += a.z;
}   
// determine the distance between the current point and v
double Vector::Distance(const Vector& v) const
{
	Vector a(x,y,z);
	a.Sub(v);
	return (a.Length());
}
double Vector::DistanceSquared(const Vector& v) const
{
	Vector a(x,y,z);
	a.Sub(v);
	return (a.x*a.x + a.y*a.y + a.z*a.z);
}
bool Vector::CompareVectors(const Vector& v, double tol) // olga mantis #7509
{
    // return true if difference between two vectors is within tolerance
    if (x - v.x > tol)
        return false;
    if (y - v.y > tol)
        return false;
    if (z - v.z > tol)
        return false;

    return true;
}
//#define TOO_SMALL (1e-6)
void Vector::XYZtoRTP( Vector* rtp )const
{     
	rtp->x = Length();			// the radius to the point from the origin.
	if (rtp->x < SMALL_TRIG)		// jmc 2003.03.19
	{
		*rtp = Vector();
		return;
	}

	if (fabs(x)< SMALL_TRIG && fabs(y)< SMALL_TRIG)
	{
		if (z >= 0.0)		// added both cases in lynchburg.
			rtp->z = 0.0;
		else
			rtp->z = 180.0;
		rtp->y = 0.0;			 // pick one (singular)
		return;
	}
	rtp->y = atan2( y, x )/DTR;		//    theta --  both should and better not be zero.
	rtp->z = acos( z / rtp->x )/DTR;			//  PHI    --- divide by possibly zero!!!!
}
void Vector::RTPtoXYZ( Vector* xyz ) const
{                        
	// convert angles and a distance to an xyz point.
	// used to determine the plane orientation!
	double theta = y * DTR;
	double phi = z * DTR;
	xyz->x = x * sin( phi ) * cos( theta );
	xyz->y = x * sin( phi ) * sin( theta );
	xyz->z = x * cos( phi ); 
}
void Vector::XYZtoRTZ( Vector* rtz ) const
{     
	//rtz->x = SafeSqrt(x*x+y*y); //R
	//if (fabs(x)< SMALL_TRIG && fabs(y)< SMALL_TRIG)
	//	rtz->y = 0.0;			 // can't tell theta
	//else
	//	rtz->y = atan2( y, x )/DTR;		//    theta --  both should and better not be zero.
	//rtz->z =  z ;			//  Z
}
void Vector::RTZtoXYZ( Vector* xyz ) const
{                        
	// convert angles and a distance to an xyz point.
	// used to determine the plane orientation!
	double theta = y * DTR;
	xyz->x = x * cos( theta );
	xyz->y = x * sin( theta );
	xyz->z = z; 
}
void Vector::Reverse()
{
	// jmc 2014.05.01 - use this instead of .Scale(-1.0)
	x = -x;
	y = -y;
	z = -z;
}

void Vector::Perturb( BYTE index, double d )
{
	// perturb the generator transform 
	switch( index )
	{
	case 0:	  
		x += d;
		break;
	case 1:
		y += d;
		break;
	case 2:	  
		z += d;
		break;
	case 3:	  
		x -= d;
		break;
	case 4:
		y -= d;
		break;
	case 5:	  
		z -= d;
		break;
	}
}
     
void Vector::MultiplyComponents(const Vector& v)
{
	x*=v.x;
	y*=v.y;
	z*=v.z;
}
Vector& Vector::operator*=(const double scale)
{
	Scale(scale);
	return *this;
}
Vector& Vector::operator/=(const double scale)
{
	ASSERT(scale!=0.0);
	Scale(1.0/scale);
	return *this;
}
Vector& Vector::operator+=(const Vector& r)
{
	Add(r);
	return *this;
}
Vector& Vector::operator-=(const Vector& r)
{
	Sub(r);
	return *this;
}
Vector Vector::operator*(const Vector& r) const
{
	Vector v;
	v.Cross(*this,r);
	return v;
}
Vector Vector::operator^(const Vector& r) const
{
	Vector v;
	v.CrossProduct(*this,r);
	return v;
}
Vector Vector::operator*(const double scale) const
{
	Vector v(*this);
	v.Scale(scale);
	return v;
}
Vector Vector::operator/(const double denom) const
{
	Vector v(*this);
	if (denom==0.0)
		return v;
	double scale = 1.0/denom;
	v.Scale(scale);
	return v;
}
void Vector::Debug() const
{   
	TRACE(_T("  ( %f, %f, %f )\n"),x,y,z);
}

// sws - allow a tag string in the debug output
void Vector::Debug(const CString & idStr) const
{   
   TRACE(_T("%s:  ( %f, %f, %f )\n"),idStr,x,y,z);
}

bool Vector::ZeroComponentsIfLessThanAbs( const double& threshold )
{
	bool hit = false;
	if (fabs(x)<threshold)
	{
		x = 0.0;
		hit = true;
	}
	if (fabs(y)<threshold)
	{
		y = 0.0;
		hit = true;
	}	
	if (fabs(z)<threshold)
	{
		z = 0.0;
		hit = true;
	}	
	return hit;
}

Vector Vector::GetProjectedAnglesInDeg(bool angleRepresentation360) const
{
	// jmc - this version allows you to pass in a preference then have the angle unwrapped as specified.
	Vector ang = GetProjectedAnglesInDeg();
	ang.UnwrapAnglesByPreference(angleRepresentation360);
	return ang;
}

Vector Vector::GetProjectedAnglesInDeg() const
{
	// jmc - make sure it matches the GetProjectedAngleDescriptions function....

	// normalize first -- for safety...
	Vector normThis(x,y,z);
	normThis.Normalize();
	normThis.ZeroComponentsIfLessThanAbs(SMALL_TRIG);

	Vector angles; 
	angles.x = atan2(normThis.z,normThis.y);	// Angle from +Y about +X in YZ plane
	angles.y = atan2(normThis.x,normThis.z);	// Angle from +Z about +Y in XZ plane
	angles.z = atan2(normThis.y,normThis.x);	// Angle from +X about +Z in XY plane
	angles /= DTR;

	return angles;
}
void Vector::GetProjectedAngleDescriptions(bool brief, CString& xTxt, CString& yTxt, CString& zTxt)
{
	// jmc - matches the getprojangles function so we can change descriptions and order easily.
	if (brief)
	{
		xTxt = _T("Rx from Y");
		yTxt = _T("Ry from Z");
		zTxt = _T("Rz from X");
	}
	else
	{
		xTxt = _T("Angle about +X from +Y in YZ plane");
		yTxt = _T("Angle about +Y from +Z in XZ plane");
		zTxt = _T("Angle about +Z from +X in XY plane");
	}
}
void Vector::UnwrapAnglesByPreference(bool angleRepresentation360)
{
	//UnwrapAngleByPreference(&x,angleRepresentation360);
	//UnwrapAngleByPreference(&y,angleRepresentation360);
	//UnwrapAngleByPreference(&z,angleRepresentation360);
}

// sws - XIT-689 (Geometry uncertainty computation improvement)
Vector & Vector::AngularDelta(Vector & a, Vector & b)
{
   *this = a - b;

   while (x < -180)
      x += 360;

   while (x > 180)
      x -= 360;

   while (y < -180)
      y += 360;

   while (y > 180)
      y -= 360;

   while (z < -180)
      z += 360;

   while (z > 180)
      z -= 360;

   return (*this);
}

void Vector::GetAsStrings(CStringArray& pieces, bool fullPrec /*= true */)
{
	CString m;
	if (fullPrec)
	{
		m.Format(_T("%.16e"),x);
		pieces.Add(m);
		m.Format(_T("%.16e"),y);
		pieces.Add(m);
		m.Format(_T("%.16e"),z);
		pieces.Add(m);
	}
	else
	{
		m.Format(_T("%f"),x);
		pieces.Add(m);
		m.Format(_T("%f"),y);
		pieces.Add(m);
		m.Format(_T("%f"),z);
		pieces.Add(m);
	}
}
bool Vector::SetFromStrings(const CStringArray& pieces, int startingIdx /*= 0*/)
{
	if ((startingIdx+2) > pieces.GetUpperBound())
		return false;
	int idx=0;
	Vector temp;

	int countx = _stscanf(pieces[startingIdx++],_T("%lf"), &temp.x);
	if (countx!=1)
		return false;

	int county = _stscanf(pieces[startingIdx++],_T("%lf"), &temp.y);
	if (county!=1)
		return false;
	
	int countz = _stscanf(pieces[startingIdx++],_T("%lf"), &temp.z);
	if (countz!=1)
		return false;

	operator=(temp);

	return true;
}

bool operator<(const Vector& lhs, const Vector& rhs)
{
   if (lhs.x < rhs.x)
      return true;
   else if (lhs.x > rhs.x)
      return false;
   else
   {
      if (lhs.y < rhs.y)
         return true;
      else if (lhs.y > rhs.y)
         return false;
      else
      {
         if (lhs.z < rhs.z)
            return true;
         else
            return false;
      }
   }
}
