#pragma once

class Vector 
{                                       
public:
	double x;
	double y;
	double z;		// data members.  xyz vector coordinates 
   Vector() { x = y = z = 0; }  // Inline functions introduced with addition of NRKVoxel     
   Vector( const Vector& a) { x = a.x; y = a.y; z = a.z; }  // Inline functions introduced with addition of NRKVoxel
	Vector( double xi, double yi, double zi ) { x = xi; y = yi; z = zi; }  // Inline functions introduced with addition of NRKVoxel
	virtual ~Vector(void) {}
   virtual Vector& operator=(const Vector& r)  { x = r.x; y = r.y; z = r.z; return (*this); }  // Inline functions introduced with addition of NRKVoxel	
   Vector operator+(const Vector& other) const { return (Vector(x + other.x, y + other.y, z + other.z)); } // Inline functions introduced with addition of NRKVoxel
   Vector operator-(const Vector& other) const { return (Vector(x - other.x, y - other.y, z - other.z)); } // Inline functions introduced with addition of NRKVoxel
	Vector operator*(const Vector& other) const;		// beware! returns by value.
	Vector operator^(const Vector& other) const;		// cross wo/normalize beware! returns by value.
	Vector operator*(const double scale) const;				// beware! returns by value.
	Vector operator/(const double scale) const;				// beware! returns by value.
	Vector& operator+=(const Vector& other);		
	Vector& operator-=(const Vector& other);	
	Vector& operator*=(const double scale);
	Vector& operator/=(const double scale);
	bool operator==(const Vector& other) const;
	bool operator!=(const Vector& other) const;
	
	void Abs();		// sets each component to fabs(component)
	void Square();	// squares each component

	void SetMax(Vector &v);
	void SetMin(Vector &v);
	void SetMid(Vector& A, Vector& B);
	double AngleBetween( Vector& v, Vector* axis );
	double AngleBetween( Vector& v );
	double AngleBetweenIgnoreSign( Vector& v );
	void AnglesBetween(const Vector& other, double& horizontalAngleDelta, double& verticalAngleDelta) const;	// jmc - for theodolite evals
	double Length() const; 
	void Normalize();
	void Cross( const Vector& a, const Vector& b);    
	void CrossProduct( const Vector& a, const Vector& b);    
	void Cross( const Vector& a, const Vector& b, double* pAngle);    
	double Dot(const Vector& a) const;
	void Sub( const Vector& a );
	void Add( const Vector& a );                 
	double Distance(const Vector& v) const;  // magnitude of delta
	double DistanceSquared(const Vector& v) const;		// for cases where we don't want to always compute sqrt... just compare to tol*tol
	double DistanceAlongAxisToPoint(const Vector& begin,const Vector& end) const;
	void XYZtoRTP( Vector* rtp )const;
	void RTPtoXYZ( Vector* xyz )const;
	void XYZtoRTZ( Vector* rtz )const;
	void RTZtoXYZ( Vector* xyz )const;
	void Perturb( BYTE index, double d );
	void Reverse();		// jmc - same as .Scale(-1), but does not multiply.
	bool ZeroComponentsIfLessThanAbs(const double& threshold);
	void Zero();		// set all components to zero.
	bool IsZero() const;	// check by ==0

	Vector GetProjectedAnglesInDeg() const;
	Vector GetProjectedAnglesInDeg(bool angleRepresentation360) const;
	static void GetProjectedAngleDescriptions(bool brief, CString& xTxt, CString& yTxt, CString& zTxt);
	void UnwrapAnglesByPreference(bool angleRepresentation360);	// jmc - assumes xyz are degrees.  true means 0-360, false -+ 180
   Vector & AngularDelta(Vector & a, Vector & b); // sws - XIT-689 (Geometry uncertainty computation improvement)
	void MultiplyComponents(const Vector& v);		// x *=v.x, etc...

	void Scale(const double& scale);	// multiply the vector by a scalar
	void ScaleFromOrigin(const double& scale, const Vector& scalingOrigin);		// jmc

	void GetAsStrings(CStringArray& pieces, bool fullPrec = true);	// jmc - add to this array the .formats for each components.
	bool SetFromStrings(const CStringArray& pieces, int startingIdx = 0);	// jmc - extract from string array pieces. (usually for CSV)
    bool CompareVectors(const Vector& v, double tol);      // olga mantis #7509 - compare two vectors. return true if within predefined tolerance

    void Debug() const;	
    void Debug(const CString & idStr) const;	// sws - allow a tag string in the debug output

    // Inline functions introduced with addition of NRKVoxel

   void InlineReverse() { x = -x; y = -y; z = -z; }
   void InlineScale(const double & scale) { x *= scale; y *= scale; z *= scale; }
   void InlineSub( const Vector & a ) { x -= a.x; y -= a.y; z -= a.z; }
   void InlineAdd( const Vector & a ) { x += a.x; y += a.y; z += a.z; }                
   double InlineDot(const Vector & a) const { return (x * a.x + y * a.y + z * a.z); }
   double InlineLength() const { double r = x * x + y * y + z * z; return ((r == 0) ? 0 : sqrt(r)); }
   double InLineLengthSquared() const { return (x * x + y * y + z * z); }
   void InlineNormalize() { double len = InlineLength(); (len == 0) ? (x = 0, y = 0, z = 1) : (x /= len, y /= len, z /= len); }
   double InlineDistance(const Vector & v) const { Vector a(x, y, z); a.InlineSub(v); return (a.InlineLength()); }
   double InlineDistanceSquared(const Vector & v) const { Vector a(x, y, z); a.InlineSub(v); return (a.InLineLengthSquared()); }
   void InlineCrossProduct(const Vector & a, const Vector & b)
      {
      x = a.y * b.z - b.y * a.z;
      y = -(a.x * b.z - b.x * a.z);
      z = a.x * b.y - b.x * a.y; 
      }   

}; 
