/* ********************************************************************
Description: 
Include header for AICON_Receiver.dll. Include this into your source and the
linker automatically includes the necessary library.

Author:
Marcel Ellerbrok

Revision:
* 09.11.2007 - ME - First stable version
* 21.09.2010 - ME - Added transfer of whole point sets
 **********************************************************************/
#ifndef __AICON_RECEIVER_DATA_MOVEINSPECT__
#define __AICON_RECEIVER_DATA_MOVEINSPECT__

#if _MSC_VER > 1000
	#pragma once
#endif

// ---------------------------------------------------------------------------
extern "C"
{

// Internally we use a structure packing of 8 bit.
#pragma pack( push, 8)

// defines the trigger signal
enum eMI_TriggerModes
{
	/*  With the constant frequency of an system-clock.
		The system clock can be implemented either inside the
		camera or as a software-loop.*/
	mov_ClockTrigger = 0,

	// by an external signal
	mov_ExternTrigger,

	// by an event from the software (e.g. button click)
	mov_SoftwareTrigger,
};

// defines how the cameras are activated
enum eMI_ActivationModes
{
	/*	The cameras are automatically activated directly
		with the start of the capture project */
	mov_ImmediateActivation = 0,

	/*	The cameras are activated with an external signal.*/
	mov_ExternActivation,
};

// different modes
enum eMI_SessionMode
{
	// store pictures to disk
	mov_Store = 0,

	// store and compute pictures
	mov_StoreComp,

	// only compute and display
	mov_Compute,

	// store and compute pictures and transfer data via realtime-server to clients
	mov_StoreRealOut,

	// compute pictures and transfer data via realtime-server to clients
	mov_RealOut,
};

// different modes
enum eMI_ProjectState
{
	mov_Prj_NotInitialized = 0,
	mov_Prj_Initialized,
	mov_Prj_Started,
	mov_Prj_Cancelled,
	mov_Prj_Stopped,
	mov_Prj_CalculationFinished
};

/* ********************************************************************
Description: 
A Unicode structure to contain information for MoveInspect project mode. 
Use this struct if your program runs in UNICODE mode. 

SeeAlso:     
tMIProjectParamA, tMIProjectParam

Remarks:     
To ensure your program to be compilable with both Unicode and ASCII you 
should use the macro tMIProjectParam that automatically chooses the right type.
******************************************************************** */
typedef struct cMIProjectParamU
{
	// constructor
	inline cMIProjectParamU()
	{
		sProjectName[0]  = L'\0';
		iSessionMode     = 0;
		iActivationMode  = 0;
		iTriggerMode     = 0;
		dCaptureInterval = 0.5;
	};

	wchar_t sProjectName[128];
	int     iSessionMode;
	int     iTriggerMode;
	int     iActivationMode;
	double  dCaptureInterval;
}tMIProjectParamU;

/* ********************************************************************
Description: 
A Unicode structure to contain information for MoveInspect project mode. 
Use this struct if your program runs in ASCII mode. 

SeeAlso:     
tMIProjectParamU, tMIProjectParam

Remarks:     
To ensure your program to be compilable with both Unicode and ASCII you 
should use the macro tMIProjectParam that automatically chooses the right type.
******************************************************************** */
typedef struct cMIProjectParamA
{
	// constructor
	inline cMIProjectParamA()
	{
		sProjectName[0]  = '\0';
		iSessionMode     = 0;
		iActivationMode  = 0;
		iTriggerMode     = 0;
		dCaptureInterval = 0.5;
	};

	char	sProjectName[128];
	int     iSessionMode;
	int     iTriggerMode;
	int     iActivationMode;
	double  dCaptureInterval;
}tMIProjectParamA;

#ifdef _UNICODE		
	/* ********************************************************************
	Description: 
	This macro lets your program automatically choose the right kind of 
	string handling. Either ASCII or Unicode.

	SeeAlso: tMIProjectParamU, tMIProjectParamA
	******************************************************************** */
	#define tMIProjectParam tMIProjectParamU
#else				
	/* ********************************************************************
	Description: 
	This macro lets your program automatically choose the right kind of 
	string handling. Either ASCII or Unicode.

	SeeAlso: tMIProjectParamU, tMIProjectParamA
	******************************************************************** */
	#define tMIProjectParam tMIProjectParamA
#endif

#pragma pack (pop)
}

// ***************************************************************************
#endif