/* ********************************************************************
Description: 
Include header for AICON_Receiver.dll. Include this into your source and the
linker automatically includes the necessary library.

Author:
Marcel Ellerbrok

Revision:
* 09.11.2007 - ME - First stable version
* 21.09.2010 - ME - Added transfer of whole point sets
 **********************************************************************/
#ifndef __AICON_RECEIVER_DLL_MOVEINSPECT__
#define __AICON_RECEIVER_DLL_MOVEINSPECT__

#if _MSC_VER > 1000
	#pragma once
#endif

// include the data types
#include "AICON_Receiver_Data_MoveInspect.h"

// ---------------------------------------------------------------------------
extern "C"
{

// Internally we use a structure packing of 8 bit.
#pragma pack( push, 8)


/* ********************************************************************
Description: 
Use this function to initialize a MoveInspect project. Use this function if your 
program runs in UNICODE mode.

Returns:
* rcSuccess        - pcCalibData contains valid data.
* rcNotInitialized - You never called gReceiver_Init
* rcFailed		   - There is no data available at the moment
******************************************************************** */
RECEIVER_API UINT gReceiver_MI_InitProjectU(UINT uMode, cMIProjectParamU* pProjectParam);

/* ********************************************************************
Description: 
Use this function to initialize a MoveInspect project. Use this function if your 
program runs in ASCII mode.

Returns:
* rcSuccess        - pcCalibData contains valid data.
* rcNotInitialized - You never called gReceiver_Init
* rcFailed		   - There is no data available at the moment
******************************************************************** */
RECEIVER_API UINT gReceiver_MI_InitProjectA(UINT uMode, cMIProjectParamA* pProjectParam);

#ifdef _UNICODE		
	/* ********************************************************************
	Description: 
	This macro lets your program automatically choose the right kind of 
	string handling. Either ASCII or Unicode.

	SeeAlso: gReceiver_MI_InitProjectU, gReceiver_MI_InitProjectA
	******************************************************************** */
	#define gReceiver_MI_InitProject gReceiver_MI_InitProjectU
#else				
	/* ********************************************************************
	Description: 
	This macro lets your program automatically choose the right kind of 
	string handling. Either ASCII or Unicode.

	SeeAlso: gReceiver_MI_InitProjectU, gReceiver_MI_InitProjectA
	******************************************************************** */
	#define gReceiver_MI_InitProject gReceiver_MI_InitProjectA
#endif

/* ********************************************************************
Description: 
Use this function to start a MoveInspect project. 

Returns:
* rcSuccess        - pcCalibData contains valid data.
* rcNotInitialized - You never called gReceiver_Init
* rcFailed		   - There is no data available at the moment
******************************************************************** */
RECEIVER_API UINT gReceiver_MI_StartProject();

/* ********************************************************************
Description: 
Use this function to stop a running MoveInspect project. 

Returns:
* rcSuccess        - pcCalibData contains valid data.
* rcNotInitialized - You never called gReceiver_Init
* rcFailed		   - There is no data available at the moment
******************************************************************** */
RECEIVER_API UINT gReceiver_MI_StopProject();

/* ********************************************************************
Description: 
Use this function to cancel a running MoveInspect project. 

Returns:
* rcSuccess        - pcCalibData contains valid data.
* rcNotInitialized - You never called gReceiver_Init
* rcFailed		   - There is no data available at the moment
******************************************************************** */
RECEIVER_API UINT gReceiver_MI_CancelProject();

/* ********************************************************************
Description: 
Get the state of a MoveInspect project

Returns:
* See: eMI_ProjectState.
******************************************************************** */
RECEIVER_API UINT gReceiver_MI_GetProjectState();

/* ********************************************************************
Description: 
Get the local data path of the MoveInspect project. Use this function if your 
program runs in UNICODE mode.

Returns:
* rcSuccess        - pcCalibData contains valid data.
* rcNotInitialized - You never called gReceiver_Init
* rcFailed		   - There is no data available at the moment
******************************************************************** */
RECEIVER_API UINT gReceiver_MI_GetProjectPathU(wchar_t* pPath, int iBufSize);

/* ********************************************************************
Description: 
Get the local data path of the MoveInspect project. Use this function if your 
program runs in ASCII mode.

Returns:
* rcSuccess        - pcCalibData contains valid data.
* rcNotInitialized - You never called gReceiver_Init
* rcFailed		   - There is no data available at the moment
******************************************************************** */
RECEIVER_API UINT gReceiver_MI_GetProjectPathA(char* pPath, int iBufSize);

#ifdef _UNICODE		
	/* ********************************************************************
	Description: 
	This macro lets your program automatically choose the right kind of 
	string handling. Either ASCII or Unicode.

	SeeAlso: gReceiver_MI_GetProjectPathU, gReceiver_MI_GetProjectPathA
	******************************************************************** */
	#define gReceiver_MI_GetProjectPath gReceiver_MI_GetProjectPathU
#else				
	/* ********************************************************************
	Description: 
	This macro lets your program automatically choose the right kind of 
	string handling. Either ASCII or Unicode.

	SeeAlso: gReceiver_MI_GetProjectPathU, gReceiver_MI_GetProjectPathA
	******************************************************************** */
	#define gReceiver_MI_GetProjectPath gReceiver_MI_GetProjectPathA
#endif

#pragma pack (pop)
}

// ***************************************************************************
#endif